@@ -1,8 +1,8 @@
-
 from sympy.core import (Basic, S, C, Add, Mul, Symbol, Equality, Interval,
     sympify, symbols, Wild)
 from sympy.functions import factorial
 from sympy.solvers import solve
+from sympy.utilities import flatten
 
 class Sum(Basic):
     """Represents unevaluated summation."""
@@ -37,6 +37,7 @@ class Sum(Basic):
 
                         continue
                 elif isinstance(V, (tuple, list)):
+                    V = flatten(V)
                     if len(V) == 1:
                         if isinstance(V[0], Symbol):
                             limits.append(V[0])
@@ -69,7 +70,11 @@ class Sum(Basic):
             f = eval_sum(f, (i, a, b))
             if f is None:
                 return self
-        return f
+
+        if hints.get('deep', True):
+            return f.doit(**hints)
+        else:
+            return f
 
     def _eval_summation(self, f, x):
         return
@@ -87,7 +92,8 @@ class Sum(Basic):
         and e is the estimated error (taken to be the magnitude of
         the first omitted term in the tail):
 
-            >>> k = Symbol('k')
+            >>> from sympy.abc import k, a, b
+            >>> from sympy import Sum
             >>> Sum(1/k, (k, 2, 5)).doit().evalf()
             1.28333333333333
             >>> s, e = Sum(1/k, (k, 2, 5)).euler_maclaurin()
@@ -99,7 +105,6 @@ class Sum(Basic):
 
         The endpoints may be symbolic:
 
-            >>> k, a, b = symbols('kab')
             >>> s, e = Sum(1/k, (k, a, b)).euler_maclaurin()
             >>> s
             -log(a) + log(b) + 1/(2*a) + 1/(2*b)
@@ -152,24 +157,26 @@ class Sum(Basic):
         return s + iterm, abs(term)
 
     def _eval_subs(self, old, new):
-        return Sum(self.args[0].subs(old, new), *self.args[1])
+        newargs = (self.args[1][0][0], self.args[1][0][1].subs(old,new),
+                   self.args[1][0][2].subs(old,new))
+        return Sum(self.args[0].subs(old, new), newargs)
 
 
 def sum(*args, **kwargs):
     summation = Sum(*args, **kwargs)
 
     if isinstance(summation, Sum):
-        return summation.doit()
+        return summation.doit(deep=False)
     else:
         return summation
 
 
 def getab(expr):
-    cls = expr.__class__
+    cls = expr.func
     return cls(expr.args[0]), cls(*expr.args[1:])
 
 def telescopic_direct(L, R, n, (i, a, b)):
-    '''Returns the direct summation of the terms of a telescopic sum
+    """Returns the direct summation of the terms of a telescopic sum
 
     L is the term with lower index
     R is the term with higher index
@@ -177,10 +184,12 @@ def telescopic_direct(L, R, n, (i, a, b)):
 
     For example:
 
-    >>> k,a,b = symbols('kab')
+    >>> from sympy.concrete.summations import telescopic_direct
+    >>> from sympy.abc import k, a, b
     >>> telescopic_direct(1/k, -1/(k+2), 2, (k, a, b))
     1/a + 1/(1 + a) - 1/(1 + b) - 1/(2 + b)
-    '''
+
+    """
     s = 0
     for m in xrange(n):
         s += L.subs(i,a+m) + R.subs(i,b-m)
@@ -198,7 +207,7 @@ def telescopic(L, R, (i, a, b)):
     #Maybe this should go inside solve
     k = Wild("k")
     sol = (-R).match(L.subs(i, i + k))
-    if sol and sol.has_key(k):
+    if sol and k in sol:
         if L.subs(i,i + sol[k]) == -R:
             #sometimes match fail(f(x+2).match(-f(x+k))->{k: -2 - 2x}))
             s = sol[k]
@@ -207,7 +216,7 @@ def telescopic(L, R, (i, a, b)):
         m = Symbol("m")
         try:
             s = solve(L.subs(i, i + m) + R, m)[0]
-        except ValueError:
+        except IndexError:#(ValueError, IndexError):
             pass
     if s and s.is_Integer:
         if s < 0:
