@@ -1,5 +1,5 @@
-
-from sympy import *
+from sympy import (symbols, product, factorial, rf, sqrt, cos,
+                   Function, Product, Rational)
 
 a, k, n = symbols('akn', integer=True)
 
@@ -28,3 +28,11 @@ def test_special_products():
     # Euler's product formula for sin
     assert product(1 + a/k**2, (k, 1, n)) == \
         rf(1 - sqrt(-a), n)*rf(1 + sqrt(-a), n)/factorial(n)**2
+
+def test__eval_product():
+    from sympy.abc import i, n
+    # 1710
+    a = Function('a')
+    assert product(2*a(i), (i, 1, n)) == 2**n * Product(a(i), (i, 1, n))
+    # 1711
+    assert product(2**i, (i, 1, n)) == 2**(n/2 + n**2/2)
