@@ -3,21 +3,21 @@ Adaptive numerical evaluation of SymPy expressions, using mpmath
 for mathematical functions.
 """
 
-from sympy.mpmath.libmpf import (from_int, from_rational, fzero, normalize,
+from sympy.mpmath.libmp import (from_int, from_rational, fzero, normalize,
         bitcount, round_nearest, to_str, fone, fnone, fhalf, from_float,
         to_float, fnone, to_int, mpf_lt, mpf_sqrt, mpf_cmp, mpf_abs,
         mpf_pow_int, mpf_shift, mpf_add, mpf_mul, mpf_neg)
 
-import sympy.mpmath.libmpc as libmpc
-from sympy.mpmath.settings import dps_to_prec
+import sympy.mpmath.libmp as libmp
+from sympy.mpmath.libmp.libmpf import dps_to_prec
 from sympy.mpmath import mpf, mpc, quadts, quadosc, mp, make_mpf
-from sympy.mpmath.libelefun import mpf_pi, mpf_log, mpf_pow, mpf_sin, mpf_cos, \
-        mpf_atan, mpf_atan2, mpf_e, mpf_exp
-from sympy.mpmath.libmpf import MP_BASE, from_man_exp
-from sympy.mpmath.calculus import nsum
+from sympy.mpmath.libmp import (mpf_pi, mpf_log, mpf_pow, mpf_sin, mpf_cos,
+        mpf_atan, mpf_atan2, mpf_e, mpf_exp, from_man_exp)
+from sympy.mpmath.libmp.backend import MPZ
+from sympy.mpmath import nsum
 from sympy.mpmath import inf as mpmath_inf
 
-from sympy.mpmath.gammazeta import mpf_bernoulli
+from sympy.mpmath.libmp.gammazeta import mpf_bernoulli
 
 import math
 
@@ -123,7 +123,7 @@ def get_abs(expr, prec, options):
     if not re:
         re, re_acc, im, im_acc = im, im_acc, re, re_acc
     if im:
-        return libmpc.mpc_abs((re, im), prec), None, re_acc, None
+        return libmp.mpc_abs((re, im), prec), None, re_acc, None
     else:
         return mpf_abs(re), None, re_acc, None
 
@@ -356,7 +356,7 @@ def evalf_mul(v, prec, options):
     prec = prec + len(args) + 5
     direction = 0
     # Empty product is 1
-    man, exp, bc = MP_BASE(1), 0, 1
+    man, exp, bc = MPZ(1), 0, 1
     direction = 0
     complex_factors = []
     # First, we multiply all pure real or pure imaginary numbers.
@@ -387,7 +387,7 @@ def evalf_mul(v, prec, options):
     if complex_factors:
         # make existing real scalar look like an imaginary and
         # multiply by the remaining complex numbers
-        re, im = v, (0, MP_BASE(0), 0, 0)
+        re, im = v, (0, MPZ(0), 0, 0)
         for wre, wim, wre_acc, wim_acc in complex_factors:
             # acc is the overall accuracy of the product; we aren't
             # computing exact accuracies of the product.
@@ -446,7 +446,7 @@ def evalf_pow(v, prec, options):
         if not re:
             return None, None, None, None
         # General complex number to arbitrary integer power
-        re, im = libmpc.mpc_pow_int((re, im), p, prec)
+        re, im = libmp.mpc_pow_int((re, im), p, prec)
         # Assumes full accuracy in input
         return finalize_complex(re, im, target_prec)
 
@@ -455,7 +455,7 @@ def evalf_pow(v, prec, options):
         xre, xim, xre_acc, yim_acc = evalf(base, prec+5, options)
         # General complex square root
         if xim:
-            re, im = libmpc.mpc_sqrt((xre or fzero, xim), prec)
+            re, im = libmp.mpc_sqrt((xre or fzero, xim), prec)
             return finalize_complex(re, im, prec)
         if not xre:
             return None, None, None, None
@@ -483,7 +483,7 @@ def evalf_pow(v, prec, options):
     # Pure exponential function; no need to evalf the base
     if base is S.Exp1:
         if yim:
-            re, im = libmpc.mpc_exp((yre or fzero, yim), prec)
+            re, im = libmp.mpc_exp((yre or fzero, yim), prec)
             return finalize_complex(re, im, target_prec)
         return mpf_exp(yre, target_prec), None, target_prec, None
 
@@ -494,16 +494,16 @@ def evalf_pow(v, prec, options):
 
     # (real ** complex) or (complex ** complex)
     if yim:
-        re, im = libmpc.mpc_pow((xre or fzero, xim or fzero), (yre or fzero, yim),
+        re, im = libmp.mpc_pow((xre or fzero, xim or fzero), (yre or fzero, yim),
             target_prec)
         return finalize_complex(re, im, target_prec)
     # complex ** real
     if xim:
-        re, im = libmpc.mpc_pow_mpf((xre or fzero, xim), yre, target_prec)
+        re, im = libmp.mpc_pow_mpf((xre or fzero, xim), yre, target_prec)
         return finalize_complex(re, im, target_prec)
     # negative ** real
     elif mpf_lt(xre, fzero):
-        re, im = libmpc.mpc_pow_mpf((xre, fzero), yre, target_prec)
+        re, im = libmp.mpc_pow_mpf((xre, fzero), yre, target_prec)
         return finalize_complex(re, im, target_prec)
     # positive ** real
     else:
@@ -824,14 +824,14 @@ def hypsum(expr, n, start, prec):
 
     # Direct summation if geometric or faster
     if h > 0 or (h == 0 and abs(g) > 1):
-        one = MP_BASE(1) << prec
+        one = MPZ(1) << prec
         term = expr.subs(n, 0)
-        term = (MP_BASE(term.p) << prec) // term.q
+        term = (MPZ(term.p) << prec) // term.q
         s = term
         k = 1
         while abs(term) > 5:
-            term *= MP_BASE(func1(k-1))
-            term //= MP_BASE(func2(k-1))
+            term *= MPZ(func1(k-1))
+            term //= MPZ(func2(k-1))
             s += term
             k += 1
         return from_man_exp(s, -prec)
@@ -845,15 +845,15 @@ def hypsum(expr, n, start, prec):
         # Need to use at least quad precision because a lot of cancellation
         # might occur in the extrapolation process
         prec2 = 4*prec
-        one = MP_BASE(1) << prec2
+        one = MPZ(1) << prec2
         term = expr.subs(n, 0)
-        term = (MP_BASE(term.p) << prec2) // term.q
+        term = (MPZ(term.p) << prec2) // term.q
 
         def summand(k, _term=[term]):
             if k:
                 k = int(k)
-                _term[0] *= MP_BASE(func1(k-1))
-                _term[0] //= MP_BASE(func2(k-1))
+                _term[0] *= MPZ(func1(k-1))
+                _term[0] //= MPZ(func2(k-1))
             return make_mpf(from_man_exp(_term[0], -prec2))
 
         orig = mp.prec
