@@ -1,17 +1,14 @@
-
-from basic import Basic, C
+from basic import Basic
 from sympify import _sympify
 
-from numbers import Number
-
 def Rel(a, b, op):
     """
     A handy wrapper around the Relational class.
     Rel(a,b, op)
 
     Example:
-    >>> from sympy import *
-    >>> x,y = symbols('xy')
+    >>> from sympy import Rel
+    >>> from sympy.abc import x, y
     >>> Rel(y, x+x**2, '==')
     y == x + x**2
 
@@ -24,8 +21,8 @@ def Eq(a, b=0):
     Eq(a,b)
 
     Example:
-    >>> from sympy import *
-    >>> x,y = symbols('xy')
+    >>> from sympy import Eq
+    >>> from sympy.abc import x, y
     >>> Eq(y, x+x**2)
     y == x + x**2
 
@@ -38,8 +35,8 @@ def Ne(a, b):
     Ne(a,b)
 
     Example:
-    >>> from sympy import *
-    >>> x,y = symbols('xy')
+    >>> from sympy import Ne
+    >>> from sympy.abc import x, y
     >>> Ne(y, x+x**2)
     y != x + x**2
 
@@ -52,8 +49,8 @@ def Lt(a, b):
     Lt(a,b)
 
     Example:
-    >>> from sympy import *
-    >>> x,y = symbols('xy')
+    >>> from sympy import Lt
+    >>> from sympy.abc import x, y
     >>> Lt(y, x+x**2)
     y < x + x**2
 
@@ -66,8 +63,8 @@ def Le(a, b):
     Le(a,b)
 
     Example:
-    >>> from sympy import *
-    >>> x,y = symbols('xy')
+    >>> from sympy import Le
+    >>> from sympy.abc import x, y
     >>> Le(y, x+x**2)
     y <= x + x**2
 
@@ -80,8 +77,8 @@ def Gt(a, b):
     Gt(a,b)
 
     Example:
-    >>> from sympy import *
-    >>> x,y = symbols('xy')
+    >>> from sympy import Gt
+    >>> from sympy.abc import x, y
     >>> Gt(y, x+x**2)
     x + x**2 < y
 
@@ -94,8 +91,8 @@ def Ge(a, b):
     Ge(a,b)
 
     Example:
-    >>> from sympy import *
-    >>> x,y = symbols('xy')
+    >>> from sympy import Ge
+    >>> from sympy.abc import x, y
     >>> Ge(y, x+x**2)
     x + x**2 <= y
 
@@ -181,12 +178,3 @@ class Inequality(Relational):
                 return self.lhs <= self.rhs
             return self.lhs.evalf()<=self.rhs.evalf()
         return self.lhs.compare(self.rhs)<=0
-
-
-# /cyclic/
-import basic as _
-_.Equality      = Equality
-_.Unequality    = Unequality
-_.Inequality    = Inequality
-_.StrictInequality = StrictInequality
-del _
