@@ -1,9 +1,7 @@
 
-from basic import Basic, Atom, S, C, sympify
+from basic import Atom
 from cache import cacheit
 
-# from function import Function, WildFunction   /cyclic/
-
 import re
 
 class Symbol(Atom):
@@ -13,11 +11,13 @@ class Symbol(Atom):
        commutative = True
 
     You can override the default assumptions in the constructor::
+       >>> from sympy import symbols
        >>> A,B = symbols('AB', commutative = False)
        >>> bool(A*B != B*A)
        True
        >>> bool(A*B*2 == 2*A*B) == True # multiplication by scalars is commutative
        True
+
     """
 
     is_comparable = False
@@ -30,6 +30,7 @@ class Symbol(Atom):
                 **assumptions):
         """if dummy == True, then this Symbol is totally unique, i.e.::
 
+        >>> from sympy import Symbol
         >>> bool(Symbol("x") == Symbol("x")) == True
         True
 
@@ -68,8 +69,8 @@ class Symbol(Atom):
     def __call__(self, *args):
         return Function(self.name, nargs=len(args))(*args, **self.assumptions0)
 
-    def _eval_expand_complex(self, *args):
-        return C.re(self) + C.im(self)*S.ImaginaryUnit
+    def _eval_expand_complex(self, deep=True, **hints):
+            return C.re(self) + C.im(self)*S.ImaginaryUnit
 
     def _sage_(self):
         import sage.all as sage
@@ -84,10 +85,12 @@ class Dummy(Symbol):
 
        use this through Symbol:
 
+       >>> from sympy import Symbol
        >>> x1 = Symbol('x', dummy=True)
        >>> x2 = Symbol('x', dummy=True)
        >>> bool(x1 == x2)
        False
+
     """
 
     dummycount = 0
@@ -157,23 +160,22 @@ class Wild(Symbol):
         return (self.name, self.exclude, self.properties )
 
     # TODO add check against another Wild
-    def matches(pattern, expr, repl_dict={}, evaluate=False):
-        for p,v in repl_dict.items():
-            if p==pattern:
-                if v==expr: return repl_dict
+    def matches(self, expr, repl_dict={}, evaluate=False):
+        if self in repl_dict:
+            if repl_dict[self] == expr:
+                return repl_dict
+            else:
                 return None
-        if pattern.exclude:
-            for x in pattern.exclude:
+        if self.exclude:
+            for x in self.exclude:
                 if x in expr:
                     return None
-                #else:
-                #    print x, expr, pattern, expr, pattern.exclude
-        if pattern.properties:
-            for f in pattern.properties:
+        if self.properties:
+            for f in self.properties:
                 if not f(expr):
                     return None
         repl_dict = repl_dict.copy()
-        repl_dict[pattern] = expr
+        repl_dict[self] = expr
         return repl_dict
 
     def __call__(self, *args, **assumptions):
@@ -186,11 +188,12 @@ def symbols(*names, **kwargs):
     argument, which can be a string, then each character
     forms a separate symbol, or a sequence of strings.
 
+    >>> from sympy import symbols
     >>> x, y, z = symbols('xyz')
 
     Please note that this syntax is deprecated and will be dropped in a
     future version of sympy. Use comma or whitespace separated characters
-    instead. Currently the old behavoiur is standard, this can be changed
+    instead. Currently the old behavior is standard, this can be changed
     using the 'each_char' keyword:
 
     >>> symbols('xyz', each_char=False)
@@ -206,6 +209,7 @@ def symbols(*names, **kwargs):
     >>> xx, yy, zz = symbols('xx', 'yy', 'zz', real=True)
     >>> xx.is_real and yy.is_real and zz.is_real
     True
+
     """
     # use new behavior if space or comma in string
     if not 'each_char' in kwargs and len(names) == 1 and \
@@ -253,6 +257,7 @@ def var(*names, **kwargs):
     the parent's *global* namespace.  It's recommended not to use "var" in
     library code, it is better to use symbols() instead.
 
+    >>> from sympy import var
     >>> var('m')
     m
     >>> var('n xx yy zz')
@@ -263,6 +268,7 @@ def var(*names, **kwargs):
     (x, y)
     >>> x.is_real and y.is_real
     True
+
     """
     import inspect
     frame = inspect.currentframe().f_back
@@ -283,9 +289,6 @@ def var(*names, **kwargs):
         # doc
         del frame
 
-# /cyclic/
-import basic as _
-_.Symbol    = Symbol
-_.Wild      = Wild
-_.Temporary = Temporary
-del _
+from basic import Basic, Atom, S, C
+from sympify import sympify
+from function import Function
