@@ -16,8 +16,8 @@ class SympifyError(ValueError):
 
 def sympify(a, locals=None, convert_xor=True):
     """Converts an arbitrary expression to a type that can be used
-       inside sympy. For example, it will convert python int's into
-       instance of sympy.Rational, floats into intances of sympy.Real,
+       inside sympy. For example, it will convert python ints into
+       instance of sympy.Rational, floats into instances of sympy.Real,
        etc. It is also able to coerce symbolic expressions which does
        inherit after Basic. This can be useful in cooperation with SAGE.
 
@@ -25,12 +25,13 @@ def sympify(a, locals=None, convert_xor=True):
            - any object defined in sympy (except maybe matrices [TODO])
            - standard numeric python types: int, long, float, Decimal
            - strings (like "0.09" or "2e-19")
+           - booleans, including None (will leave them unchanged)
 
        If the argument is already a type that sympy understands, it will do
-       nothing but return that value. This can be used at the begining of a
+       nothing but return that value. This can be used at the beginning of a
        function to ensure you are working with the correct type.
 
-       >>> from sympy import *
+       >>> from sympy import sympify
 
        >>> sympify(2).is_integer
        True
@@ -53,9 +54,9 @@ def sympify(a, locals=None, convert_xor=True):
     #                                      -- kirr
     if locals is None:
         locals = {}
-    if isinstance(a, Basic):
+    if isinstance(a, (Basic, BasicType, bool)):
         return a
-    if isinstance(a, BasicType):
+    if a is None:
         return a
     elif isinstance(a, (int, long)):
         return Integer(a)
@@ -63,13 +64,7 @@ def sympify(a, locals=None, convert_xor=True):
         return Real(a)
     elif isinstance(a, complex):
         real, imag = map(sympify, (a.real, a.imag))
-        ireal, iimag = int(real), int(imag)
-
-        if ireal + iimag*1j == a:
-            return ireal + iimag*S.ImaginaryUnit
         return real + S.ImaginaryUnit * imag
-    elif isinstance(a, bool):
-        raise NotImplementedError("bool support")
     elif isinstance(a, (list,tuple,set)):
         return type(a)([sympify(x) for x in a])
 
@@ -123,15 +118,12 @@ def sympify(a, locals=None, convert_xor=True):
 
 
 def _sympify(a):
-    """short version of sympify for internal usage
-
-       When adding and comparing symbolic expressions, it is unwise to allow
-       e.g. strings to mixin. On the other hand Python integers and floats are
-       allowed.
-
-       So we don't use full-featured sympify in __add__ and __eq__ methods, but
-       instead use this small-crafted function there instead.
+    """Short version of sympify for internal usage for __add__ and __eq__
+       methods where it is ok to allow some things (like Python integers
+       and floats) in the expression. This excludes things (like strings)
+       that are unwise to allow into such an expression.
 
+       >>> from sympy import Integer
        >>> Integer(1) == 1
        True
 
@@ -139,7 +131,7 @@ def _sympify(a):
        False
 
        >>> from sympy import Symbol
-       >>> x = Symbol('x')
+       >>> from sympy.abc import x
        >>> x + 1
        1 + x
 
@@ -193,104 +185,8 @@ def _sympify(a):
     raise SympifyError("%r is NOT a valid SymPy expression" % (a,))
 
 
-def __sympifyit(func, arg, retval=None):
-    """decorator to _sympify `arg` argument for function `func`
-
-       don't use directly -- use _sympifyit instead
-    """
-
-    # we support f(a,b) only
-    assert func.func_code.co_argcount
-    # only b is _sympified
-    assert func.func_code.co_varnames[1] == arg
-
-    def __sympifyit_wrapper(a, b):
-        # our task is to call `func` with `b` _sympified.
-        #
-        # if we _sympify from the beginning, we'll get unneccesary overhead,
-        # because _sympify has far non-zero cost even for Basic instances.
-        #
-        # the idea is to first run `func` with `b` as is, catch any error, and
-        # try to rerun with b _sympified.
-        #
-        # so for Basic instances we'll get almost no overhead, and for other
-        # objects we agree to take additional overhead because `func` has to be
-        # run first, and only when it raises we can proceed with the second
-        # phase.
-        #
-        # however there is one important exception -- python ints.
-        # ints are used heavily, e.g. in sum([x**i for i in range(n)]) and
-        # other places, so it is important to sympify ints as fast as possible
-        # too.
 
 
-        # python ints are used frequently -- it is important to convert them as
-        # fast as possible
-        #
-        # %timeit type(1) is int            ->  1.43 us
-        # %timeit type('abc') is int        ->  1.48 us
-        # %timeit isinstance(1, int)        ->  1.29 us
-        # %timeit isinstance('abc', int)    ->  2.23 us
-        # %timeit isinstance(x, int)        ->  4.28 us
-        # z = S.Half
-        # %timeit isinstance(z, int)        ->  5.2 us
-        #
-        # so we use:
-        if type(b) is int:
-            b = Integer(b)
-
-        try:
-            # fast-path: let's hope b is already SymPy object
-            return func(a, b)
-
-        except Exception, e:
-
-            # we've got an exception.
-            # maybe it's from nested __sympifyit? then we have to quit.
-            if isinstance(e, SympifyError):
-                #print 'double deep sympify'
-                if retval is not None:
-                    return retval
-                else:
-                    raise
-
-            # slow-path: b seems to be not SymPy object -- let's _sympify it
-            try:
-                b = _sympify(b)
-                #print 'deep sympify'
-            except SympifyError:
-                # sympify failed, let's return requested value
-                if retval is not None:
-                    return retval
-                else:
-                    # or pass exception through
-                    raise
-
-            # b successfully _sympified, lets call func again.
-            # if it raises here -- exception goes to caller
-            return func(a, b)
-
-    return __sympifyit_wrapper
-
-
-def _sympifyit(arg, retval=None):
-    """decorator to smartly _sympify function arguments
-
-       @_sympifyit('other', NotImplemented)
-       def add(self, other):
-           ...
-
-       In add, other can be though as being SymPy object already.
-
-       If it is not, the code is likely to catch an exception, then other will
-       be explicitly _sympified, and the whole code restarted.
-
-       if _sympify(arg) fails, NotImplemented will be returned
-
-       see: __sympifyit
-    """
-    def deco(func):
-        return __sympifyit(func, arg, retval)
-
-    return deco
 
+from numbers import Integer, Real
+from basic import Basic, BasicType, S
