@@ -1,9 +1,10 @@
 from sympy import Symbol, sin, cos, exp, O, sqrt, Rational, Real, re, pi, \
-        sympify, sqrt, Add, Mul, Pow, I
+        sympify, sqrt, Add, Mul, Pow, I, log
 from sympy.utilities.pytest import XFAIL
 
-x = Symbol("x")
+x = Symbol('x')
 y = Symbol('y')
+z = Symbol('z')
 
 def test_bug1():
     assert re(x) != x
@@ -90,10 +91,6 @@ def test_pow():
     assert e == a**9
     e=a*a*a*a**Rational(6)-a**Rational(9)
     assert e == Rational(0)
-    e=a**(b+c)*a**(-b)
-    assert e == a**c
-    e=a**(b+c)*a*a**(-b)*a**(-c)/a
-    assert e == Rational(1)
     e=a**(b-b)
     assert e == Rational(1)
     e=(a-a)**b
@@ -137,6 +134,10 @@ def test_pow():
     assert (x**5*(3*x)**(-3)).expand() == Rational(1,27) * x**2
     assert (x**5*(-3*x)**(-3)).expand() == -Rational(1,27) * x**2
 
+    # expand_power_exp
+    assert (x**(y**(x+exp(x+y))+z)).expand(deep=False) == x**z*x**(y**(x + exp(x + y)))
+    assert (x**(y**(x+exp(x+y))+z)).expand() == x**z*x**(y**x*y**(exp(x)*exp(y)))
+
     n = Symbol('k', even=False)
     k = Symbol('k', even=True)
 
@@ -186,6 +187,12 @@ def test_expand():
     e=s.series(x,0,3)/x**2
     assert e.expand() ==  1+x**2/2+O(x**4)
 
+    e = (x*(y+z))**(x*(y+z))*(x+y)
+    assert e.expand(power_exp=False, power_base=False) == x*(x*y + x*z)**(x*y + x*z) + y*(x*y + x*z)**(x*y + x*z)
+    assert e.expand(power_exp=False, power_base=False, deep=False) == x*(x*(y + z))**(x*(y + z)) + y*(x*(y + z))**(x*(y + z))
+    e = (x*(y+z))**z
+    assert e.expand(power_base=True, mul=True, deep=True) in [x**z*(y + z)**z, (x*y + x*z)**z]
+
     # Check that this isn't too slow
     x = Symbol('x')
     W = 1
@@ -259,6 +266,19 @@ def test_powerbug():
 
     assert (2*x)**2 == (-2*x)**2
 
+def test_Mul_doesnt_expand_exp():
+    x = Symbol('x')
+    y = Symbol('y')
+    assert exp(x)*exp(y) == exp(x)*exp(y)
+    assert 2**x*2**y == 2**x*2**y
+    assert x**2*x**3 == x**5
+    assert 2**x*3**x == 6**x
+    assert x**(y)*x**(2*y) == x**(3*y)
+    assert 2**Rational(1,2)*2**Rational(1,2) == 2
+    assert 2**x*2**(2*x) == 2**(3*x)
+    assert 2**Rational(1,2)*2**Rational(1,4)*5**Rational(3,4) == 10**Rational(3,4)
+    assert (x**(-log(5)/log(3))*x)/(x*x**( - log(5)/log(3))) == sympify(1)
+
 def test_Add_Mul_is_integer():
     x = Symbol('x')
 
@@ -1047,7 +1067,9 @@ def test_Add_as_coeff_terms():
     assert (n-2).as_coeff_terms()   == (-1, (2-n,) )
     assert (n-3).as_coeff_terms()   == (-1, (3-n,) )
 
+def test_Pow_as_coeff_terms_doesnt_expand():
+    assert exp(x + y).as_coeff_terms() == (1, (exp(x + y),))
+    assert exp(x + exp(x + y)) != exp(x + exp(x)*exp(y))
 
 def test_issue974():
     assert -1/(-1-x)    == 1/(1+x)
-
