@@ -1,6 +1,6 @@
-import py
 from sympy.core import Symbol, S, Rational, Integer
-from sympy.utilities.pytest import XFAIL
+from sympy.utilities.pytest import XFAIL, raises
+from sympy import I, sqrt
 
 def test_symbol_unset():
     x = Symbol('x',real=True, integer=True)
@@ -8,9 +8,6 @@ def test_symbol_unset():
     assert x.is_integer == True
     assert x.is_imaginary == False
     assert x.is_noninteger == False
-    x.assume(real=None)
-    assert x.is_real == None
-    assert x.is_integer == None
 
 def test_zero():
     z = Integer(0)
@@ -22,7 +19,6 @@ def test_zero():
     assert z.is_noninteger == False
     assert z.is_irrational == False
     assert z.is_imaginary == False
-    assert z.is_noncomplex == False
     assert z.is_positive == False
     assert z.is_negative == False
     assert z.is_nonpositive == True
@@ -35,7 +31,7 @@ def test_zero():
     assert z.is_infinitesimal == True
     assert z.is_comparable == True
     assert z.is_prime == False
-    assert z.is_composite == True
+    assert z.is_composite == False
 
 def test_one():
     z = Integer(1)
@@ -47,7 +43,6 @@ def test_one():
     assert z.is_noninteger == False
     assert z.is_irrational == False
     assert z.is_imaginary == False
-    assert z.is_noncomplex == False
     assert z.is_positive == True
     assert z.is_negative == False
     assert z.is_nonpositive == False
@@ -72,7 +67,6 @@ def test_negativeone():
     assert z.is_noninteger == False
     assert z.is_irrational == False
     assert z.is_imaginary == False
-    assert z.is_noncomplex == False
     assert z.is_positive == False
     assert z.is_negative == True
     assert z.is_nonpositive == True
@@ -85,7 +79,82 @@ def test_negativeone():
     assert z.is_infinitesimal == False
     assert z.is_comparable == True
     assert z.is_prime == False
-    assert z.is_composite == True
+    assert z.is_composite == False
+
+def test_infinity():
+    oo = S.Infinity
+
+    assert oo.is_commutative    == True
+    assert oo.is_integer        == None
+    assert oo.is_rational       == None
+    assert oo.is_real           == True
+    assert oo.is_complex        == True
+    assert oo.is_noninteger     == None
+    assert oo.is_irrational     == None
+    assert oo.is_imaginary      == False
+    assert oo.is_positive       == True
+    assert oo.is_negative       == False
+    assert oo.is_nonpositive    == False
+    assert oo.is_nonnegative    == True
+    assert oo.is_even           == None
+    assert oo.is_odd            == None
+    assert oo.is_bounded        == False
+    assert oo.is_unbounded      == True
+    assert oo.is_finite         == False
+    assert oo.is_infinitesimal  == False
+    assert oo.is_comparable     == True
+    assert oo.is_prime          == None
+    assert oo.is_composite      == None
+
+def test_neg_infinity():
+    mm = S.NegativeInfinity
+
+    assert mm.is_commutative    == True
+    assert mm.is_integer        == None
+    assert mm.is_rational       == None
+    assert mm.is_real           == True
+    assert mm.is_complex        == True
+    assert mm.is_noninteger     == None
+    assert mm.is_irrational     == None
+    assert mm.is_imaginary      == False
+    assert mm.is_positive       == False
+    assert mm.is_negative       == True
+    assert mm.is_nonpositive    == True
+    assert mm.is_nonnegative    == False
+    assert mm.is_even           == None
+    assert mm.is_odd            == None
+    assert mm.is_bounded        == False
+    assert mm.is_unbounded      == True
+    assert mm.is_finite         == False
+    assert mm.is_infinitesimal  == False
+    assert mm.is_comparable     == True
+    assert mm.is_prime          == False
+    assert mm.is_composite      == False
+
+def test_nan():
+    nan = S.NaN
+
+    assert nan.is_commutative   == True
+    assert nan.is_integer       == None
+    assert nan.is_rational      == None
+    assert nan.is_real          == None
+    assert nan.is_complex       == None
+    assert nan.is_noninteger    == None
+    assert nan.is_irrational    == None
+    assert nan.is_imaginary     == None
+    assert nan.is_positive      == None
+    assert nan.is_negative      == None
+    assert nan.is_nonpositive   == None
+    assert nan.is_nonnegative   == None
+    assert nan.is_even          == None
+    assert nan.is_odd           == None
+    assert nan.is_bounded       == None
+    assert nan.is_unbounded     == None
+    assert nan.is_finite        == None
+    assert nan.is_infinitesimal == None
+    assert nan.is_comparable    == False
+    assert nan.is_prime         == None
+    assert nan.is_composite     == None
 
 def test_pos_rational():
     r = Rational(3,4)
@@ -97,20 +166,19 @@ def test_pos_rational():
     assert r.is_noninteger == True
     assert r.is_irrational == False
     assert r.is_imaginary == False
-    assert r.is_noncomplex == False
     assert r.is_positive == True
     assert r.is_negative == False
     assert r.is_nonpositive == False
     assert r.is_nonnegative == True
-    assert r.is_even == None
-    assert r.is_odd == None
+    assert r.is_even == False
+    assert r.is_odd == False
     assert r.is_bounded == True
     assert r.is_unbounded == False
     assert r.is_finite == True
     assert r.is_infinitesimal == False
     assert r.is_comparable == True
-    assert r.is_prime == None
-    assert r.is_composite == None
+    assert r.is_prime == False
+    assert r.is_composite == False
 
     r = Rational(1,4)
     assert r.is_nonpositive == False
@@ -153,77 +221,74 @@ def test_neg_rational():
 def test_pi():
     z = S.Pi
     assert z.is_commutative == True
-    assert z.is_integer == None
+    assert z.is_integer == False
     assert z.is_rational == False
     assert z.is_real == True
     assert z.is_complex == True
-    assert z.is_noninteger == None
+    assert z.is_noninteger == True
     assert z.is_irrational == True
     assert z.is_imaginary == False
-    assert z.is_noncomplex == False
     assert z.is_positive == True
     assert z.is_negative == False
     assert z.is_nonpositive == False
     assert z.is_nonnegative == True
-    assert z.is_even == None
-    assert z.is_odd == None
+    assert z.is_even == False
+    assert z.is_odd == False
     assert z.is_bounded == True
     assert z.is_unbounded == False
     assert z.is_finite == True
     assert z.is_infinitesimal == False
     assert z.is_comparable == True
-    assert z.is_prime == None
-    assert z.is_composite == None
+    assert z.is_prime == False
+    assert z.is_composite == False
 
 def test_E():
     z = S.Exp1
     assert z.is_commutative == True
-    assert z.is_integer == None
+    assert z.is_integer == False
     assert z.is_rational == False
     assert z.is_real == True
     assert z.is_complex == True
-    assert z.is_noninteger == None
+    assert z.is_noninteger == True
     assert z.is_irrational == True
     assert z.is_imaginary == False
-    assert z.is_noncomplex == False
     assert z.is_positive == True
     assert z.is_negative == False
     assert z.is_nonpositive == False
     assert z.is_nonnegative == True
-    assert z.is_even == None
-    assert z.is_odd == None
+    assert z.is_even == False
+    assert z.is_odd == False
     assert z.is_bounded == True
     assert z.is_unbounded == False
     assert z.is_finite == True
     assert z.is_infinitesimal == False
     assert z.is_comparable == True
-    assert z.is_prime == None
-    assert z.is_composite == None
+    assert z.is_prime == False
+    assert z.is_composite == False
 
 def test_I():
     z = S.ImaginaryUnit
     assert z.is_commutative == True
-    assert z.is_integer == None
-    assert z.is_rational == None
+    assert z.is_integer == False
+    assert z.is_rational == False
     assert z.is_real == False
     assert z.is_complex == True
-    assert z.is_noninteger == None
-    assert z.is_irrational == None
+    assert z.is_noninteger == False
+    assert z.is_irrational == False
     assert z.is_imaginary == True
-    assert z.is_noncomplex == False
-    assert z.is_positive == None
-    assert z.is_negative == None
-    assert z.is_nonpositive == None
-    assert z.is_nonnegative == None
-    assert z.is_even == None
-    assert z.is_odd == None
+    assert z.is_positive == False
+    assert z.is_negative == False
+    assert z.is_nonpositive == False
+    assert z.is_nonnegative == False
+    assert z.is_even == False
+    assert z.is_odd == False
     assert z.is_bounded == True
     assert z.is_unbounded == False
     assert z.is_finite == True
     assert z.is_infinitesimal == False
     assert z.is_comparable == None
-    assert z.is_prime == None
-    assert z.is_composite == None
+    assert z.is_prime == False
+    assert z.is_composite == False
 
 def test_symbol_zero():
     x = Symbol('x',zero=True)
@@ -250,7 +315,6 @@ def test_neg_symbol_positive():
     assert x.is_negative == True
     assert x.is_nonnegative == False
 
-@XFAIL
 def test_neg_symbol_positive2():
     x = -Symbol('x',positive=True)
     assert x.is_zero == False
@@ -297,14 +361,16 @@ def test_other_symbol():
     assert x.is_real == True
 
     x = Symbol('x', integer=True, nonnegative=True)
-    assert x.is_nni == True
-    assert x.is_ni == False
-    assert x.is_pi == None
+    assert x.is_integer     == True
+    assert x.is_nonnegative == True
+    assert x.is_negative    == False
+    assert x.is_positive    == None
 
     x = Symbol('x', integer=True, nonpositive=True)
-    assert x.is_npi == True
-    assert x.is_pi == False
-    assert x.is_ni == None
+    assert x.is_integer     == True
+    assert x.is_nonpositive == True
+    assert x.is_positive    == False
+    assert x.is_negative    == None
 
     x = Symbol('x', odd=True)
     assert x.is_odd == True
@@ -316,18 +382,17 @@ def test_other_symbol():
     assert x.is_odd == False
     assert x.is_integer == True
 
-    x = Symbol('x', nni=True)
+    x = Symbol('x', integer=True, nonnegative=True)
     assert x.is_integer == True
     assert x.is_nonnegative == True
 
-    x = Symbol('x', npi=True)
+    x = Symbol('x', integer=True, nonpositive=True)
     assert x.is_integer == True
     assert x.is_nonpositive == True
 
-    py.test.raises(AttributeError, "x.is_real = False")
+    raises(AttributeError, "x.is_real = False")
 
 
-@XFAIL
 def test_other_symbol_fail1():
     # XXX x.is_even currently will be True
     x = Symbol('x', odd=False)
@@ -335,7 +400,6 @@ def test_other_symbol_fail1():
     assert x.is_even == None
     assert x.is_integer == None
 
-@XFAIL
 def test_other_symbol_fail2():
     # XXX x.is_odd currently will be True
     x = Symbol('x', even=False)
@@ -344,7 +408,20 @@ def test_other_symbol_fail2():
     assert x.is_integer == None
 
 
+def test_issue749():
+    a = Symbol('a', real=False)
+
+    assert a.is_real        == False
+    assert a.is_integer     == False
+    assert a.is_negative    == False
+    assert a.is_positive    == False
+    assert a.is_nonnegative == False
+    assert a.is_nonpositive == False
+    assert a.is_zero        == False
+
+
 def test_issue726():
+    """catch: hash instability"""
     x = Symbol("x")
     y = Symbol("y")
     a1 = x+y
@@ -354,3 +431,47 @@ def test_issue726():
     h1 = hash(a1)
     h2 = hash(a2)
     assert h1 == h2
+
+def test_issue1723():
+    z = (-1)**Rational(1,3)*(1-I*sqrt(3))
+    assert z.is_real in [True, None]
+
+def test_hash_vs_typeinfo():
+    """seemingly different typeinfo, but in fact equal"""
+    x = Symbol('x')
+
+    # the following two are semantically equal
+    x1= Symbol('x', even=True)
+    x2= Symbol('x', integer=True, odd=False)
+
+    assert hash(x1) == hash(x2)
+    assert x1 == x2
+
+
+@XFAIL  # our __eq__ just does not look at it (yet)
+def test_hash_vs_typeinfo_2():
+    """different typeinfo should mean !eq"""
+    # the following two are semantically different
+    x = Symbol('x')
+    x1= Symbol('x', even=True)
+
+    assert x != x1
+
+
+def test_hash_vs_eq():
+    """catch: different hash for equal objects"""
+    a = 1 + S.Pi    # important: do not fold it into a Number instance
+    ha= hash(a)     #            it should be Add/Mul/... to trigger the bug
+
+    a.is_positive   # this uses .evalf() and deduces it is positive
+    assert a.is_positive == True
+
+    # be sure that hash stayed the same
+    assert ha == hash(a)
+
+    # now b should be the same expression
+    b = a.expand(trig=True)
+    hb= hash(b)
+
+    assert a == b
+    assert ha== hb
