@@ -1,6 +1,7 @@
 from sympy import Basic, S, Symbol, Real, Integer, Rational,  \
     sin, cos, exp, log, oo, sqrt, symbols, Integral, sympify, \
-    WildFunction, Poly, Function, Derivative, Number, pi, var
+    WildFunction, Poly, Function, Derivative, Number, pi, var, \
+    NumberSymbol, zoo, Piecewise
 
 from sympy.utilities.pytest import XFAIL, raises
 
@@ -10,7 +11,7 @@ class DummyNumber(object):
 
     If one has a Number class (e.g. Sage Integer, or some other custom class)
     that one wants to work well with SymPy, one has to implement at least the
-    methods of this class DummyNumber, resp. it's subclasses I5 and F1_1.
+    methods of this class DummyNumber, resp. its subclasses I5 and F1_1.
 
     Basically, one just needs to implement either __int__() or __float__() and
     then one needs to make sure that the class works with Python integers and
@@ -141,6 +142,17 @@ def test_ibasic():
         x /= b
     dotest(s)
 
+def test_relational():
+    assert (pi < 3) == False
+    assert (pi <= 3) == False
+    assert (pi > 3) == True
+    assert (pi >= 3) == True
+    assert (-pi < 3) == True
+    assert (-pi <= 3) == True
+    assert (-pi > 3) == False
+    assert (-pi >= 3) == False
+    assert (x - 2 < x - 3) == False
+
 def test_basic_nostr():
     for obj in basic_objs:
         for op in ['+','-','*','/','**']:
@@ -183,27 +195,36 @@ def test_as_leading_term3():
     assert (2*x+pi*x+x**2).as_leading_term(x) == 2*x + pi*x
 
 def test_atoms():
-   assert sorted(list(x.atoms())) == [x]
-   assert sorted(list((1+x).atoms())) == sorted([1, x])
+    assert sorted(list(x.atoms())) == [x]
+    assert sorted(list((1+x).atoms())) == sorted([1, x])
 
-   assert sorted(list((1+2*cos(x)).atoms(Symbol))) == [x]
-   assert sorted(list((1+2*cos(x)).atoms(Symbol,Number))) == sorted([1, 2, x])
+    assert sorted(list((1+2*cos(x)).atoms(Symbol))) == [x]
+    assert sorted(list((1+2*cos(x)).atoms(Symbol,Number))) == sorted([1, 2, x])
 
-   assert sorted(list((2*(x**(y**x))).atoms())) == sorted([2, x, y])
+    assert sorted(list((2*(x**(y**x))).atoms())) == sorted([2, x, y])
 
-   assert sorted(list(Rational(1,2).atoms())) == [S.Half]
-   assert sorted(list(Rational(1,2).atoms(Symbol))) == []
+    assert sorted(list(Rational(1,2).atoms())) == [S.Half]
+    assert sorted(list(Rational(1,2).atoms(Symbol))) == []
 
-   assert sorted(list(sin(oo).atoms(oo))) == [oo]
+    assert sorted(list(sin(oo).atoms(oo))) == [oo]
 
-   assert sorted(list(Poly(0, x).atoms())) == [S.Zero]
-   assert sorted(list(Poly(1, x).atoms())) == [S.One]
-   assert sorted(list(Poly(x, x).atoms())) == sorted([S.One, x])
-   assert sorted(list(Poly(x, x, y).atoms())) == sorted([S.One, x])
-   assert sorted(list(Poly(x + y, x, y).atoms())) == sorted([S.One, x, y])
-   assert sorted(list(Poly(x + y, x, y, z).atoms())) == sorted([S.One, x, y])
-   assert sorted(list(Poly(x + y*t, x, y, z).atoms())) == \
-           sorted([S.One, t, x, y])
+    assert sorted(list(Poly(0, x).atoms())) == [S.Zero]
+    assert sorted(list(Poly(1, x).atoms())) == [S.One]
+    assert sorted(list(Poly(x, x).atoms())) == sorted([S.One, x])
+    assert sorted(list(Poly(x, x, y).atoms())) == sorted([S.One, x])
+    assert sorted(list(Poly(x + y, x, y).atoms())) == sorted([S.One, x, y])
+    assert sorted(list(Poly(x + y, x, y, z).atoms())) == sorted([S.One, x, y])
+    assert sorted(list(Poly(x + y*t, x, y, z).atoms())) == \
+            sorted([S.One, t, x, y])
+    I=S.ImaginaryUnit
+    assert list((I*pi).atoms(NumberSymbol)) == [pi]
+    assert sorted((I*pi).atoms(NumberSymbol, I)) == \
+           sorted((I*pi).atoms(I,NumberSymbol)) == [pi, I]
+
+    I = S.ImaginaryUnit
+    assert list((I*pi).atoms(NumberSymbol)) == [pi]
+    assert sorted((I*pi).atoms(NumberSymbol, I)) == \
+           sorted((I*pi).atoms(I,NumberSymbol)) == [pi, I]
 
 def test_is_polynomial():
     z = Symbol('z')
@@ -252,30 +273,30 @@ def test_is_polynomial():
     assert ((x**2)*(y**2) + x*(y**2) + y*x + exp(2)).is_polynomial(x, y) == True
     assert ((x**2)*(y**2) + x*(y**2) + y*x + exp(x)).is_polynomial(x, y) == False
 
-def test_is_fraction():
+def test_is_rational_function():
     x,y = symbols('xy')
 
-    assert Integer(1).is_fraction() == True
-    assert Integer(1).is_fraction(x) == True
+    assert Integer(1).is_rational_function() == True
+    assert Integer(1).is_rational_function(x) == True
 
-    assert Rational(17,54).is_fraction() == True
-    assert Rational(17,54).is_fraction(x) == True
+    assert Rational(17,54).is_rational_function() == True
+    assert Rational(17,54).is_rational_function(x) == True
 
-    assert (12/x).is_fraction() == True
-    assert (12/x).is_fraction(x) == True
+    assert (12/x).is_rational_function() == True
+    assert (12/x).is_rational_function(x) == True
 
-    assert (x/y).is_fraction() == True
-    assert (x/y).is_fraction(x) == True
-    assert (x/y).is_fraction(x, y) == True
+    assert (x/y).is_rational_function() == True
+    assert (x/y).is_rational_function(x) == True
+    assert (x/y).is_rational_function(x, y) == True
 
-    assert (x**2+1/x/y).is_fraction() == True
-    assert (x**2+1/x/y).is_fraction(x) == True
-    assert (x**2+1/x/y).is_fraction(x, y) == True
+    assert (x**2+1/x/y).is_rational_function() == True
+    assert (x**2+1/x/y).is_rational_function(x) == True
+    assert (x**2+1/x/y).is_rational_function(x, y) == True
 
-    assert (sin(y)/x).is_fraction() == False
-    assert (sin(y)/x).is_fraction(y) == False
-    assert (sin(y)/x).is_fraction(x) == True
-    assert (sin(y)/x).is_fraction(x, y) == False
+    assert (sin(y)/x).is_rational_function() == False
+    assert (sin(y)/x).is_rational_function(y) == False
+    assert (sin(y)/x).is_rational_function(x) == True
+    assert (sin(y)/x).is_rational_function(x, y) == False
 
 def test_SAGE1():
     #see http://code.google.com/p/sympy/issues/detail?id=247
@@ -325,25 +346,18 @@ def test_doit():
 
     assert (2*Integral(x, x)).doit() == x**2
 
-def test_is_number():
-    assert Rational(8).is_number
-    assert not x.is_number
-    assert (8+log(2)).is_number
-    assert not (8+log(2)+x).is_number
-    assert (1+x**2/x-x).is_number
-
 def test_attribute_error():
     raises(AttributeError, "x.cos()")
     raises(AttributeError, "x.sin()")
     raises(AttributeError, "x.exp()")
 
 def test_args():
-    assert (x*y).args[:] in ((x, y), (y, x))
-    assert (x+y).args[:] in ((x, y), (y, x))
-    assert (x*y+1).args[:] in ((x*y, 1), (1, x*y))
-    assert sin(x*y).args[:] == (x*y,)
+    assert (x*y).args in ((x, y), (y, x))
+    assert (x+y).args in ((x, y), (y, x))
+    assert (x*y+1).args in ((x*y, 1), (1, x*y))
+    assert sin(x*y).args == (x*y,)
     assert sin(x*y).args[0] == x*y
-    assert (x**y).args[:] == (x,y)
+    assert (x**y).args == (x,y)
     assert (x**y).args[0] == x
     assert (x**y).args[1] == y
 
@@ -361,6 +375,15 @@ def test_noncommutative_expand_issue658():
     assert (A*(A+B)*B).expand() == A**2*B + A*B**2
     assert (A*(A+B+C)*B).expand() == A**2*B + A*B**2 + A*C*B
 
+def test_as_numer_denom():
+    assert oo.as_numer_denom() == (1, 0)
+    assert (-oo).as_numer_denom() == (-1, 0)
+    assert zoo.as_numer_denom() == (zoo, 1)
+    assert (-zoo).as_numer_denom() == (zoo, 1)
+    assert (1/x).as_numer_denom() == (1, x)
+    assert x.as_numer_denom() == (x, 1)
+    assert (x/y).as_numer_denom() == (x, y)
+
 def test_as_independent():
     assert (2*x*sin(x)+y+x).as_independent(x) == (y, x + 2*x*sin(x))
     assert (2*x*sin(x)+y+x).as_independent(y) == (x + 2*x*sin(x), y)
@@ -525,19 +548,25 @@ def test_is_number():
     assert Real(3.14).is_number == True
     assert Integer(737).is_number == True
     assert Rational(3, 2).is_number == True
-
+    assert Rational(8).is_number == True
     assert x.is_number == False
     assert (2*x).is_number == False
-
     assert (x + y).is_number == False
-
     assert log(2).is_number == True
     assert log(x).is_number == False
-
     assert (2 + log(2)).is_number == True
+    assert (8+log(2)).is_number == True
     assert (2 + log(x)).is_number == False
-
+    assert (8+log(2)+x).is_number == False
     assert (2*g).is_number == False
+    assert (1+x**2/x-x).is_number == True
+
+    # test extensibility of .is_number
+    # on subinstances of Basic
+    class A(Basic):
+        pass
+    a = A()
+    assert a.is_number == False
 
 
 # TODO write more tests for as_coeff_factors
@@ -565,6 +594,11 @@ def test_as_coeff_exponent():
     assert (2*x**3+pi*x**3).as_coeff_exponent(x) == (2+pi, 3)
     assert (x*log(2)/(2*x + pi*x)).as_coeff_exponent(x) == \
             (log(2)/(2+pi), 0)
+    # 1685
+    D = Derivative
+    f = Function('f')
+    fx  = D(f(x), x)
+    assert fx.as_coeff_exponent(f(x)) == (fx ,0)
 
 def test_extractions():
     x, y = symbols("xy")
@@ -589,6 +623,17 @@ def test_extractions():
     assert (x-y).could_extract_minus_sign() != (-x+y).could_extract_minus_sign()
     assert (1-x-y).could_extract_minus_sign() == True
     assert (1-x+y).could_extract_minus_sign() == False
+    assert ((-x-x*y)/y).could_extract_minus_sign() == True
+    assert (-(x+x*y)/y).could_extract_minus_sign() ==  True
+    assert ((x+x*y)/(-y)).could_extract_minus_sign() == True
+    assert ((x+x*y)/y).could_extract_minus_sign() == False
+    assert (x*(-x-x**3)).could_extract_minus_sign() == True # used to give inf recurs
+    assert ((-x-y)/(x+y)).could_extract_minus_sign() == True # is_Mul odd case
+    # The results of each of these will vary on different machines, e.g.
+    # the first one might be False and the other (then) is true or vice versa,
+    # so both are included.
+    assert ((-x-y)/(x-y)).could_extract_minus_sign() == False or\
+           ((-x-y)/(y-x)).could_extract_minus_sign() == False # is_Mul even case
 
 def test_coeff():
     assert (3+2*x+4*x**2).coeff(1) == None
@@ -603,6 +648,9 @@ def test_coeff():
     assert (x/8 + x*y).coeff(2*y*x) == S(1)/2
     assert (x/8 + x*y).coeff(y*x/2) == 2
 
+    f = Function('f')
+    assert (2*f(x) + 3*f(x).diff(x)).coeff(f(x)) == 2
+
 def test_coeff2():
     var('r, kappa')
     psi = Function("psi")
@@ -618,6 +666,15 @@ def test_coeff2_0():
 
     assert g.coeff(psi(r).diff(r, 2)) == 1
 
+def test_coeff_expand():
+    x, y, z = symbols('x y z')
+    expr = z*(x+y)**2
+    expr2 = z*(x+y)**2 + z*(2*x + 2*y)**2
+    assert expr.coeff(z) == 2*x*y + x**2 + y**2
+    assert expr.coeff(z, expand=False) == (x+y)**2
+    assert expr2.coeff(z) == 10*x*y + 5*x**2 + 5*y**2
+    assert expr2.coeff(z, expand=False) == (x+y)**2 + (2*x + 2*y)**2
+
 def test_integrate():
     assert (log(x)).integrate((x, 0, 1)) == -1
     assert sin(x).integrate(x) == -cos(x)
@@ -627,3 +684,43 @@ def test_count_ops():
     f = (x*y + 3/y)**(3 + 2)
     assert f.count_ops() == Symbol('ADD') + 2*Symbol('MUL') + 2*Symbol('POW')
     assert f.count_ops(symbolic=False) == 5
+
+def test_contains():
+    f = (x*y + 3/y)**(3 + 2)
+    g = Function('g')
+    h = Function('h')
+    p = Piecewise( (g, x<-1), (1, x<=1), (f, True))
+    assert x in p
+    assert y in p
+    assert not z in p
+    assert 1 in p
+    assert 3 in p
+    assert not 4 in p
+    assert f in p
+    assert g in p
+    assert not h in p
+
+################# Tests involving only Basic instances ########
+b1 = Basic(); b2 = Basic(b1); b3 = Basic(b2)
+b21 = Basic(b2, b1)
+
+def test_equality():
+    instances = [b1, b2, b3, b21, Basic(b1,b1,b1)]
+    for i, b_i in enumerate(instances):
+        for j, b_j in enumerate(instances):
+            assert (b_i == b_j) == (i == j)
+
+def test_matches_basic():
+    instances = [Basic(b1,b1,b2), Basic(b1,b2,b1), Basic(b2, b1, b1),
+                    Basic(b1, b2), Basic(b2, b1), b2, b1]
+    for i, b_i in enumerate(instances):
+        for j, b_j in enumerate(instances):
+            if i ==j:
+                assert b_i.matches(b_j) == {}
+            else:
+                assert b_i.matches(b_j) is None
+
+def test_subs():
+    assert b21.subs(b2, b1) == Basic(b1, b1)
+    assert b21.subs(b2, b21) == Basic(b21, b1)
+    assert b3.subs(b2, b1) == b2
