@@ -1,8 +1,8 @@
 from sympy import Symbol, exp, Integer, Real, sin, cos, log, Poly, Lambda, \
-        Function, I, S, sqrt,  raises
+        Function, I, S, sqrt,  raises, srepr
 from sympy.abc import x, y
-from sympy.core.sympify import sympify, _sympify, _sympifyit, SympifyError
-
+from sympy.core.sympify import sympify, _sympify, SympifyError
+from sympy.core.decorators import _sympifyit
 
 def test_439():
     v = sympify("exp(x)")
@@ -35,6 +35,12 @@ def test_sympify3():
     raises(SympifyError, "_sympify('x**3')")
     raises(SympifyError, "_sympify('1/2')")
 
+def test_sympify_bool():
+    """Test that sympify accepts boolean values
+    and that output leaves them unchanged"""
+    assert sympify(True) == True
+    assert sympify(False)== False
+
 def test_sympify4():
     class A:
         def _sympy_(self):
@@ -245,3 +251,10 @@ def test_issue883():
 def test_S_sympify():
     assert S(1)/2 == sympify(1)/2
     assert (-2)**(S(1)/2) == sqrt(2)*I
+
+def test_issue1689():
+    assert srepr(S(1.0+0J)) == srepr(S(1.0)) == srepr(Real(1.0))
+    assert srepr(Real(1)) != srepr(Real(1.0))
+
+def test_issue1699_None():
+    assert S(None) == None
