@@ -1,9 +1,10 @@
-
 from sympy.core.basic import Basic, S, C, sympify
 from sympy.core.function import Function
 from sympy.ntheory import sieve
 from math import sqrt
 
+from sympy.utilities.decorator import deprecated
+
 ###############################################################################
 ######################## FACTORIAL and MULTI-FACTORIAL ########################
 ###############################################################################
@@ -19,7 +20,7 @@ class Factorial(Function):
        arises in calculus, probability, number theory etc.
 
        There is strict relation of factorial with gamma function. In
-       fact n! = gamma(n+1) for nonnegarive integers. Rewrite of this
+       fact n! = gamma(n+1) for nonnegative integers. Rewrite of this
        kind is very useful in case of combinatorial simplification.
 
        Computation of the factorial is done using two algorithms. For
@@ -28,7 +29,7 @@ class Factorial(Function):
        known and computes n! via prime factorization of special class
        of numbers, called here the 'Swing Numbers'.
 
-       >>> from sympy import *
+       >>> from sympy import Symbol, factorial
        >>> n = Symbol('n', integer=True)
 
        >>> factorial(-2)
@@ -67,7 +68,7 @@ class Factorial(Function):
                 p, q = 1, n
 
                 while True:
-                    q /= prime
+                    q //= prime
 
                     if q > 0:
                         if q & 1 == 1:
@@ -78,13 +79,13 @@ class Factorial(Function):
                 if p > 1:
                     primes.append(p)
 
-            for prime in sieve.primerange(N+1, n/3 + 1):
-                if (n / prime) & 1 == 1:
+            for prime in sieve.primerange(N+1, n//3 + 1):
+                if (n // prime) & 1 == 1:
                     primes.append(prime)
 
             L_product = R_product = 1
 
-            for prime in sieve.primerange(n/2 + 1, n+1):
+            for prime in sieve.primerange(n//2 + 1, n+1):
                 L_product *= prime
 
             for prime in primes:
@@ -97,10 +98,10 @@ class Factorial(Function):
         if n < 2:
             return 1
         else:
-            return (cls._recursive(n/2)**2)*cls._swing(n)
+            return (cls._recursive(n//2)**2)*cls._swing(n)
 
     @classmethod
-    def canonize(cls, n):
+    def eval(cls, n):
         n = sympify(n)
 
         if n.is_Number:
@@ -139,9 +140,6 @@ class Factorial(Function):
     def _eval_rewrite_as_gamma(self, arg):
         return C.gamma(1 + arg)
 
-    def tostr(self, level=0):
-        return '%s!' % self.args[0].tostr(self.precedence)
-
     def _eval_is_integer(self):
         return self.args[0].is_integer
 
@@ -158,7 +156,7 @@ factorial   = Factorial
 class RisingFactorial(Function):
     """Rising factorial (also called Pochhammer symbol) is a double valued
        function arising in concrete mathematics, hypergeometric functions
-       and series expanansions. It is defined by
+       and series expansions. It is defined by
 
                    rf(x, k) = x * (x+1) * ... * (x + k-1)
 
@@ -166,8 +164,8 @@ class RisingFactorial(Function):
        more information check "Concrete mathematics" by Graham, pp. 66
        or visit http://mathworld.wolfram.com/RisingFactorial.html page.
 
-       >>> from sympy import *
-       >>> x = Symbol('x')
+       >>> from sympy import rf
+       >>> from sympy.abc import x
 
        >>> rf(x, 0)
        1
@@ -183,7 +181,7 @@ class RisingFactorial(Function):
     nargs = 2
 
     @classmethod
-    def canonize(cls, x, k):
+    def eval(cls, x, k):
         x = sympify(x)
         k = sympify(k)
 
@@ -221,7 +219,7 @@ class RisingFactorial(Function):
 class FallingFactorial(Function):
     """Falling factorial (related to rising factorial) is a double valued
        function arising in concrete mathematics, hypergeometric functions
-       and series expanansions. It is defined by
+       and series expansions. It is defined by
 
                    ff(x, k) = x * (x-1) * ... * (x - k+1)
 
@@ -229,8 +227,8 @@ class FallingFactorial(Function):
        more information check "Concrete mathematics" by Graham, pp. 66
        or visit http://mathworld.wolfram.com/FallingFactorial.html page.
 
-       >>> from sympy import *
-       >>> x = Symbol('x')
+       >>> from sympy import ff
+       >>> from sympy.abc import x
 
        >>> ff(x, 0)
        1
@@ -246,7 +244,7 @@ class FallingFactorial(Function):
     nargs = 2
 
     @classmethod
-    def canonize(cls, x, k):
+    def eval(cls, x, k):
         x = sympify(x)
         k = sympify(k)
 
@@ -295,21 +293,21 @@ class Binomial(Function):
 
            C(n,k) = n!/(k!(n-k)!)   or   C(n, k) = ff(n, k)/k!
 
-       First formula has strict combinatorial meaning, definig the
-       number of ways we can choose 'k' elements from 'n' element
-       set. In this case both arguments are nonnegative integers
-       and binomial is computed using efficient algorithm based
-       on prime factorisation.
+       First, in a strict combinatorial sense it defines the
+       number of ways we can choose 'k' elements from a set of
+       'n' elements. In this case both arguments are nonnegative
+       integers and binomial is computed using an efficient
+       algorithm based on prime factorization.
 
-       The other definition is generalisation for arbitaty 'n',
-       however 'k' must be also nonnegative. This case is very
-       useful in case for evaluating summations.
+       The other definition is generalization for arbitrary 'n',
+       however 'k' must also be nonnegative. This case is very
+       useful when evaluating summations.
 
        For the sake of convenience for negative 'k' this function
        will return zero no matter what valued is the other argument.
 
-       >>> from sympy import *
-       >>> n = symbols('n', integer=True)
+       >>> from sympy import Symbol, Rational, binomial
+       >>> n = Symbol('n', integer=True)
 
        >>> binomial(15, 8)
        6435
@@ -332,14 +330,14 @@ class Binomial(Function):
        -5/128
 
        >>> binomial(n, 3)
-       (1/6)*n*(-1 + n)*(-2 + n)
+       n*(1 - n)*(2 - n)/6
 
     """
 
     nargs = 2
 
     @classmethod
-    def canonize(cls, r, k):
+    def eval(cls, r, k):
         r, k = map(sympify, (r, k))
 
         if k.is_Number:
@@ -354,7 +352,7 @@ class Binomial(Function):
 
                         if k > r:
                             return S.Zero
-                        elif k > r / 2:
+                        elif k > r // 2:
                             k = r - k
 
                         M, result = int(sqrt(r)), 1
@@ -362,7 +360,7 @@ class Binomial(Function):
                         for prime in sieve.primerange(2, r+1):
                             if prime > r - k:
                                 result *= prime
-                            elif prime > r / 2:
+                            elif prime > r // 2:
                                 continue
                             elif prime > M:
                                 if r % prime < k % prime:
@@ -373,7 +371,7 @@ class Binomial(Function):
 
                                 while R > 0:
                                     a = int((R % prime) < (K % prime + a))
-                                    R, K = R / prime, K / prime
+                                    R, K = R // prime, K // prime
                                     exp = a + exp
 
                                 if exp > 0:
