@@ -1,9 +1,9 @@
-
 from sympy.core.basic import Basic, S, C, sympify
-from sympy.core.function import Function
+from sympy.core.function import Function, Derivative
 from sympy.functions.elementary.miscellaneous import sqrt
 
 from sympy.utilities.decorator import deprecated
+from sympy.utilities.iterables import make_list, iff
 
 ###############################################################################
 ######################### REAL and IMAGINARY PARTS ############################
@@ -16,9 +16,8 @@ class re(Function):
        is needed then use Basic.as_real_imag() or perform complex
        expansion on instance of this function.
 
-       >>> from sympy import *
-
-       >>> x, y = symbols('x', 'y')
+       >>> from sympy import re, im, I, E
+       >>> from sympy.abc import x, y
 
        >>> re(2*E)
        2*E
@@ -38,29 +37,18 @@ class re(Function):
 
     is_real = True
 
-    @classmethod
-    def _eval_apply_subs(self, *args):
-        return
-
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         if arg is S.NaN:
             return S.NaN
         elif arg.is_real:
             return arg
+        elif arg.is_Function and arg.func == conjugate:
+            return re(arg.args[0])
         else:
-            if not arg.is_Add:
-                arg = [arg]
 
             included, reverted, excluded = [], [], []
-
-            if isinstance(arg, Basic):
-                arg = arg.args
+            arg = make_list(arg, C.Add)
             for term in arg:
                 coeff = term.as_coefficient(S.ImaginaryUnit)
 
@@ -72,7 +60,7 @@ class re(Function):
                 else:
                     included.append(term)
 
-            if len(arg[:]) != len(included):
+            if len(arg) != len(included):
                 a, b, c = map(lambda xs: C.Add(*xs),
                     [included, reverted, excluded])
 
@@ -81,9 +69,17 @@ class re(Function):
     def _eval_conjugate(self):
         return self
 
-    def _eval_expand_complex(self, *args):
+    def _eval_expand_complex(self, deep=True, **hints):
+#        if deep:
+#            return self.args[0].expand(deep, **hints).as_real_imag()[0]
+#        else:
         return self.args[0].as_real_imag()[0]
 
+    def _eval_derivative(self, x):
+        if not self.has(x):
+            return S.Zero
+        return re(Derivative(self.args[0], x, **{'evaluate': True}))
+
 class im(Function):
     """Returns imaginary part of expression. This function performs
        only elementary analysis and so it will fail to decompose
@@ -91,9 +87,8 @@ class im(Function):
        result is needed then use Basic.as_real_imag() or perform complex
        expansion on instance of this function.
 
-       >>> from sympy import *
-
-       >>> x, y = symbols('x', 'y')
+       >>> from sympy import re, im, E, I
+       >>> from sympy.abc import x, y
 
        >>> im(2*E)
        0
@@ -113,29 +108,17 @@ class im(Function):
 
     is_real = True
 
-    @classmethod
-    def _eval_apply_subs(self, *args):
-        return
-
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         if arg is S.NaN:
             return S.NaN
         elif arg.is_real:
             return S.Zero
+        elif arg.is_Function and arg.func == conjugate:
+            return -im(arg.args[0])
         else:
-            if not arg.is_Add:
-                arg = [arg]
-
             included, reverted, excluded = [], [], []
-
-            if isinstance(arg, Basic):
-                arg = arg.args
+            arg = make_list(arg, C.Add)
             for term in arg:
                 coeff = term.as_coefficient(S.ImaginaryUnit)
 
@@ -147,7 +130,7 @@ class im(Function):
                 elif term.has(S.ImaginaryUnit) or not term.is_real:
                     included.append(term)
 
-            if len(arg[:]) != len(included):
+            if len(arg) != len(included):
                 a, b, c = map(lambda xs: C.Add(*xs),
                     [included, reverted, excluded])
 
@@ -156,22 +139,29 @@ class im(Function):
     def _eval_conjugate(self):
         return self
 
-    def _eval_expand_complex(self, *args):
+    def _eval_expand_complex(self, deep=True, **hints):
+#        if deep:
+#            return self.args[0].expand(deep, **hints).as_real_imag()[1]
         return self.args[0].as_real_imag()[1]
 
+    def _eval_derivative(self, x):
+        if not self.has(x):
+            return S.Zero
+        return im(Derivative(self.args[0], x, **{'evaluate': True}))
+
 ###############################################################################
 ############### SIGN, ABSOLUTE VALUE, ARGUMENT and CONJUGATION ################
 ###############################################################################
 
 class sign(Function):
+    """Return the sign of an expression, that is:
+        -1 if expr <  0
+         0 if expr == 0
+         1 if expr >  0
+    """
 
     nargs = 1
 
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         if arg is S.NaN:
@@ -179,10 +169,20 @@ class sign(Function):
         if arg is S.Zero: return S.Zero
         if arg.is_positive: return S.One
         if arg.is_negative: return S.NegativeOne
+        if arg.is_Function:
+            if arg.func is sign: return arg
         if arg.is_Mul:
-            coeff, terms = arg.as_coeff_terms()
-            if coeff is not S.One:
-                return cls(coeff) * cls(C.Mul(*terms))
+            c, args = arg.as_coeff_terms()
+            unk = []
+            is_neg = c.is_negative
+            for ai in args:
+                if ai.is_negative == None:
+                    unk.append(ai)
+                elif ai.is_negative:
+                    is_neg = not is_neg
+            if c is S.One and len(unk) == len(args):
+                return None
+            return iff(is_neg, S.NegativeOne, S.One) * cls(C.Mul(*unk))
 
     is_bounded = True
 
@@ -190,7 +190,7 @@ class sign(Function):
         return self
 
     def _eval_is_zero(self):
-        return (self[0] is S.Zero)
+        return (self.args[0] is S.Zero)
 
 class abs(Function):
     """Return the absolute value of the argument. This is an extension of the built-in
@@ -206,6 +206,7 @@ class abs(Function):
         abs(x)
         >>> abs(x**2)
         x**2
+
     """
 
     nargs = 1
@@ -219,15 +220,6 @@ class abs(Function):
         else:
             raise ArgumentIndexError(self, argindex)
 
-    @classmethod
-    def _eval_apply_subs(self, *args):
-        return
-
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         if arg is S.NaN:
@@ -279,18 +271,23 @@ class abs(Function):
         import sage.all as sage
         return sage.abs_symbolic(self.args[0]._sage_())
 
+    def _eval_derivative(self, x):
+        if not self.has(x):
+            return S.Zero
+        if self.args[0].is_real:
+            return Derivative(self.args[0], x, **{'evaluate': True}) * sign(self.args[0])
+        return (re(self.args[0]) * re(Derivative(self.args[0], x,
+            **{'evaluate': True})) + im(self.args[0]) * im(Derivative(self.args[0],
+                x, **{'evaluate': True}))) / abs(self.args[0])
+
 class arg(Function):
+    """Returns the argument (in radians) of a complex number"""
 
     nargs = 1
 
     is_real = True
     is_bounded = True
 
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         x, y = re(arg), im(arg)
@@ -301,10 +298,17 @@ class arg(Function):
     def _eval_conjugate(self):
         return self
 
+    def _eval_derivative(self, t):
+        x, y = re(self.args[0]), im(self.args[0])
+        if not self.has(t):
+            return S.Zero
+        return (x * Derivative(y, t, **{'evaluate': True}) - y *
+                Derivative(x, t, **{'evaluate': True})) / (x**2 + y**2)
+
 class conjugate(Function):
     """Changes the sign of the imaginary part of a complex number.
 
-        >>> from sympy import *
+        >>> from sympy import conjugate, I
 
         >>> conjugate(1 + I)
         1 - I
@@ -313,11 +317,6 @@ class conjugate(Function):
 
     nargs = 1
 
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         obj = arg._eval_conjugate()
@@ -327,6 +326,10 @@ class conjugate(Function):
     def _eval_conjugate(self):
         return self.args[0]
 
+    def _eval_derivative(self, x):
+        if not self.has(x):
+            return S.Zero
+        return conjugate(Derivative(self.args[0], x, **{'evaluate': True}))
 
 # /cyclic/
 from sympy.core import basic as _
