@@ -1,10 +1,12 @@
-
 from sympy.core.basic import Basic, S, C, sympify, Wild
-from sympy.core.function import Lambda, Function, Function
+from sympy.core.function import Lambda, Function, Function, expand_log
 from sympy.core.cache import cacheit
 
 from sympy.utilities.decorator import deprecated
 
+from sympy.ntheory import multiplicity
+from sympy.core.basic import Mul
+
 class exp(Function):
 
     nargs = 1
@@ -18,15 +20,6 @@ class exp(Function):
     def inverse(self, argindex=1):
         return log
 
-    @classmethod
-    def _eval_apply_subs(self, *args):
-        return
-
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         if arg.is_Number:
@@ -63,7 +56,7 @@ class exp(Function):
                 return S.NaN
 
         if arg.is_Add:
-            args = arg.args[:]
+            args = arg.args
         else:
             args = [arg]
 
@@ -110,8 +103,11 @@ class exp(Function):
                 return p * x / n
         return x**n/C.Factorial()(n)
 
-    def _eval_expand_complex(self, *args):
+    def _eval_expand_complex(self, deep=True, **hints):
         re, im = self.args[0].as_real_imag()
+        if deep:
+            re = re.expand(deep, **hints)
+            im = im.expand(deep, **hints)
         cos, sin = C.cos(im), C.sin(im)
         return exp(re) * cos + S.ImaginaryUnit * exp(re) * sin
 
@@ -126,8 +122,6 @@ class exp(Function):
         if x is not None:
             c,f = arg.as_coeff_factors(x)
             return self.func(c), tuple( self.func(a) for a in f )
-        if arg.is_Add:
-            return S.One, tuple( self.func(a) for a in arg.args )
         return S.One,(self,)
 
     def _eval_subs(self, old, new):
@@ -199,11 +193,11 @@ class exp(Function):
             yield term
 
     def _eval_nseries(self, x, x0, n):
+        from sympy import limit, Symbol, oo, powsimp
         arg = self.args[0]
         arg_series = arg.nseries(x, x0, n)
         if arg_series.is_Order:
             return 1+arg_series
-        from sympy import limit, Symbol, oo
         arg0 = limit(arg_series, x, x0)
         if arg0 in [-oo, oo]:
             return self
@@ -211,7 +205,7 @@ class exp(Function):
         exp_series = exp(s)._taylor(s, x0, n)
         r = exp(arg0)*exp_series.subs(s, arg_series-arg0)
         r = r.expand()
-        return r
+        return powsimp(r, deep=True, combine='exp')
 
     def _taylor(self, x, x0, n):
         l = []
@@ -231,12 +225,17 @@ class exp(Function):
             return S.One
         return exp(arg)
 
-    def _eval_expand_basic(self, *args):
-        arg = self.args[0].expand()
+    def _eval_expand_power_exp(self, deep=True, **hints):
+        if deep:
+            arg = self.args[0].expand(deep=deep, **hints)
+        else:
+            arg = self.args[0]
         if arg.is_Add:
             expr = 1
             for x in arg.args:
-                expr *= self.func(x).expand()
+                if deep:
+                    x = x.expand(deep=deep, **hints)
+                expr *= self.func(x)
             return expr
         return self.func(arg)
 
@@ -267,20 +266,17 @@ class log(Function):
     def inverse(self, argindex=1):
         return exp
 
-    @classmethod
-    def _eval_apply_subs(self, *args):
-        return
-
-    @classmethod
-    @deprecated
-    def canonize(cls, arg, base=None):
-        return cls.eval(arg, base)
-
     @classmethod
     def eval(cls, arg, base=None):
         if base is not None:
             base = sympify(base)
 
+            if arg.is_positive and arg.is_Integer and \
+               base.is_positive and base.is_Integer:
+                base = int(base)
+                arg = int(arg)
+                n = multiplicity(base, arg)
+                return S(n) + log(arg // base ** n) / log(base)
             if base is not S.Exp1:
                 return cls(arg)/cls(base)
             else:
@@ -304,7 +300,7 @@ class log(Function):
         elif arg is S.Exp1:
             return S.One
         #this doesn't work due to caching: :(
-        #elif arg.func is exp and arg[0].is_real:
+        #elif arg.func is exp and arg.args[0].is_real:
         #using this one instead:
         elif arg.func is exp:
             return arg.args[0]
@@ -331,25 +327,63 @@ class log(Function):
 
     def as_base_exp(self):
         return self, S.One
-        #why is this here:?
-        return exp, S.NegativeOne
 
     @staticmethod
     @cacheit
     def taylor_term(n, x, *previous_terms): # of log(1+x)
+        from sympy import powsimp
         if n<0: return S.Zero
         x = sympify(x)
         if n==0: return x
         if previous_terms:
             p = previous_terms[-1]
             if p is not None:
-                return (-n) * p * x / (n+1)
+                return powsimp((-n) * p * x / (n+1), deep=True, combine='exp')
         return (1-2*(n%2)) * x**(n+1)/(n+1)
 
-    def _eval_expand_complex(self, *args):
-        abs = C.abs(self.args[0])
-        arg = C.arg(self.args[0])
-        return log(abs) + S.ImaginaryUnit * arg
+    def _eval_expand_log(self, deep=True, **hints):
+        if deep:
+            arg = self.args[0].expand(deep=deep, **hints)
+        else:
+            arg = self.args[0]
+        if arg.is_Mul:
+            expr = sympify(0)
+            nonpos = sympify(1)
+            for x in arg.args:
+                if deep:
+                    x = x.expand(deep=deep, **hints)
+                if x.is_positive:
+                    expr += self.func(x)._eval_expand_log(deep=deep, **hints)
+                else:
+                    nonpos *= x
+            return expr + log(nonpos)
+        elif arg.is_Pow:
+            if arg.exp.is_real:# and arg.base.is_positive:
+                # This should only run when base.is_positive, but it breaks
+                # nseries, so it will have to wait for the new assumptions system.
+                # See the variable obj2 in log._eval_nseries.
+                if deep:
+                    b = arg.base.expand(deep=deep, **hints)
+                    e = arg.exp.expand(deep=deep, **hints)
+                else:
+                    b = arg.base
+                    e = arg.exp
+                return e * self.func(b)._eval_expand_log(deep=deep,\
+                **hints)
+        return self.func(arg)
+
+    def _eval_expand_complex(self, deep=True, **hints):
+        if deep:
+            abs = C.abs(self.args[0].expand(deep, **hints))
+            arg = C.arg(self.args[0].expand(deep, **hints))
+        else:
+            abs = C.abs(self.args[0])
+            arg = C.arg(self.args[0])
+        if hints['log']: # Expand the log
+            hints['complex'] = False
+            return log(abs).expand(deep, **hints) + S.ImaginaryUnit * arg
+        else:
+            return log(abs) + S.ImaginaryUnit * arg
 
     def _eval_is_real(self):
         return self.args[0].is_positive
@@ -380,6 +414,7 @@ class log(Function):
         return (self.func(n) - self.func(d)).as_numer_denom()
 
     def _eval_nseries(self, x, x0, n):
+        from sympy import powsimp
         arg = self.args[0]
         k, l = Wild("k"), Wild("l")
         r = arg.match(k*x**l)
@@ -399,7 +434,7 @@ class log(Function):
         if use_lt: # singularity, #example: self = log(sin(x))
             # arg = (arg / lt) * lt
             lt = arg.as_leading_term(x) # arg = sin(x); lt = x
-            a = (arg/lt).expand() # a = sin(x)/x
+            a = powsimp((arg/lt).expand(), deep=True, combine='exp') # a = sin(x)/x
             # the idea is to recursively call ln(a).series(), but one needs to
             # make sure that ln(sin(x)/x) doesn't get "simplified" to
             # -log(x)+ln(sin(x)) and an infinite recursion occurs, see also the
@@ -435,33 +470,22 @@ class log(Function):
                 g = g.nseries(x, x0, n)
                 l.append(g)
             obj = C.Add(*l) + ln(arg0)
-        obj2 = obj.expand()
+        obj2 = expand_log(powsimp(obj, deep=True, combine='exp'))
         if obj2 != obj:
             r = obj2.nseries(x, x0, n)
         else:
             r = obj
-        if r==self:
+        if r == self:
             return self
         return r + order
 
+
     def _eval_as_leading_term(self, x):
         arg = self.args[0].as_leading_term(x)
         if arg is S.One:
             return (self.args[0] - 1).as_leading_term(x)
         return self.func(arg)
 
-    def _eval_expand_basic(self, *args):
-        arg = self.args[0]
-        if arg.is_Mul and arg.is_real:
-            expr = 0
-            for x in arg.args:
-                expr += self.func(x).expand()
-            return expr
-        elif arg.is_Pow:
-            if arg.exp.is_number or arg.exp.is_NumberSymbol:
-                return arg.exp * self.func(arg.base).expand()
-        return self
-
     #this is a lot faster:
     @classmethod
     def _eval_apply_evalf(cls, arg):
@@ -498,11 +522,6 @@ class LambertW(Function):
     """
     nargs = 1
 
-    @classmethod
-    @deprecated
-    def canonize(cls, x):
-        return cls.eval(x)
-
     @classmethod
     def eval(cls, x):
         if x == S.Zero: return S.Zero
