@@ -1,4 +1,3 @@
-
 from sympy.core.basic import Basic, S, C, sympify
 from sympy.core.function import Function, Lambda
 from sympy.core.cache import cacheit
@@ -12,7 +11,11 @@ from sympy.utilities.decorator import deprecated
 ###############################################################################
 
 class sinh(Function):
-
+    """
+    Usage
+    =====
+      sinh(x) -> Returns the hyperbolic sine of x
+    """
     nargs = 1
 
     def fdiff(self, argindex=1):
@@ -24,15 +27,6 @@ class sinh(Function):
     def inverse(self, argindex=1):
         return asinh
 
-    @classmethod
-    def _eval_apply_subs(self, *args):
-        return
-
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return eval(cls, arg)
-
     @classmethod
     def eval(cls, arg):
         arg = sympify(arg)
@@ -59,14 +53,14 @@ class sinh(Function):
                 if coeff.is_negative:
                     return -cls(-arg)
 
-            if isinstance(arg, asinh):
+            if arg.func == asinh:
                 return arg.args[0]
 
-            if isinstance(arg, acosh):
+            if arg.func == acosh:
                 x = arg.args[0]
                 return sqrt(x-1) * sqrt(x+1)
 
-            if isinstance(arg, atanh):
+            if arg.func == atanh:
                 x = arg.args[0]
                 return x/sqrt(1-x**2)
 
@@ -87,10 +81,17 @@ class sinh(Function):
     def _eval_conjugate(self):
         return self.func(self.args[0].conjugate())
 
-    def _eval_expand_complex(self, *args):
+    def _eval_expand_complex(self, deep=True, **hints):
         if self.args[0].is_real:
-            return self
-        re, im = self.args[0].as_real_imag()
+            if deep:
+                hints['complex'] = False
+                return self.expand(deep, **hints)
+            else:
+                return self
+        if deep:
+            re, im = self.args[0].expand(deep, **hints).as_real_imag()
+        else:
+            re, im = self.args[0].as_real_imag()
         return sinh(re)*C.cos(im) + cosh(re)*C.sin(im)*S.ImaginaryUnit
 
     def _eval_rewrite_as_exp(self, arg):
@@ -128,7 +129,11 @@ class sinh(Function):
         return sage.sinh(self.args[0]._sage_())
 
 class cosh(Function):
-
+    """
+    Usage
+    =====
+      cosh(x) -> Returns the hyperbolic cosine of x
+    """
     nargs = 1
 
     def fdiff(self, argindex=1):
@@ -140,15 +145,6 @@ class cosh(Function):
     def inverse(self, argindex=1):
         return acosh
 
-    @classmethod
-    def _eval_apply_subs(self, *args):
-        return
-
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         arg = sympify(arg)
@@ -175,13 +171,13 @@ class cosh(Function):
                 if coeff.is_negative:
                     return cls(-arg)
 
-            if isinstance(arg, asinh):
+            if arg.func == asinh:
                 return sqrt(1+arg.args[0]**2)
 
-            if isinstance(arg, acosh):
+            if arg.func == acosh:
                 return arg.args[0]
 
-            if isinstance(arg, atanh):
+            if arg.func == atanh:
                 return 1/sqrt(1-arg.args[0]**2)
 
     @staticmethod
@@ -201,10 +197,17 @@ class cosh(Function):
     def _eval_conjugate(self):
         return self.func(self.args[0].conjugate())
 
-    def _eval_expand_complex(self, *args):
+    def _eval_expand_complex(self, deep=True, **hints):
         if self.args[0].is_real:
-            return self
-        re, im = self.args[0].as_real_imag()
+            if deep:
+                hints['complex'] = False
+                return self.expand(deep, **hints)
+            else:
+                return self
+        if deep:
+            re, im = self.args[0].expand(deep, **hints).as_real_imag()
+        else:
+            re, im = self.args[0].as_real_imag()
         return cosh(re)*C.cos(im) + sinh(re)*C.sin(im)*S.ImaginaryUnit
 
     def _eval_rewrite_as_exp(self, arg):
@@ -242,7 +245,11 @@ class cosh(Function):
         return sage.cosh(self.args[0]._sage_())
 
 class tanh(Function):
-
+    """
+    Usage
+    =====
+      tanh(x) -> Returns the hyperbolic tangent of x
+    """
     nargs = 1
 
     def fdiff(self, argindex=1):
@@ -254,15 +261,6 @@ class tanh(Function):
     def inverse(self, argindex=1):
         return atanh
 
-    @classmethod
-    def _eval_apply_subs(self, *args):
-        return
-
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         arg = sympify(arg)
@@ -289,15 +287,15 @@ class tanh(Function):
                 if coeff.is_negative:
                     return -cls(-arg)
 
-            if isinstance(arg, asinh):
+            if arg.func == asinh:
                 x = arg.args[0]
                 return x/sqrt(1+x**2)
 
-            if isinstance(arg, acosh):
+            if arg.func == acosh:
                 x = arg.args[0]
                 return sqrt(x-1) * sqrt(x+1) / x
 
-            if isinstance(arg, atanh):
+            if arg.func == atanh:
                 return arg.args[0]
 
     @staticmethod
@@ -318,10 +316,17 @@ class tanh(Function):
     def _eval_conjugate(self):
         return self.func(self.args[0].conjugate())
 
-    def _eval_expand_complex(self, *args):
+    def _eval_expand_complex(self, deep=True, **hints):
         if self.args[0].is_real:
-            return self
-        re, im = self.args[0].as_real_imag()
+            if deep:
+                hints['complex'] = False
+                return self.expand(deep, **hints)
+            else:
+                return self
+        if deep:
+            re, im = self.args[0].expand(deep, **hints).as_real_imag()
+        else:
+            re, im = self.args[0].as_real_imag()
         denom = sinh(re)**2 + C.cos(im)**2
         return (sinh(re)*cosh(re) + \
             S.ImaginaryUnit*C.sin(im)*C.cos(im))/denom
@@ -360,7 +365,11 @@ class tanh(Function):
         return sage.tanh(self.args[0]._sage_())
 
 class coth(Function):
-
+    """
+    Usage
+    =====
+      coth(x) -> Returns the hyperbolic cotangent of x
+    """
     nargs = 1
 
     def fdiff(self, argindex=1):
@@ -372,15 +381,6 @@ class coth(Function):
     def inverse(self, argindex=1):
         return acoth
 
-    @classmethod
-    def _eval_apply_subs(self, *args):
-        return
-
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         arg = sympify(arg)
@@ -425,10 +425,16 @@ class coth(Function):
     def _eval_conjugate(self):
         return self.func(self.args[0].conjugate())
 
-    def _eval_expand_complex(self, *args):
+    def _eval_expand_complex(self, deep=True, **hints):
         if self.args[0].is_real:
-            return self
-        re, im = self.args[0].as_real_imag()
+            if deep:
+                return self.expand(deep, **hints)
+            else:
+                return self
+        if deep:
+            re, im = self.args[0].expand(deep, **hints).as_real_imag()
+        else:
+            re, im = self.args[0].as_real_imag()
         denom = sinh(re)**2 + C.sin(im)**2
         return (sinh(re)*cosh(re) - \
             S.ImaginaryUnit*C.sin(im)*C.cos(im))/denom
@@ -464,7 +470,11 @@ class coth(Function):
 ###############################################################################
 
 class asinh(Function):
-
+    """
+    Usage
+    =====
+      asinh(x) -> Returns the inverse hyperbolic sine of x
+    """
     nargs = 1
 
     def fdiff(self, argindex=1):
@@ -473,15 +483,6 @@ class asinh(Function):
         else:
             raise ArgumentIndexError(self, argindex)
 
-    @classmethod
-    def _eval_apply_subs(self, *args):
-        return
-
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         arg = sympify(arg)
@@ -519,16 +520,13 @@ class asinh(Function):
             return S.Zero
         else:
             x = sympify(x)
-
-            if len(previous_terms) > 2:
+            if len(previous_terms) >= 2 and n > 2:
                 p = previous_terms[-2]
-                return -p * (n-2)**2/(k*(k-1)) * x**2
+                return -p * (n-2)**2/(n*(n-1)) * x**2
             else:
                 k = (n - 1) // 2
-
                 R = C.RisingFactorial(S.Half, k)
                 F = C.Factorial(k)
-
                 return (-1)**k * R / F * x**n / n
 
     def _eval_as_leading_term(self, x):
@@ -544,7 +542,11 @@ class asinh(Function):
         return sage.asinh(self.args[0]._sage_())
 
 class acosh(Function):
-
+    """
+    Usage
+    =====
+      acosh(x) -> Returns the inverse hyperbolic cosine of x
+    """
     nargs = 1
 
     def fdiff(self, argindex=1):
@@ -553,15 +555,6 @@ class acosh(Function):
         else:
             raise ArgumentIndexError(self, argindex)
 
-    @classmethod
-    def _eval_apply_subs(self, *args):
-        return
-
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         arg = sympify(arg)
@@ -603,16 +596,13 @@ class acosh(Function):
             return S.Zero
         else:
             x = sympify(x)
-
-            if len(previous_terms) > 2:
+            if len(previous_terms) >= 2 and n > 2:
                 p = previous_terms[-2]
-                return p * (n-2)**2/(k*(k-1)) * x**2
+                return p * (n-2)**2/(n*(n-1)) * x**2
             else:
                 k = (n - 1) // 2
-
                 R = C.RisingFactorial(S.Half, k)
                 F = C.Factorial(k)
-
                 return -R / F * S.ImaginaryUnit * x**n / n
 
     def _eval_as_leading_term(self, x):
@@ -628,7 +618,11 @@ class acosh(Function):
         return sage.acosh(self.args[0]._sage_())
 
 class atanh(Function):
-
+    """
+    Usage
+    =====
+      atanh(x) -> Returns the inverse hyperbolic tangent of x
+    """
     nargs = 1
 
     def fdiff(self, argindex=1):
@@ -637,15 +631,6 @@ class atanh(Function):
         else:
             raise ArgumentIndexError(self, argindex)
 
-    @classmethod
-    def _eval_apply_subs(self, *args):
-        return
-
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         arg = sympify(arg)
@@ -694,7 +679,11 @@ class atanh(Function):
         return sage.atanh(self.args[0]._sage_())
 
 class acoth(Function):
-
+    """
+    Usage
+    =====
+      acoth(x) -> Returns the inverse hyperbolic cotangent of x
+    """
     nargs = 1
 
     def fdiff(self, argindex=1):
@@ -703,15 +692,6 @@ class acoth(Function):
         else:
             raise ArgumentIndexError(self, argindex)
 
-    @classmethod
-    def _eval_apply_subs(self, *args):
-        return
-
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         arg = sympify(arg)
