@@ -1,6 +1,6 @@
-
 from sympy.core.basic import Basic, S, C, sympify
 from sympy.core.function import Lambda, Function
+from sympy.utilities.decorator import deprecated
 
 ###############################################################################
 ############################# SQUARE ROOT FUNCTION ############################
@@ -19,7 +19,7 @@ class max_(Function):
     nargs = 2
 
     @classmethod
-    def canonize(cls, x, y):
+    def eval(cls, x, y):
         if x.is_Number and y.is_Number:
             return max(x, y)
         if x.is_positive:
@@ -42,6 +42,6 @@ class min_(Function):
     nargs = 2
 
     @classmethod
-    def canonize(cls, x, y):
+    def eval(cls, x, y):
         if x.is_Number and y.is_Number:
             return min(x, y)
