@@ -1,5 +1,5 @@
 from sympy import symbols, Symbol, sqrt, oo, re, nan, im, sign, I, E, log, \
-        pi, arg, conjugate, expand
+        pi, arg, conjugate, expand, exp, sin, cos, Function
 from sympy.utilities.pytest import XFAIL
 
 
@@ -86,16 +86,28 @@ def test_sign():
     assert sign(1.2) == 1
     assert sign(-1.2) == -1
     assert sign(0) == 0
+    x = Symbol('x')
+    assert sign(x).is_zero == False
+    assert sign(2*x) == sign(x)
+    p = Symbol('p', positive = True)
+    n = Symbol('n', negative = True)
+    m = Symbol('m', negative = True)
+    assert sign(2*p*x) == sign(x)
+    assert sign(n*x) == -sign(x)
+    assert sign(n*m*x) == sign(x)
+    x = 0
+    assert sign(x).is_zero == True
+
 
 def test_abs():
     x, y = symbols('xy')
     assert abs(0) == 0
     assert abs(1) == 1
     assert abs(-1)== 1
-    assert abs(x).diff(x) == sign(x)
     x = Symbol('x',real=True)
     n = Symbol('n',integer=True)
     assert x**(2*n) == abs(x)**(2*n)
+    assert abs(x).diff(x) == sign(x)
 
 def test_abs_real():
     # test some properties of abs that only apply
@@ -158,3 +170,18 @@ def test_issue936():
     assert abs(x).expand(trig=True)     == abs(x)
     assert sign(x).expand(trig=True)    == sign(x)
     assert arg(x).expand(trig=True)     == arg(x)
+
+def test_issue1655_derivative_conjugate():
+    x = Symbol('x')
+    f = Function('f')
+    assert (f(x).conjugate()).diff(x) == (f(x).diff(x)).conjugate()
+
+def test_derivatives_issue1658():
+    x = Symbol('x')
+    f = Function('f')
+    assert re(f(x)).diff(x) == re(f(x).diff(x))
+    assert im(f(x)).diff(x) == im(f(x).diff(x))
+
+    x = Symbol('x', real=True)
+    assert abs(f(x)).diff(x).subs(f(x), 1+I*x) == x/sqrt(1 + x**2)
+    assert arg(f(x)).diff(x).subs(f(x), 1+I*x**2) == 2*x/(1+x**4)
