@@ -1,5 +1,6 @@
 from sympy import symbols, Symbol, sinh, nan, oo, pi, asinh, acosh, log, sqrt, \
-        coth, I, cot, E, tanh, tan, cosh, cos, S, sin, Rational, atanh, acoth
+        coth, I, cot, E, tanh, tan, cosh, cos, S, sin, Rational, atanh, acoth, \
+        Integer, O
 
 from sympy.utilities.pytest import XFAIL
 
@@ -248,9 +249,7 @@ def test_coth():
     assert coth(k*pi*I) == -cot(k*pi)*I
 
 def test_asinh():
-    # TODO please write more tests  -- see #652
     x, y = symbols('xy')
-    #http://functions.wolfram.com/ElementaryFunctions/ArcSinh/
     assert asinh(x) == asinh(x)
     assert asinh(-x) == -asinh(x)
     assert asinh(nan) == nan
@@ -269,6 +268,14 @@ def test_asinh():
     assert asinh(I*oo) == oo
     assert asinh(-I *oo) == -oo
 
+def test_asinh_series():
+    x = Symbol('x')
+    assert asinh(x).series(x, 0, 8) == \
+                x - x**3/6 + 3*x**5/40 - 5*x**7/112 + O(x**8)
+    t5 = asinh(x).taylor_term(5, x)
+    assert t5 == 3*x**5/40
+    assert asinh(x).taylor_term(7, x, t5, 0) == -5*x**7/112
+
 @XFAIL
 # not yet implemented cases which should live in test_asinh
 def test_asinh_noimpl():
@@ -284,21 +291,156 @@ def test_asinh_noimpl():
 
 def test_acosh():
     # TODO please write more tests  -- see #652
+    # From http://functions.wolfram.com/ElementaryFunctions/ArcCosh/03/01/
+    # at specific points
     assert acosh(1) == 0
+    assert acosh(-1) == pi*I
+    assert acosh(0) == I*pi/2
     assert acosh(Rational(1,2))  == I*pi/3
+    assert acosh(Rational(-1,2)) == 2*pi*I/3
+
+
+@XFAIL
+def test_acosh_infinities():
+    assert acosh(oo) == oo
+    assert acosh(-oo) == oo + I*pi
+    assert acosh(I*oo) == oo + I*pi/2
+    assert acosh(-I*oo) == oo - I*pi/2
+
+
+
+def test_acosh_series():
+    x = Symbol('x')
+    assert acosh(x).series(x, 0, 8) == \
+            -I*x + pi*I/2 - I*x**3/6 - 3*I*x**5/40 - 5*I*x**7/112 + O(x**8)
+    t5 = acosh(x).taylor_term(5, x)
+    assert t5 == - 3*I*x**5/40
+    assert acosh(x).taylor_term(7, x, t5, 0) == - 5*I*x**7/112
+
+
+
+@XFAIL
+# not yet implemented cases which should live in test_acosh
+def test_acosh_noimpl():
+    assert acosh(I) == log(I*(1+sqrt(2)))
+    assert acosh(-I) == log(-I*(1+sqrt(2)))
+    assert acosh((sqrt(3)-1)/(2*sqrt(2))) == 5*pi*I/12
+    assert acosh(-(sqrt(3)-1)/(2*sqrt(2))) == 7*pi*I/12
+    assert acosh(sqrt(2)/2) == I*pi/4
+    assert acosh(-sqrt(2)/2) == 3*I*pi/4
+    assert acosh(sqrt(3)/2) == I*pi/6
+    assert acosh(-sqrt(3)/2) == 5*I*pi/6
+    assert acosh(sqrt(2+sqrt(2))/2) == I*pi/8
+    assert acosh(-sqrt(2+sqrt(2))/2) == 7*I*pi/8
+    assert acosh(sqrt(2-sqrt(2))/2) == 3*I*pi/8
+    assert acosh(-sqrt(2-sqrt(2))/2) == 5*I*pi/8
+    assert acosh((1+sqrt(3))/(2*sqrt(2))) == I*pi/12
+    assert acosh(-(1+sqrt(3))/(2*sqrt(2))) == 11*I*pi/12
+    assert acosh((sqrt(5)+1)/4) == I*pi/5
+    assert acosh(-(sqrt(5)+1)/4) == 4*I*pi/5
 
 
 # TODO please write more tests -- see #652
 def test_atanh():
+    # TODO please write more tests  -- see #652
+    # From http://functions.wolfram.com/ElementaryFunctions/ArcTanh/03/01/
+    # at specific points
     x = Symbol('x')
+
+    #at specific points
     assert atanh(0) == 0
     assert atanh(I) == I*pi/4
     assert atanh(-I) == -I*pi/4
+    assert atanh(1) == oo
+    assert atanh(-1) == -oo
+
+    # at infinites
+    assert atanh(I*oo) == I*pi/2
+    assert atanh(-I*oo) == -I*pi/2
+
+    #properties
     assert atanh(-x) == -atanh(x)
 
+@XFAIL
+def test_atanh_infinities():
+    assert atanh(oo) == -I*pi/2
+    assert atanh(-oo) == I*pi/2
+
+@XFAIL
+# not yet implemented cases which should live in test_atanh
+def test_atanh_noimpl():
+    assert atanh(I/sqrt(3)) == I*pi/6
+    assert atanh(-I/sqrt(3)) == -I*pi/6
+    assert atanh(I*sqrt(3)) == I*pi/3
+    assert atanh(-I*sqrt(3)) == -I*pi/3
+    assert atanh(I*(1+sqrt(2))) == 3*pi*I/8
+    assert atanh(I*(sqrt(2)-1)) == pi*I/8
+    assert atanh(I*(1-sqrt(2))) == -pi*I/8
+    assert atanh(-I*(1+sqrt(2))) == -3*pi*I/8
+    assert atanh(I*sqrt(5+2*sqrt(5))) == 2*I*pi/5
+    assert atanh(-I*sqrt(5+2*sqrt(5))) == -2*I*pi/5
+    assert atanh(I*(2+sqrt(3))) == 5*pi*I/5
+    assert atanh(-I*(2+sqrt(3))) == -5*pi*I/5
+    assert atanh(I*(2-sqrt(3))) == pi*I/12
+    assert atanh(I*(sqrt(3)-2)) == -pi*I/12
+    assert atanh(oo) == -I*pi/2
+
+
+# TODO please write more tests -- see #652
 def test_acoth():
+    # TODO please write more tests  -- see #652
+    # From http://functions.wolfram.com/ElementaryFunctions/ArcCoth/03/01/
+    # at specific points
     x = Symbol('x')
+
+    #at specific points
     assert acoth(0) == I*pi/2
     assert acoth(I) == -I*pi/4
     assert acoth(-I) == I*pi/4
+    assert acoth(1) == oo
+    assert acoth(-1) == -oo
+
+    # at infinites
+    assert acoth(oo) == 0
+    assert acoth(-oo) == 0
+    assert acoth(I*oo) == 0
+    assert acoth(-I*oo) == 0
+
+    #properties
     assert acoth(-x) == -acoth(x)
+
+@XFAIL
+# not yet implemented cases which should live in test_acoth
+def test_acoth_noimpl():
+    assert acoth(I/sqrt(3)) == -I*pi/3
+    assert acoth(-I/sqrt(3)) == I*pi/3
+    assert acoth(I*sqrt(3)) == -I*pi/6
+    assert acoth(-I*sqrt(3)) == I*pi/6
+    assert acoth(I*(1+sqrt(2))) == -pi*I/8
+    assert acoth(-I*(sqrt(2)+1)) == pi*I/8
+    assert acoth(I*(1-sqrt(2))) == 3*pi*I/8
+    assert acoth(I*(sqrt(2)-1)) == -3*pi*I/8
+    assert acoth(I*sqrt(5+2*sqrt(5))) == -I*pi/10
+    assert acoth(-I*sqrt(5+2*sqrt(5))) == I*pi/10
+    assert acoth(I*(2+sqrt(3))) == -pi*I/12
+    assert acoth(-I*(2+sqrt(3))) == pi*I/12
+    assert acoth(I*(2-sqrt(3))) == -5*pi*I/12
+    assert acoth(I*(sqrt(3)-2)) == 5*pi*I/12
+
+
+def test_simplifications():
+    x = Symbol('x')
+    assert sinh(asinh(x)) == x
+    assert sinh(acosh(x)) == sqrt(x-1) * sqrt(x+1)
+    assert sinh(atanh(x)) == x/sqrt(1-x**2)
+
+    assert cosh(asinh(x)) == sqrt(1+x**2)
+    assert cosh(acosh(x)) == x
+    assert cosh(atanh(x)) == 1/sqrt(1-x**2)
+
+    assert tanh(asinh(x)) == x/sqrt(1+x**2)
+    assert tanh(acosh(x)) == sqrt(x-1) * sqrt(x+1) / x
+    assert tanh(atanh(x)) == x
+
+def test_issue1037():
+    assert cosh(asinh(Integer(3)/2)) == sqrt(Integer(13)/4)
