@@ -1,6 +1,6 @@
 from sympy import symbols, Symbol, nan, oo, I, sinh, sin, acot, pi, atan, \
         acos, Rational, sqrt, asin, acot, cot, coth, E, S, tan, tanh, cos, \
-        cosh, atan2
+        cosh, atan2, exp, asinh, acoth, atanh, O
 
 def test_sin():
     x, y = symbols('xy')
@@ -13,20 +13,18 @@ def test_sin():
 
     assert sin(oo*I) == oo*I
     assert sin(-oo*I) == -oo*I
+    assert sin(oo).args[0] == oo
 
     assert sin(0) == 0
 
-    assert sin(1) == sin(1)
-    assert sin(-1) == -sin(1)
-
-    assert sin(x) == sin(x)
-    assert sin(-x) == -sin(x)
+    assert sin(asin(x)) == x
+    assert sin(atan(x)) == x / sqrt(1 + x**2)
+    assert sin(acos(x)) == sqrt(1 - x**2)
+    assert sin(acot(x)) == 1 / (sqrt(1 + 1 / x**2) * x)
 
     assert sin(pi*I) == sinh(pi)*I
     assert sin(-pi*I) == -sinh(pi)*I
-
-    assert sin(2**1024 * E) == sin(2**1024 * E)
-    assert sin(-2**1024 * E) == -sin(2**1024 * E)
+    assert sin(-2*I) == -sinh(2)*I
 
     assert sin(pi) == 0
     assert sin(-pi) == 0
@@ -53,10 +51,20 @@ def test_sin():
     assert sin(7*pi/6) == -S.Half
     assert sin(-5*pi/6) == -S.Half
 
-    assert sin(pi/105) == sin(pi/105)
-    assert sin(-pi/105) == -sin(pi/105)
+    assert sin(1*pi/5) == sqrt((5 - sqrt(5)) / 8)
+    assert sin(2*pi/5) == sqrt((5 + sqrt(5)) / 8)
+    assert sin(3*pi/5) == sin(2*pi/5)
+    assert sin(4*pi/5) == sin(1*pi/5)
+    assert sin(6*pi/5) == -sin(1*pi/5)
+    assert sin(8*pi/5) == -sin(2*pi/5)
 
-    assert sin(2 + 3*I) == sin(2 + 3*I)
+    assert sin(-1273*pi/5) == -sin(2*pi/5)
+
+    assert sin(104*pi/105) == sin(pi/105)
+    assert sin(106*pi/105) == -sin(pi/105)
+
+    assert sin(-104*pi/105) == -sin(pi/105)
+    assert sin(-106*pi/105) == sin(pi/105)
 
     assert sin(x*I) == sinh(x)*I
 
@@ -67,6 +75,7 @@ def test_sin():
 
     assert sin(r).is_real == True
 
+
 def test_cos():
     x, y = symbols('xy')
 
@@ -81,17 +90,14 @@ def test_cos():
 
     assert cos(0) == 1
 
-    assert cos(1) == cos(1)
-    assert cos(-1) == cos(1)
-
-    assert cos(x) == cos(x)
-    assert cos(-x) == cos(x)
+    assert cos(acos(x)) == x
+    assert cos(atan(x)) == 1 / sqrt(1 + x**2)
+    assert cos(asin(x)) == sqrt(1 - x**2)
+    assert cos(acot(x)) == 1 / sqrt(1 + 1 / x**2)
 
     assert cos(pi*I) == cosh(pi)
     assert cos(-pi*I) == cosh(pi)
-
-    assert cos(2**1024 * E) == cos(2**1024 * E)
-    assert cos(-2**1024 * E) == cos(2**1024 * E)
+    assert cos(-2*I) == cosh(2)
 
     assert cos(pi/2) == 0
     assert cos(-pi/2) == 0
@@ -119,16 +125,22 @@ def test_cos():
     assert cos(7*pi/6) == -S.Half*sqrt(3)
     assert cos(-5*pi/6) == -S.Half*sqrt(3)
 
-    assert cos(pi/105) == cos(pi/105)
-    assert cos(-pi/105) == cos(pi/105)
+    assert cos(1*pi/5) == (sqrt(5) + 1)/4
+    assert cos(2*pi/5) == (sqrt(5) - 1)/4
+    assert cos(3*pi/5) == -cos(2*pi/5)
+    assert cos(4*pi/5) == -cos(1*pi/5)
+    assert cos(6*pi/5) == -cos(1*pi/5)
+    assert cos(8*pi/5) == cos(2*pi/5)
 
-    assert cos(2 + 3*I) == cos(2 + 3*I)
+    assert cos(-1273*pi/5) == -cos(2*pi/5)
+
+    assert cos(104*pi/105) == -cos(pi/105)
+    assert cos(106*pi/105) == -cos(pi/105)
+
+    assert cos(-104*pi/105) == -cos(pi/105)
+    assert cos(-106*pi/105) == -cos(pi/105)
 
     assert cos(x*I) == cosh(x)
-
-    assert cos(k*pi) == cos(k*pi)
-    assert cos(17*k*pi) == cos(17*k*pi)
-
     assert cos(k*pi*I) == cosh(k*pi)
 
     assert cos(r).is_real == True
@@ -147,29 +159,20 @@ def test_tan():
 
     assert tan(0) == 0
 
-    assert tan(1) == tan(1)
-    assert tan(-1) == -tan(1)
-
-    assert tan(x) == tan(x)
-    assert tan(-x) == -tan(x)
+    assert tan(atan(x)) == x
+    assert tan(asin(x)) == x / sqrt(1 - x**2)
+    assert tan(acos(x)) == sqrt(1 - x**2) / x
+    assert tan(acot(x)) == 1 / x
 
     assert tan(pi*I) == tanh(pi)*I
     assert tan(-pi*I) == -tanh(pi)*I
-
-    assert tan(2**1024 * E) == tan(2**1024 * E)
-    assert tan(-2**1024 * E) == -tan(2**1024 * E)
+    assert tan(-2*I) == -tanh(2)*I
 
     assert tan(pi) == 0
     assert tan(-pi) == 0
     assert tan(2*pi) == 0
     assert tan(-2*pi) == 0
     assert tan(-3*10**73*pi) == 0
-    assert tan(7*10**103*pi) == 0
-
-    assert tan(pi/2) == tan(pi/2)
-    assert tan(-pi/2) == -tan(pi/2)
-    assert tan(5*pi/2) == tan(5*pi/2)
-    assert tan(7*pi/2) == tan(7*pi/2)
 
     assert tan(pi/3) == sqrt(3)
     assert tan(-2*pi/3) == sqrt(3)
@@ -184,11 +187,6 @@ def test_tan():
     assert tan(7*pi/6) == 1/sqrt(3)
     assert tan(-5*pi/6) == 1/sqrt(3)
 
-    assert tan(pi/105) == tan(pi/105)
-    assert tan(-pi/105) == -tan(pi/105)
-
-    assert tan(2 + 3*I) == tan(2 + 3*I)
-
     assert tan(x*I) == tanh(x)*I
 
     assert tan(k*pi) == 0
@@ -198,6 +196,15 @@ def test_tan():
 
     assert tan(r).is_real == True
 
+def test_tan_rewrite():
+    x = Symbol('x')
+    neg_exp, pos_exp = exp(-x*I), exp(x*I)
+    assert tan(x).rewrite(exp) == I*(neg_exp-pos_exp)/(neg_exp+pos_exp)
+    assert tan(x).rewrite(sin) == 2*sin(x)**2/sin(2*x)
+    assert tan(x).rewrite(cos) == -cos(x + S.Pi/2)/cos(x)
+    assert tan(x).rewrite(cot) == 1/cot(x)
+
+
 def test_cot():
     x, y = symbols('xy')
 
@@ -210,26 +217,14 @@ def test_cot():
     assert cot(oo*I) == -I
     assert cot(-oo*I) == I
 
-    assert cot(0) == cot(0)
-
-    assert cot(1) == cot(1)
-    assert cot(-1) == -cot(1)
-
-    assert cot(x) == cot(x)
-    assert cot(-x) == -cot(x)
+    assert cot(acot(x)) == x
+    assert cot(atan(x)) == 1 / x
+    assert cot(asin(x)) == sqrt(1 - x**2) / x
+    assert cot(acos(x)) == x / sqrt(1 - x**2)
 
     assert cot(pi*I) == -coth(pi)*I
     assert cot(-pi*I) == coth(pi)*I
-
-    assert cot(2**1024 * E) == cot(2**1024 * E)
-    assert cot(-2**1024 * E) == -cot(2**1024 * E)
-
-    assert cot(pi) == cot(pi)
-    assert cot(-pi) == -cot(pi)
-    assert cot(2*pi) == cot(2*pi)
-    assert cot(-2*pi) == -cot(2*pi)
-    assert cot(-3*10**73*pi) == -cot(3*10**73*pi)
-    assert cot(7*10**103*pi) == cot(7*10**103*pi)
+    assert cot(-2*I) == coth(2)*I
 
     assert cot(pi/2) == 0
     assert cot(-pi/2) == 0
@@ -249,24 +244,27 @@ def test_cot():
     assert cot(7*pi/6) == sqrt(3)
     assert cot(-5*pi/6) == sqrt(3)
 
-    assert cot(pi/105) == cot(pi/105)
-    assert cot(-pi/105) == -cot(pi/105)
-
-    assert cot(2 + 3*I) == cot(2 + 3*I)
-
     assert cot(x*I) == -coth(x)*I
-
-    assert cot(k*pi) == cot(k*pi)
-    assert cot(17*k*pi) == cot(17*k*pi)
-
     assert cot(k*pi*I) == -coth(k*pi)*I
 
     assert cot(r).is_real == True
 
-# TODO write me
+def test_cot_rewrite():
+    x = Symbol('x')
+    neg_exp, pos_exp = exp(-x*I), exp(x*I)
+    assert cot(x).rewrite(exp) == I*(pos_exp+neg_exp)/(pos_exp-neg_exp)
+    assert cot(x).rewrite(sin) == 2*sin(2*x)/sin(x)**2
+    assert cot(x).rewrite(cos) == -cos(x)/cos(x + S.Pi/2)
+    assert cot(x).rewrite(tan) == 1/tan(x)
+
 def test_asin():
     x = Symbol('x')
 
+    assert asin(nan) == nan
+
+    assert asin(oo) == -I*oo
+    assert asin(-oo) == I*oo
+
     assert asin(0)  == 0
     assert asin(Rational(1,2)) == pi/6
     assert asin(1)  == pi/2
@@ -277,28 +275,53 @@ def test_asin():
     assert asin(0.2).is_real == True
     assert asin(-2).is_real == False
 
-# TODO write me
+    assert asin(-2*I) == -I*asinh(2)
+
+def test_asin_series():
+    x = Symbol('x')
+    assert asin(x).series(x, 0, 9) == \
+                    x + x**3/6 + 3*x**5/40 + 5*x**7/112 + O(x**9)
+    t5 = asin(x).taylor_term(5, x)
+    assert t5 == 3*x**5/40
+    assert asin(x).taylor_term(7, x, t5, 0) == 5*x**7/112
+
 def test_acos():
     x = Symbol('x')
-
     r = Symbol('r', real=True)
 
+    assert acos(nan) == nan
+    assert acos(oo) == I*oo
+    assert acos(-oo) == -I*oo
+
     assert acos(0)  == pi/2
     assert acos(Rational(1,2)) == pi/3
     assert acos(1)  == 0
+    assert acos(-1) == pi
     assert acos(sqrt(2)/2) == pi/4
     assert acos(x).diff(x) == -1/sqrt(1-x**2)
 
     assert acos(0.2).is_real == True
     assert acos(-2).is_real == False
 
+def test_acos_series():
+    x = Symbol('x')
+    assert acos(x).series(x, 0, 8) == \
+            pi/2 - x - x**3/6 - 3*x**5/40 - 5*x**7/112 + O(x**8)
+    assert acos(x).series(x, 0, 8) == pi/2 - asin(x).series(x, 0, 8)
+    t5 = acos(x).taylor_term(5, x)
+    assert t5 == -3*x**5/40
+    assert acos(x).taylor_term(7, x, t5, 0) == -5*x**7/112
 
-# TODO write me
 def test_atan():
     x = Symbol('x')
 
     r = Symbol('r', real=True)
 
+    assert atan(nan) == nan
+
+    assert atan(oo) == pi/2
+    assert atan(-oo) == -pi/2
+
     assert atan(0)  == 0
     assert atan(1)  == pi/4
     assert atan(sqrt(3)) == pi/3
@@ -307,6 +330,8 @@ def test_atan():
 
     assert atan(r).is_real == True
 
+    assert atan(-2*I) == -I*atanh(2)
+
 def test_atan2():
     assert atan2(0, 0) == S.NaN
     assert atan2(0, 1) == 0
@@ -315,23 +340,78 @@ def test_atan2():
     assert atan2(-1, 1) == -pi/4
     assert atan2(0, -1) == pi
 
-# TODO write me
 def test_acot():
     x = Symbol('x')
 
     r = Symbol('r', real=True)
 
+    assert acot(nan) == nan
+
+    assert acot(-oo) == 0
     assert acot(oo) == 0
     assert acot(1)  == pi/4
     assert acot(0)  == pi/2
+    assert acot(sqrt(3)/3) == pi/3
+    assert acot(1/sqrt(3)) == pi/3
+    assert acot(-1/sqrt(3)) == -pi/3
     assert acot(x).diff(x) == -1/(1+x**2)
 
     assert acot(r).is_real == True
 
-
+    assert acot(I*pi) == -I*acoth(pi)
+    assert acot(-2*I) == I*acoth(2)
 
 def test_attributes():
     x = Symbol('x')
-    assert sin(x).args[:] == (x,)
-    assert sin(x).args[0] != sin
-    assert sin(x).args[0] == x
+    assert sin(x).args == (x,)
+
+def test_sincos_rewrite():
+    x = Symbol("x")
+    y = Symbol("y")
+    assert sin(pi/2-x) == cos(x)
+    assert sin(pi-x) == sin(x)
+    assert cos(pi/2-x) == sin(x)
+    assert cos(pi-x) == -cos(x)
+
+def _check_even_rewrite(func, arg):
+    """Checks that the expr has been rewritten using f(-x) -> f(x)
+    arg : -x
+    """
+    return func(arg).args[0] == -arg
+
+def _check_odd_rewrite(func, arg):
+    """Checks that the expr has been rewritten using f(-x) -> -f(x)
+    arg : -x
+    """
+    return func(arg).func.is_Mul
+
+def _check_no_rewrite(func, arg):
+    """Checks that the expr is not rewritten"""
+    return func(arg).args[0] == arg
+
+def test_evenodd_rewrite():
+    x, y = symbols('xy')
+    a = cos(2) #negative
+    b = sin(1) #positive
+    even = [cos]
+    odd = [sin, tan, cot, asin, atan, acot]
+    with_minus = [-1, -2**1024 * E, -pi/105, -x*y, -x-y]
+    for func in even:
+        for expr in with_minus:
+            assert _check_even_rewrite(func, expr)
+        assert _check_no_rewrite(func, a*b)
+        assert func(x-y) == func(y-x)   #it doesn't matter which form is canonical
+    for func in odd:
+        for expr in with_minus:
+            assert _check_odd_rewrite(func, expr)
+        assert _check_no_rewrite(func, a*b)
+        assert func(x-y) == -func(y-x)  #it doesn't matter which form is canonical
+
+def test_issue1448():
+    x = Symbol('x')
+    assert cot(x).inverse() == acot
+    assert sin(x).rewrite(cot) == 2*cot(x/2)/(1 + cot(x/2)**2)
+    assert cos(x).rewrite(cot) == -(1 - cot(x/2)**2)/(1 + cot(x/2)**2)
+    assert tan(x).rewrite(cot) == 1/cot(x)
+    assert cot(x).fdiff() == -1 - cot(x)**2
+
