@@ -1,8 +1,8 @@
-
 from sympy.core.basic import Basic, S, C, sympify
 from sympy.core.function import Function
 from sympy.functions.elementary.miscellaneous import sqrt
 from sympy.core.cache import cacheit
+from sympy.utilities.decorator import deprecated
 
 ###############################################################################
 ################################ ERROR FUNCTION ###############################
@@ -19,11 +19,7 @@ class erf(Function):
             raise ArgumentIndexError(self, argindex)
 
     @classmethod
-    def _eval_apply_subs(cls, *args):
-        return
-
-    @classmethod
-    def canonize(cls, arg):
+    def eval(cls, arg):
         if arg.is_Number:
             if arg is S.NaN:
                 return S.NaN
