@@ -1,4 +1,3 @@
-
 from sympy.core import Basic, Add, S, C, sympify
 from sympy.core.function import Function
 from zeta_functions import zeta
@@ -21,11 +20,6 @@ class gamma(Function):
         else:
             raise ArgumentIndexError(self, argindex)
 
-    @classmethod
-    @deprecated
-    def canonize(cls, arg):
-        return cls.eval(arg)
-
     @classmethod
     def eval(cls, arg):
         if arg.is_Number:
@@ -61,11 +55,14 @@ class gamma(Function):
                         return 2**n*sqrt(S.Pi) / coeff
 
 
-    def _eval_expand_func(self, *args):
-        arg = self.args[0].expand()
+    def _eval_expand_func(self, deep=True, **hints):
+        if deep:
+            arg = self.args[0].expand(deep, **hints)
+        else:
+            arg = self.args[0]
 
         if arg.is_Add:
-            for i, coeff in enumerate(arg.args[:]):
+            for i, coeff in enumerate(arg.args):
                 if arg.args[i].is_Number:
                     terms = C.Add(*(arg.args[:i] + arg.args[i+1:]))
 
@@ -93,11 +90,6 @@ class lowergamma(Function):
 
     nargs = 2
 
-    @classmethod
-    @deprecated
-    def canonize(cls, a, x):
-        return cls.eval(a, x)
-
     @classmethod
     def eval(cls, a, x):
         if a.is_Number:
@@ -122,11 +114,6 @@ class uppergamma(Function):
         else:
             raise ArgumentIndexError(self, argindex)
 
-    @classmethod
-    @deprecated
-    def canonize(cls, a, z):
-        return cls.eval(a, z)
-
     @classmethod
     def eval(cls, a, z):
         if z.is_Number:
@@ -163,10 +150,6 @@ class polygamma(Function):
         else:
             raise ArgumentIndexError(self, argindex)
 
-    @classmethod
-    def canonize(cls, n, z):
-        return cls.eval(n, z)
-
     @classmethod
     def eval(cls, n, z):
         n, z = map(sympify, (n, z))
@@ -194,8 +177,13 @@ class polygamma(Function):
                                 return (-1)**(n+1)*C.Factorial(n)*zeta(n+1, z)
 
 
-    def _eval_expand_func(self, *args):
-        n, z = self.args[0], self.args[1].expand(func=True)
+    def _eval_expand_func(self, deep=True, **hints):
+        if deep:
+            hints['func'] = False
+            n = self.args[0].expand(deep, **hints)
+            z = self.args[1].expand(deep, **hints)
+        else:
+            n, z = self.args[0], self.args[1].expand(deep, func=True)
 
         if n.is_Integer and n.is_nonnegative:
             if z.is_Add:
