@@ -19,6 +19,7 @@ class Ellipse(GeometryEntity):
 
     Example:
     ========
+        >>> from sympy.geometry import Ellipse, Point
         >>> e = Ellipse(Point(0, 0), 5, 1)
         >>> e.hradius, e.vradius
         (5, 1)
@@ -91,7 +92,7 @@ class Ellipse(GeometryEntity):
 
         hr, vr = self.hradius, self.vradius
         if hr.atoms(C.Symbol) or vr.atoms(C.Symbol):
-            raise Exception("foci can only be determined on non-symbolic radii")
+            raise ValueError("foci can only be determined on non-symbolic radii")
 
         v = sqrt(abs(vr**2 - hr**2))
         if hr < vr:
@@ -276,17 +277,19 @@ class Ellipse(GeometryEntity):
 
 class Circle(Ellipse):
     """
-    A circle in space. Consturcted simply from a center and a radius, or
+    A circle in space. Constructed simply from a center and a radius, or
     from three non-collinear points.
 
     Example:
     ========
+        >>> from sympy.geometry import Point, Circle
         >>> c1 = Circle(Point(0, 0), 5)
         >>> c1.hradius, c1.vradius, c1.radius
         (5, 5, 5)
         >>> c2 = Circle(Point(0, 0), Point(1, 1), Point(1, 0))
         >>> c2.hradius, c2.vradius, c2.radius, c2.center
         (2**(1/2)/2, 2**(1/2)/2, 2**(1/2)/2, Point(1/2, 1/2))
+
     """
     def __new__(cls, *args, **kwargs):
         c, r = None, None
