@@ -1,4 +1,3 @@
-
 # How entities are ordered; used by __cmp__ in GeometryEntity
 ordering_of_classes = [
     "Point",
@@ -9,7 +8,8 @@ ordering_of_classes = [
     "RegularPolygon",
     "Polygon",
     "Circle",
-    "Ellipse"
+    "Ellipse",
+    "Curve"
 ]
 
 class GeometryEntity(tuple):
@@ -18,6 +18,9 @@ class GeometryEntity(tuple):
     def __new__(cls, *args, **kwargs):
         return tuple.__new__(cls, args)
 
+    def __getnewargs__(self):
+        return tuple(self)
+
     @staticmethod
     def do_intersection(e1, e2):
         """
