@@ -48,7 +48,7 @@ class LinearEntity(GeometryEntity):
         ===========================
             Simply take the first two lines and find their intersection.
             If there is no intersection, then the first two lines were
-            parallel and had no intersection so concurrenecy is impossible
+            parallel and had no intersection so concurrency is impossible
             amongst the whole set. Otherwise, check to see if the
             intersection point of the first two lines is a member on
             the rest of the lines. If so, the lines are concurrent.
@@ -284,7 +284,7 @@ class LinearEntity(GeometryEntity):
             px = simplify((b1*c2 - c1*b2) / t)
             py = simplify((a2*c1 - a1*c2) / t)
             inter = Point(px, py)
-            if inter in self and inter in o:
+            if (inter in self) and (inter in o):
                 return [inter]
             return []
         raise NotImplementedError()
@@ -332,7 +332,30 @@ class LinearEntity(GeometryEntity):
 
 
 class Line(LinearEntity):
-    """A line in space."""
+    """A line in space.
+
+    A line is declared with two distinct points.
+
+    Note:
+    At the moment only lines in a 2D space can be declared, because
+    Points can be defined only for 2D spaces.
+
+    *Example*
+
+        >>> import sympy
+        >>> from sympy import Point
+        >>> from sympy.abc import L
+        >>> from sympy.geometry import Line
+        >>> L = Line(Point(2,3), Point(3,5))
+        >>> L
+        Line(Point(2, 3), Point(3, 5))
+        >>> L.points
+        (Point(2, 3), Point(3, 5))
+        >>> L.equation()
+        1 + y - 2*x
+        >>> L.coefficients
+        (-2, 1, 1)
+    """
 
     def arbitrary_point(self, parameter_name='t'):
         """Returns a symbolic point that is on this line."""
@@ -377,11 +400,43 @@ class Line(LinearEntity):
 
 
 class Ray(LinearEntity):
-    """A ray in space."""
+    """
+    A ray is a semi-line in the space. It starts at one point and
+    propagates in one unique direction.
+
+    A ray is declared with two distinct points: the first point is the source,
+    whereas the second point lies on the semi-line. Therefore, the second
+    point determines the direction to which the semi-line propagates.
+
+    Note:
+    At the moment only rays in a 2D space can be declared, because
+    Points can be defined only for 2D spaces.
+
+    *Example*
+
+        >>> import sympy
+        >>> from sympy import Point
+        >>> from sympy.abc import r
+        >>> from sympy.geometry import Ray
+        >>> r = Ray(Point(2, 3), Point(3, 5))
+        >>> r = Ray(Point(2, 3), Point(3, 5))
+        >>> r
+        Ray(Point(2, 3), Point(3, 5))
+        >>> r.points
+        (Point(2, 3), Point(3, 5))
+        >>> r.source
+        Point(2, 3)
+        >>> r.xdirection
+        oo
+        >>> r.ydirection
+        oo
+        >>> r.slope
+        2
+    """
 
     @property
     def source(self):
-        """The point from which the ray eminates."""
+        """The point from which the ray emanates."""
         return self.p1
 
     @property
@@ -452,7 +507,32 @@ class Ray(LinearEntity):
 
 
 class Segment(LinearEntity):
-    """An undirected line segment in space."""
+    """An undirected line segment in space.
+
+    A segment is declared with two distinct points.
+
+    Note:
+    At the moment only segments in a 2D space can be declared, because
+    Points can be defined only for 2D spaces.
+
+    *Example*
+
+        >>> import sympy
+        >>> from sympy import Point
+        >>> from sympy.abc import s
+        >>> from sympy.geometry import Segment
+        >>> s = Segment(Point(4, 3), Point(1, 1))
+        >>> s
+        Segment(Point(1, 1), Point(4, 3))
+        >>> s.points
+        (Point(1, 1), Point(4, 3))
+        >>> s.slope
+        2/3
+        >>> s.length
+        13**(1/2)
+        >>> s.midpoint
+        Point(5/2, 2)
+    """
 
     def __new__(cls, p1, p2, **kwargs):
         # Reorder the two points under the following ordering:
@@ -511,13 +591,9 @@ class Segment(LinearEntity):
             return ((o.p1 in self) and (o.p2 in self))
         elif isinstance(o, Point):
             if Point.is_collinear(self.p1, self.p2, o):
-                x1,x2 = self.p1[0], self.p2[0]
-                if not (x1.atoms(C.Symbol)) or (x2.atoms(C.Symbol)):
-                    return (min(x1,x2) <= o[0]) and (o[0] <= max(x1,x2))
-                else:
+                d = self.length
+                if (Point.distance(self.p1,o) <= d) and (Point.distance(self.p2,o) <= d):
                     return True
-            else:
-                return False
 
         # No other known entity can be contained in a Ray
         return False
