@@ -1,4 +1,3 @@
-
 def intersection(*entities):
     """
     Finds the intersection between a list GeometryEntity instances. Returns a
@@ -7,7 +6,7 @@ def intersection(*entities):
 
     Examples:
     =========
-        >>> from sympy.geometry import *
+        >>> from sympy.geometry import Point, Line, Circle, intersection
         >>> p1,p2,p3 = Point(0,0), Point(1,1), Point(-1, 5)
         >>> l1, l2 = Line(p1, p2), Line(p3, p2)
         >>> c = Circle(p2, 1)
@@ -31,6 +30,8 @@ def intersection(*entities):
         - It is possible for intersection() to miss intersections that one
           knows exists because the required quantities were not fully
           simplified internally.
+        - Reals should be converted to Rationals, e.g. Rational(str(real_num))
+          or else failures due to floating point issues may result.
     """
     from entity import GeometryEntity
 
@@ -56,10 +57,10 @@ def convex_hull(*args):
 
     Example:
     ========
-        >>> from sympy.geometry import Point
+        >>> from sympy.geometry import Point, convex_hull
         >>> points = [ Point(x) for x in [(1,1), (1,2), (3,1), (-5,2), (15,4)] ]
         >>> convex_hull(points)
-        Polygon(Point(3, 1), Point(15, 4), Point(-5, 2), Point(1, 1))
+        Polygon(Point(-5, 2), Point(1, 1), Point(3, 1), Point(15, 4))
 
     Description of method used:
     ===========================
@@ -69,71 +70,53 @@ def convex_hull(*args):
     from line import Segment
     from polygon import Polygon
 
+    def uniquify(a):
+        ret = {}
+        # not order preserving
+        map(ret.__setitem__, a, [])
+        return ret.keys()
+
     p = args[0]
     if isinstance(p, Point):
-        p = args
+        p = uniquify(args)
 
-    # Basic checks
-    if len(p) == 1:
+    if len(p) is 1:
         return p[0]
-    elif len(p) == 2:
-        return Segment(p[0], p[1])
+    else:
+        if len(p) is 2:
+            return Segment(p[0],p[1])
 
-    # Find lowest+rightmost point
-    m = 0
-    for i in xrange(1, len(p)):
-        if (p[i][1] < p[m][1]) or ((p[i][1] == p[m][1]) and (p[i][0] > p[m][0])):
-            m = i
-    p[0], p[m] = p[m], p[0]
 
-    def tarea(a, b, c):
-        return (b[0] - a[0])*(c[1] - a[1]) - (c[0] - a[0])*(b[1] - a[1])
 
-    # Radial sort of points with respect to p[0] (our pivot)
-    destroy = {}
-    p0 = p[0]
-    def pcompare(p1, p2):
-        a = tarea(p0, p1, p2)
-        if a > 0:
-            return -1
-        elif a < 0:
-            return 1
-        else:
-            x = abs(p1[0] - p0[0]) - abs(p2[0] - p0[0])
-            y = abs(p1[1] - p0[1]) - abs(p2[1] - p0[1])
-            if (x < 0) or (y < 0):
-                destroy[p1] = True
-                return -1
-            elif (x > 0) or (y > 0):
-                destroy[p2] = True
-                return 1
-            else:
-                destroy[p1] = True
-                return 0
-    p = p[1:]
-    p.sort(pcompare)
-    p.insert(0, p0)
+    def orientation(p,q,r):
+        '''Return positive if p-q-r are clockwise, neg if ccw, zero if colinear.'''
+        return (q[1]-p[1])*(r[0]-p[0]) - (q[0]-p[0])*(r[1]-p[1])
+
+
+    '''scan to find upper and lower convex hulls of a set of 2d points.'''
+    U = []
+    L = []
+    #print "[graham] points:",Points
+    p.sort()
+    for p_i in p:
+        while len(U) > 1 and orientation(U[-2],U[-1],p_i) <= 0: U.pop()
+        while len(L) > 1 and orientation(L[-2],L[-1],p_i) >= 0: L.pop()
+        U.append(p_i)
+        L.append(p_i)        #print "[graham] result:", U,L
+    U.reverse()
+    convexHull = tuple(L+U[1:-1])
+
+    #convexHull = uniquify(convexHull)
+    #print "U:",U.reverse()
+    #print "L:",L
+    #print "ch(",len(convexHull),"):",convexHull
+    if len(convexHull) is 2:
+        return Segment(convexHull[0],convexHull[1])
+    return Polygon(convexHull)
+
 
-    # Destroy points as found by sorting
-    for i in xrange(len(p)-1, -1, -1):
-        if p[i] in destroy:
-            del p[i]
 
-    # Graham scan
-    def isleft(a, b, c):
-        return (tarea(a, b, c) > 0)
 
-    top = [p[0], p[1]]
-    i = 2
-    while i < len(p):
-        p1 = top[-2]
-        p2 = top[-1]
-        if isleft(p1, p2, p[i]):
-            top.append(p[i])
-            i += 1
-        else:
-            top.pop()
-    return Polygon(top)
 
 def are_similar(e1, e2):
     """
