@@ -1,5 +1,5 @@
 from sympy import Rational, sqrt, symbols, sin, exp, log, sinh, cosh, cos, pi, \
-        I, S, erf, tan, asin, asinh, Function, Derivative, diff, trim
+        I, S, erf, tan, asin, asinh, acos, acosh, Function, Derivative, diff, trim
 from sympy.integrals.risch import heurisch, components
 from sympy.utilities.pytest import XFAIL, skip
 
@@ -34,7 +34,7 @@ def test_heurisch_fractions():
     assert heurisch(1/(2 + x), x) == log(x + 2)
     assert heurisch(1/(x+sin(y)), x) == log(x+sin(y))
 
-    # Up to a constant, where C = 5*pi*I/12, Matematica gives identical
+    # Up to a constant, where C = 5*pi*I/12, Mathematica gives identical
     # result in the first case. The difference is because sympy changes
     # signs of expressions without any care.
     # XXX ^ ^ ^ is this still correct?
@@ -46,7 +46,7 @@ def test_heurisch_fractions():
 
 def test_heurisch_log():
     assert heurisch(log(x), x) == x*log(x) - x
-    assert heurisch(log(3*x), x) == x*log(3*x) - x
+    assert heurisch(log(3*x), x) == -x + x*log(3) + x*log(x)
     assert heurisch(log(x**2), x) in [x*log(x**2) - 2*x, 2*x*log(x) - 2*x]
 
 def test_heurisch_exp():
@@ -78,6 +78,9 @@ def test_heurisch_trigonometric():
     assert heurisch(x*sin(7*x), x) == sin(7*x) / 49 - x*cos(7*x) / 7
     assert heurisch(1/pi/4 * x**2*cos(x), x) == 1/pi/4*(x**2*sin(x) - 2*sin(x) + 2*x*cos(x))
 
+    assert heurisch(acos(x/4) * asin(x/4), x) == 2*x - ((16-x**2)**Rational(1,2))*asin(x/4) \
+        + ((16 - x**2)**Rational(1,2))*acos(x/4) + x*asin(x/4)*acos(x/4)
+
 def test_heurisch_hyperbolic():
     assert heurisch(sinh(x), x) == cosh(x)
     assert heurisch(cosh(x), x) == sinh(x)
@@ -85,6 +88,8 @@ def test_heurisch_hyperbolic():
     assert heurisch(x*sinh(x), x) == x*cosh(x) - sinh(x)
     assert heurisch(x*cosh(x), x) == x*sinh(x) - cosh(x)
 
+    assert heurisch(x*asinh(x/2), x) == x**2*asinh(x/2)/2 + asinh(x/2) - x*(4+x**2)**Rational(1,2)/4
+
 def test_heurisch_mixed():
     assert heurisch(sin(x)*exp(x), x) == exp(x)*sin(x)/2 - exp(x)*cos(x)/2
 
@@ -107,8 +112,10 @@ def test_heurisch_symbolic_coeffs():
     assert trim(diff(heurisch(log(x+y+z), y), y)) == log(x+y+z)
 
 def test_heurisch_symbolic_coeffs_1130():
-    assert heurisch(1/(x**2+y), x)      == I*y**(-S.Half)*log(x + (-y)**S.Half)/2 - \
-                                           I*y**(-S.Half)*log(x - (-y)**S.Half)/2
+    assert heurisch(1/(x**2+y), x) in [I*y**(-S.Half)*log(x + (-y)**S.Half)/2 - \
+    I*y**(-S.Half)*log(x - (-y)**S.Half)/2, I*log(x + I*y**Rational(1,2)) / \
+    (2*y**Rational(1,2)) - I*log(x - I*y**Rational(1,2))/(2*y**Rational(1,2))]
+
 
 def test_heurisch_hacking():
     assert heurisch(sqrt(1 + 7*x**2), x, hints=[]) == \
