@@ -1,6 +1,6 @@
 from sympy import symbols, Matrix, eye, I, Symbol, Rational, wronskian, cos, \
         sin, exp, hessian, sqrt, zeros, ones, randMatrix, Poly, S, pi, \
-        integrate, oo, raises, trigsimp
+        integrate, oo, raises, trigsimp, Integer, block_diag, N
 from sympy.matrices.matrices import ShapeError, MatrixError
 from sympy.printing import srepr
 from sympy.utilities.pytest import XFAIL
@@ -65,6 +65,7 @@ def test_power():
     assert A**1 == A
     assert (Matrix([[2]]) ** 100)[0,0] == 2**100
     assert eye(2)**10000000 == eye(2)
+    assert Matrix([[1, 2], [3, 4]])**Integer(2) == Matrix([[7, 10], [15, 22]])
 
 def test_creation():
     raises(MatrixError, 'Matrix(5,5,range(20))')
@@ -72,12 +73,12 @@ def test_creation():
     x = Symbol("x")
     a = Matrix([[x, 0], [0, 0]])
     m = a
-    assert m.cols == m.lines
+    assert m.cols == m.rows
     assert m.cols == 2
     assert m[:] == [x,0,0,0]
     b = Matrix(2,2, [x, 0, 0, 0])
     m = b
-    assert m.cols == m.lines
+    assert m.cols == m.rows
     assert m.cols == 2
     assert m[:] == [x,0,0,0]
 
@@ -281,8 +282,6 @@ def test_LUdecomp():
     P, L, Dee, U = M.LUdecompositionFF()
     assert P*M == L*Dee.inv()*U
 
-
-
 def test_LUsolve():
     A = Matrix([[2,3,5],
                 [3,6,2],
@@ -299,6 +298,31 @@ def test_LUsolve():
     soln = A.LUsolve(b)
     assert soln == x
 
+def test_QRsolve():
+    A = Matrix([[2,3,5],
+                [3,6,2],
+                [8,3,6]])
+    x = Matrix(3,1,[3,7,5])
+    b = A*x
+    soln = A.QRsolve(b)
+    assert soln == x
+    x = Matrix([[1,2],[3,4],[5,6]])
+    b = A*x
+    soln = A.QRsolve(b)
+    assert soln == x
+
+    A = Matrix([[0,-1,2],
+                [5,10,7],
+                [8,3,4]])
+    x = Matrix(3,1,[-1,2,5])
+    b = A*x
+    soln = A.QRsolve(b)
+    assert soln == x
+    x = Matrix([[7,8],[9,10],[11,12]])
+    b = A*x
+    soln = A.QRsolve(b)
+    assert soln == x
+
 def test_inverse():
     A = eye(4)
     assert A.inv() == eye(4)
@@ -353,7 +377,7 @@ def test_QR():
 
     A = Matrix([[1,1,1],[1,1,3],[2,3,4]])
     Q, R = A.QRdecomposition()
-    assert Q*Q.T == eye(Q.lines)
+    assert Q*Q.T == eye(Q.rows)
     assert R.is_upper()
     assert A == Q*R
 
@@ -380,9 +404,9 @@ def test_nullspace():
                 [-1,-3,0,1,0,9,3]])
     out, tmp = M.rref()
     assert out == Matrix([[1,3,0,0,2,0,0],
-                               [0,0,0,1,2,0,0],
-                               [0,0,0,0,0,1,R(1)/3],
-                               [0,0,0,0,0,0,0]])
+                          [0,0,0,1,2,0,0],
+                          [0,0,0,0,0,1,R(1)/3],
+                          [0,0,0,0,0,0,0]])
 
     # now check the vectors
     basis = M.nullspace()
@@ -391,15 +415,25 @@ def test_nullspace():
     assert basis[2] == Matrix([-2,0,0,-2,1,0,0])
     assert basis[3] == Matrix([0,0,0,0,0,R(-1)/3, 1])
 
+    # issue 1698; just see that we can do it when rows > cols
+    M = Matrix([[1,2],[2,4],[3,6]])
+    assert M.nullspace()
+
 def test_wronskian():
     x = Symbol('x')
     assert wronskian([cos(x), sin(x)], x) == cos(x)**2 + sin(x)**2
     assert wronskian([exp(x), exp(2*x)], x) == exp(3*x)
     assert wronskian([exp(x), x], x) == exp(x) - x*exp(x)
     assert wronskian([1, x, x**2], x) == 2
-    assert wronskian([exp(x), cos(x), x**3], x).expand() == \
-        -6*exp(x)*sin(x)*x + 6*cos(x)*exp(x)*x**2 - 6*exp(x)*cos(x)*x - \
+    w1 = -6*exp(x)*sin(x)*x + 6*cos(x)*exp(x)*x**2 - 6*exp(x)*cos(x)*x - \
         exp(x)*cos(x)*x**3 + exp(x)*sin(x)*x**3
+    assert wronskian([exp(x), cos(x), x**3], x).expand() == w1
+    assert wronskian([exp(x), cos(x), x**3], x, method='berkowitz').expand() == w1
+    w2 = -x**3*cos(x)**2 - x**3*sin(x)**2 - 6*x*cos(x)**2 - 6*x*sin(x)**2
+    assert wronskian([sin(x), cos(x), x**3], x).expand() == w2
+    assert wronskian([sin(x), cos(x), x**3], x, \
+        method='berkowitz').expand() == w2
+
 
 def canonicalize(v):
     """
@@ -444,6 +478,31 @@ def test_eigen():
          (-1, 1, [Matrix([-1, 1, 0])]),
          ( 0, 1, [Matrix([ 0,-1, 1])])])
 
+    M = Matrix([ [1, -1],
+                 [1,  3]])
+    assert canonicalize(M.eigenvects()) == canonicalize(
+        [[2, 2, [Matrix(1,2,[-1,1])]]])
+
+    M = Matrix([ [1, 2, 3], [4, 5, 6], [7, 8, 9] ])
+    a=R(15,2)
+    b=3*33**R(1,2)
+    c=R(13,2)
+    d=(R(33,8) + 3*b/8)
+    e=(R(33,8) - 3*b/8)
+    def NS(e, n):
+        return str(N(e, n))
+    r = [
+        (a - b/2, 1, [Matrix([(12 + 24/(c - b/2))/((c - b/2)*e) + 3/(c - b/2),
+                              (6 + 12/(c - b/2))/e,1])]),
+        (      0, 1, [Matrix([1,-2,1])]),
+        (a + b/2, 1, [Matrix([(12 + 24/(c + b/2))/((c + b/2)*d) + 3/(c + b/2),
+                              (6 + 12/(c + b/2))/d,1])]),
+        ]
+    r1 = [(NS(r[i][0],2),NS(r[i][1],2),[NS(j,2) for j in r[i][2][0]]) for i in range(len(r))]
+    r = M.eigenvects()
+    r2=[(NS(r[i][0],2),NS(r[i][1],2),[NS(j,2) for j in r[i][2][0]]) for i in range(len(r))]
+    assert sorted(r1) == sorted(r2)
+
     eps = Symbol('eps',real=True)
 
     M = Matrix([[abs(eps), I*eps    ],
@@ -457,7 +516,7 @@ def test_sparse_matrix():
     return
     def eye(n):
         tmp = SMatrix(n,n,lambda i,j:0)
-        for i in range(tmp.lines):
+        for i in range(tmp.rows):
             tmp[i,i] = 1
         return tmp
     def zeros(n):
@@ -510,12 +569,12 @@ def test_sparse_matrix():
     x = Symbol("x")
     a = SMatrix([x, 0], [0, 0])
     m = a
-    assert m.cols == m.lines
+    assert m.cols == m.rows
     assert m.cols == 2
     assert m[:] == [x,0,0,0]
     b = SMatrix(2,2, [x, 0, 0, 0])
     m = b
-    assert m.cols == m.lines
+    assert m.cols == m.rows
     assert m.cols == 2
     assert m[:] == [x,0,0,0]
 
@@ -939,7 +998,7 @@ def test_zeros_ones_fill():
     b = 5 * ones( (n, m) )
 
     assert a == b
-    assert a.lines == b.lines == 3
+    assert a.rows == b.rows == 3
     assert a.cols == b.cols == 5
     assert a.shape == b.shape == (3, 5)
 
@@ -969,3 +1028,144 @@ def test_jacobian_metrics():
     g = J.T*eye(J.shape[0])*J
     g = g.applyfunc(trigsimp)
     assert g == Matrix([[1, 0], [0, rho**2]])
+
+def test_jacobian2():
+    rho, phi = symbols("rho phi")
+    X = Matrix([rho*cos(phi), rho*sin(phi), rho**2])
+    Y = Matrix([rho, phi])
+    J = Matrix([
+            [cos(phi), -rho*sin(phi)],
+            [sin(phi),  rho*cos(phi)],
+            [   2*rho,             0],
+        ])
+    assert X.jacobian(Y) == J
+
+def test_issue1465():
+    x, y, z = symbols('x', 'y', 'z')
+    X = Matrix([exp(x + y + z), exp(x + y + z), exp(x + y + z)])
+    Y = Matrix([x, y, z])
+    for i in range(1, 3):
+        for j in range(1, 3):
+            X_slice = X[:i,:]
+            Y_slice = Y[:j,:]
+            J = X_slice.jacobian(Y_slice)
+            assert J.rows == i
+            assert J.cols == j
+            for k in range(j):
+                assert J[:,k] == X_slice
+
+def test_nonvectorJacobian():
+    x, y, z = symbols('x', 'y', 'z')
+    X = Matrix([ [exp(x + y + z), exp(x + y + z)],
+                 [exp(x + y + z), exp(x + y + z)] ])
+    Y = Matrix([x, y, z])
+    raises(TypeError, 'X.jacobian(Y)')
+    X = X[0,:]
+    Y = Matrix([ [x, y], [x,z] ])
+    raises(TypeError, 'X.jacobian(Y)')
+
+def test_vec():
+    m = Matrix([ [1,3], [2,4] ])
+    m_vec = m.vec()
+    assert m_vec.cols == 1
+    for i in xrange(4):
+        assert m_vec[i] == i + 1
+
+def test_vech():
+    m = Matrix([ [1,2], [2,3] ])
+    m_vech = m.vech()
+    assert m_vech.cols == 1
+    for i in xrange(3):
+        assert m_vech[i] == i + 1
+    m_vech = m.vech(diagonal = False)
+    assert m_vech[0] == 2
+
+def test_vech_TypeError():
+    m = Matrix([ [1,3] ])
+    raises(TypeError, 'm.vech()')
+    m = Matrix([ [1,3], [2,4] ])
+    raises(TypeError, 'm.vech()')
+
+def test_block_diag1():
+    x, y, z = symbols("x y z")
+    a = Matrix([[1, 2], [2, 3]])
+    b = Matrix([[3, x], [y, 3]])
+    c = Matrix([[3, x, 3], [y, 3, z], [x, y, z]])
+    assert block_diag([a, b, b]) == Matrix([
+            [1, 2, 0, 0, 0, 0],
+            [2, 3, 0, 0, 0, 0],
+            [0, 0, 3, x, 0, 0],
+            [0, 0, y, 3, 0, 0],
+            [0, 0, 0, 0, 3, x],
+            [0, 0, 0, 0, y, 3],
+            ])
+    assert block_diag([a, b, c]) == Matrix([
+            [1, 2, 0, 0, 0, 0, 0],
+            [2, 3, 0, 0, 0, 0, 0],
+            [0, 0, 3, x, 0, 0, 0],
+            [0, 0, y, 3, 0, 0, 0],
+            [0, 0, 0, 0, 3, x, 3],
+            [0, 0, 0, 0, y, 3, z],
+            [0, 0, 0, 0, x, y, z],
+            ])
+    assert block_diag([a, c, b]) == Matrix([
+            [1, 2, 0, 0, 0, 0, 0],
+            [2, 3, 0, 0, 0, 0, 0],
+            [0, 0, 3, x, 3, 0, 0],
+            [0, 0, y, 3, z, 0, 0],
+            [0, 0, x, y, z, 0, 0],
+            [0, 0, 0, 0, 0, 3, x],
+            [0, 0, 0, 0, 0, y, 3],
+            ])
+
+def test_get_diag_blocks1():
+    x, y, z = symbols("x y z")
+    a = Matrix([[1, 2], [2, 3]])
+    b = Matrix([[3, x], [y, 3]])
+    c = Matrix([[3, x, 3], [y, 3, z], [x, y, z]])
+    a.get_diag_blocks() == [a]
+    b.get_diag_blocks() == [b]
+    c.get_diag_blocks() == [c]
+
+def test_get_diag_blocks2():
+    x, y, z = symbols("x y z")
+    a = Matrix([[1, 2], [2, 3]])
+    b = Matrix([[3, x], [y, 3]])
+    c = Matrix([[3, x, 3], [y, 3, z], [x, y, z]])
+    assert block_diag([a, b, b]).get_diag_blocks() == [a, b, b]
+    assert block_diag([a, b, c]).get_diag_blocks() == [a, b, c]
+    assert block_diag([a, c, b]).get_diag_blocks() == [a, c, b]
+    assert block_diag([c, c, b]).get_diag_blocks() == [c, c, b]
+
+def test_inv_block():
+    x, y, z = symbols("x y z")
+    a = Matrix([[1, 2], [2, 3]])
+    b = Matrix([[3, x], [y, 3]])
+    c = Matrix([[3, x, 3], [y, 3, z], [x, y, z]])
+    A = block_diag([a, b, b])
+    assert A.inv(try_block_diag=True) == block_diag([a.inv(), b.inv(), b.inv()])
+    A = block_diag([a, b, c])
+    assert A.inv(try_block_diag=True) == block_diag([a.inv(), b.inv(), c.inv()])
+    A = block_diag([a, c, b])
+    assert A.inv(try_block_diag=True) == block_diag([a.inv(), c.inv(), b.inv()])
+    A = block_diag([a, a, b, a, c, a])
+    assert A.inv(try_block_diag=True) == block_diag([
+        a.inv(), a.inv(), b.inv(), a.inv(), c.inv(), a.inv()])
+    assert A.inv(try_block_diag=True, method="ADJ") == block_diag([
+        a.inv(method="ADJ"), a.inv(method="ADJ"), b.inv(method="ADJ"),
+        a.inv(method="ADJ"), c.inv(method="ADJ"), a.inv(method="ADJ")])
+
+def test_creation():
+    """
+    Check that matrix dimensions can be specified using any reasonable type
+    (see issue 1515).
+    """
+    raises(ValueError, 'zeros((3, 0))')
+    raises(ValueError, 'zeros((1,2,3,4))')
+    assert zeros(3L) == zeros(3)
+    assert zeros(Integer(3)) == zeros(3)
+    assert zeros(3.) == zeros(3)
+    assert eye(3L) == eye(3)
+    assert eye(Integer(3)) == eye(3)
+    assert eye(3.) == eye(3)
+    assert ones((3L, Integer(4))) == ones((3, 4))
