@@ -2,9 +2,14 @@
 
 from __future__ import division
 
-from sympy.mpmath.matrices import matrix, norm, mnorm, randmatrix, eye, zeros, diag
-from sympy.mpmath.linalg import * # TODO: absolute imports
-from sympy.mpmath.mptypes import *
+from sympy.mpmath import *
+
+# XXX: these shouldn't be visible(?)
+LU_decomp = mp.LU_decomp
+L_solve = mp.L_solve
+U_solve = mp.U_solve
+householder = mp.householder
+improve_solution = mp.improve_solution
 
 A1 = matrix([[3, 1, 6],
              [2, 1, 3],
@@ -142,6 +147,11 @@ def test_solve():
     assert norm(residual(A10, lu_solve(A10, b10), b10), 2) < 1.e-10
     assert norm(residual(A10, qr_solve(A10, b10)[0], b10), 2) < 1.e-10
 
+def test_solve_overdet_complex():
+    A = matrix([[1, 2j], [3, 4j], [5, 6]])
+    b = matrix([1 + j, 2, -j])
+    assert norm(residual(A, lu_solve(A, b), b)) < 1.0208
+
 def test_singular():
     mp.dps = 15
     A = [[5.6, 1.2], [7./15, .1]]
@@ -158,10 +168,9 @@ def test_singular():
         _assert_ZeroDivisionError(i)
 
 def test_cholesky():
-    A9.force_type = float
-    assert cholesky(A9) == matrix([[2, 0, 0], [1, 2, 0], [-1, -3/2, 3/2]])
-    x = cholesky_solve(A9, b9)
-    assert norm(residual(A9, x, b9), inf) == 0
+    assert fp.cholesky(fp.matrix(A9)) == fp.matrix([[2, 0, 0], [1, 2, 0], [-1, -3/2, 3/2]])
+    x = fp.cholesky_solve(A9, b9)
+    assert fp.norm(fp.residual(A9, x, b9), fp.inf) == 0
 
 def test_det():
     assert det(A1) == 1
@@ -223,7 +232,7 @@ def test_exp_pade():
         a = diag([1,2,3])
         a1 = m**-1 * a * m
         mp.dps = dps
-        e1 = exp_pade(a1)
+        e1 = expm(a1, method='pade')
         mp.dps = dps + extra
         e2 = m * a1 * m**-1
         d = e2 - a
@@ -231,3 +240,4 @@ def test_exp_pade():
         mp.dps = dps
         assert norm(d, inf).ae(0)
     mp.dps = 15
+
