@@ -4,7 +4,7 @@ Number theory module (primes, etc)
 
 from generate import nextprime, prevprime, prime, primepi, primerange,\
 randprime, Sieve, sieve
-from primetest import isprime, mr, mr_safe
+from primetest import isprime, mr
 from factor_ import divisors, factorint, multiplicity, perfect_power,\
 pollard_pm1, pollard_rho, primefactors, totient, trailing
 from partitions_ import npartitions
