@@ -1,5 +1,6 @@
-from sympy import Rational
-from sympy.physics.units import *
+from sympy import Rational, Symbol, integrate
+from sympy.physics.units import m, s, day, km, foot, meter, au, \
+        speed_of_light, minute
 
 def test_units():
     assert (5*m/s * day) / km == 432
@@ -11,3 +12,7 @@ def test_units():
 
     assert (m**2)**Rational(1,2) == m
     assert (m**Rational(1,2))**2 == m
+
+    t = Symbol('t')
+    assert integrate(t*m/s,(t, 1*s, 5*s)) == 12*m*s
+    assert (t * m/s).integrate((t, 1*s, 5*s)) == 12*m*s
