@@ -5,10 +5,9 @@ The base class is Unit, where all here defined units (~200) inherit from.
 """
 
 from sympy import Rational, pi
-from sympy.core.basic import Atom
-from sympy.core.methods import ArithMeths, RelMeths
+from sympy.core.basic import Basic, Atom
 
-class Unit(Atom, RelMeths, ArithMeths):
+class Unit(Atom):
     """
     Base class for all physical units.
 
@@ -20,18 +19,25 @@ class Unit(Atom, RelMeths, ArithMeths):
 
     __slots__ = ["name", "abbrev"]
 
-    def __init__(self, name, abbrev):
-        self.name = name
-        self.abbrev = abbrev
+    def __new__(cls, name, abbrev, **assumptions):
+        obj = Basic.__new__(cls, **assumptions)
+        assert isinstance(name, str),`type(name)`
+        assert isinstance(abbrev, str),`type(abbrev)`
+        obj.name = name
+        obj.abbrev = abbrev
+        return obj
 
-    def tostr(self, level=0):
-        return self.abbrev
+    def __getnewargs__(self):
+        return (self.name, self.abbrev)
 
     def __eq__(self, other):
         return isinstance(other, Unit) and self.name == other.name
 
+    def _hashable_content(self):
+        return (self.name,self.abbrev)
+
 # Delete this so it doesn't pollute the namespace
-del Atom, ArithMeths, RelMeths
+del Atom
 
 def defunit(value, *names):
     u = value
@@ -160,7 +166,7 @@ u0 = magnetic_constant = 4*pi * ten**-7 * N/A**2
 e0 = electric_constant = 1/(u0 * c**2)
 Z0 = vacuum_impedance = u0 * c
 
-planck = Rational('6.2606896') * ten**-34 * J*s
+planck = Rational('6.62606896') * ten**-34 * J*s
 hbar = planck / (2*pi)
 
 avogadro = (Rational('6.02214179') * 10**23) / mol
@@ -169,6 +175,15 @@ boltzmann = Rational('1.3806505') * ten**-23 * J / K
 gee = gees = Rational('9.80665') * m/s**2
 atmosphere = atmospheres = atm = 101325 * pascal
 
+kPa = kilo*Pa
+bar = bars = 100*kPa
+pound = pounds = 0.45359237 * kg * gee #exact
+psi = pound / inch ** 2
+dHg0 = 13.5951 # approx value at 0 C
+mmHg = dHg0 * 9.80665 * Pa
+amu = amus = gram / avogadro
+quart = quarts = 231 * inch**3
+eV = 1.602176487e-19 * J
 
 # Other convenient units and magnitudes
 
@@ -176,4 +191,4 @@ defunit(c*julian_year, 'ly', 'lightyear', 'lightyears')
 defunit(149597870691*m, 'au', 'astronomical_unit', 'astronomical_units')
 
 # Delete this so it doesn't pollute the namespace
-del Rational, pi
\ No newline at end of file
+del Rational, pi
