@@ -2,6 +2,10 @@ from sympy import Integer
 
 from threading import RLock
 
+# it is sufficient to import "pyglet" here once
+from sympy.thirdparty import import_thirdparty
+pyglet = import_thirdparty("pyglet")
+
 from pyglet.gl import *
 
 from plot_object import PlotObject
@@ -25,7 +29,7 @@ class Plot(object):
 
 
     >>> from sympy import symbols, Plot
-    >>> x,y,z = symbols('xyz')
+    >>> from sympy.abc import x, y, z
 
     >>> Plot(x*y**3-y*x**3)
 
@@ -61,13 +65,13 @@ class Plot(object):
     Plot supports several curvilinear coordinate modes, and
     they independent for each plotted function. You can specify
     a coordinate mode explicitly with the 'mode' named argument,
-    but it can be automatically determined for cartesian or
+    but it can be automatically determined for Cartesian or
     parametric plots, and therefore must only be specified for
     polar, cylindrical, and spherical modes.
 
     Specifically, Plot(function arguments) and Plot[n] =
     (function arguments) will interpret your arguments as a
-    cartesian plot if you provide one function and a parametric
+    Cartesian plot if you provide one function and a parametric
     plot if you provide two or three functions. Similarly, the
     arguments will be interpreted as a curve is one variable is
     used, and a surface if two are used.
@@ -147,7 +151,7 @@ class Plot(object):
         other words...
 
         >>> from sympy.core import Symbol
-        >>> x = Symbol('x')
+        >>> from sympy.abc import x
         >>> p = Plot(x**2, visible=False)
 
         ...is equivalent to...
