@@ -107,7 +107,7 @@ class PlotMode(PlotObject):
             usually the number of functions to
             be evaluated in plotting.
 
-        For example, a cartesian function y = f(x) has
+        For example, a Cartesian function y = f(x) has
         one i_var (x) and one d_var (y). A parametric
         form x,y,z = f(u,v), f(u,v), f(u,v) has two
         two i_vars (u,v) and three d_vars (x,y,z).
@@ -206,7 +206,7 @@ class PlotMode(PlotObject):
         cls._init_mode()
 
         #except Exception, e:
-        #    raise Exception( ("Failed to initialize "
+        #    raise RuntimeError( ("Failed to initialize "
         #                      "plot mode %s. Reason: %s")
         #                       % (name, (str(e))) )
 
@@ -228,7 +228,7 @@ class PlotMode(PlotObject):
                 PlotMode._mode_default_map[d][i] = cls
 
         except Exception, e:
-            raise Exception( ("Failed to register "
+            raise RuntimeError( ("Failed to register "
                               "plot mode %s. Reason: %s")
                                % (name, (str(e))) )
 
@@ -320,7 +320,7 @@ class PlotMode(PlotObject):
         # i_vars in given functions
         # (aka d_vars)
         for f in functions:
-            for a in f.atoms(type=Symbol):
+            for a in f.atoms(Symbol):
                 if a not in i_vars:
                     i_vars.append(a)
 
