@@ -1,4 +1,3 @@
-
 from sympy.core.add import Add
 from sympy.core.mul import Mul
 from sympy.core.power import Pow
@@ -122,7 +121,7 @@ def poly_pdiv(f, g, *symbols):
        For more information on the implemented algorithm refer to:
 
        [1] M. Bronstein, Symbolic Integration I: Transcendental
-           Functions, Second Edition, Springer-Verlang, 2005
+           Functions, Second Edition, Springer-Verlag, 2005
 
     """
     if not isinstance(f, Poly):
@@ -180,8 +179,8 @@ def poly_groebner(f, *symbols, **flags):
        variable at a time, provided that the ideal is zero-dimensional
        (finite number of solutions).
 
-       >>> from sympy import *
-       >>> x,y = symbols('xy')
+       >>> from sympy.polys.algorithms import poly_groebner
+       >>> from sympy.abc import x, y
 
        >>> G = poly_groebner([x**2 + y**3, y**2-x], x, y, order='lex')
 
@@ -240,7 +239,7 @@ def poly_groebner(f, *symbols, **flags):
         if h.is_zero:
             return None
         else:
-            if not F.has_key(h):
+            if not h in F:
                 F[h] = len(f)
                 f.append(h)
 
@@ -270,11 +269,11 @@ def poly_groebner(f, *symbols, **flags):
                     continue
 
                 if i < j:
-                   k = (i, j)
+                    k = (i, j)
                 else:
-                   k = (j, i)
+                    k = (j, i)
 
-                if not B.has_key(k):
+                if k not in B:
                     B[k] = monomial_lcm(f[i].LM, f[j].LM)
 
         G = set([ normal(f[g], G - set([g]))[0] for g in G ])
@@ -306,10 +305,10 @@ def poly_groebner(f, *symbols, **flags):
             if g == i or g == j:
                 continue
 
-            if not B.has_key((min(i, g), max(i, g))):
+            if (min(i, g), max(i, g)) not in B:
                 continue
 
-            if not B.has_key((min(j, g), max(j, g))):
+            if (min(j, g), max(j, g)) not in B:
                 continue
 
             if not monomial_div(M, f[g].LM):
@@ -483,7 +482,7 @@ def poly_gcd(f, g, *symbols):
     if f.is_multivariate:
         h = poly_div(f*g, poly_lcm(f, g))[0]
     else:
-        h = poly_subresultants(f, g)[-1]
+        h = poly_subresultants(f, g, res=False)[-1]
 
     if gcd != 1:
         return h.mul_term(gcd / h.LC)
@@ -500,7 +499,7 @@ def poly_gcdex(f, g, *symbols):
        For more information on the implemented algorithm refer to:
 
        [1] M. Bronstein, Symbolic Integration I: Transcendental
-           Functions, Second Edition, Springer-Verlang, 2005
+           Functions, Second Edition, Springer-Verlag, 2005
 
     """
     s, h = poly_half_gcdex(f, g, *symbols)
@@ -517,7 +516,7 @@ def poly_half_gcdex(f, g, *symbols):
        For more information on the implemented algorithm refer to:
 
        [1] M. Bronstein, Symbolic Integration I: Transcendental
-           Functions, Second Edition, Springer-Verlang, 2005
+           Functions, Second Edition, Springer-Verlag, 2005
 
     """
     if not isinstance(f, Poly):
@@ -555,8 +554,8 @@ def poly_resultant(f, g, *symbols):
        matrices whose entries are computed using O(n**2) additions
        and multiplications where n = max(deg(f), deg(g)).
 
-       >>> from sympy import *
-       >>> x,y = symbols('xy')
+       >>> from sympy.polys.algorithms import poly_resultant
+       >>> from sympy.abc import x, y
 
        Polynomials x**2-1 and (x-1)**2 have common root:
 
@@ -597,10 +596,10 @@ def poly_resultant(f, g, *symbols):
 
     for i in xrange(N):
         for j in xrange(i, N):
-            if p.has_key(i) and q.has_key(j+1):
+            if i in p and j+1 in q:
                 B[i, j] += p[i] * q[j+1]
 
-            if p.has_key(j+1) and q.has_key(i):
+            if j+1 in p and i in q:
                 B[i, j] -= p[j+1] * q[i]
 
     for i in xrange(1, N-1):
@@ -628,15 +627,15 @@ def poly_resultant(f, g, *symbols):
         else:
             return sign * Poly.cancel(det)
 
-def poly_subresultants(f, g, *symbols):
+def poly_subresultants(f, g, *symbols, **flags):
     """Computes subresultant PRS of two univariate polynomials.
 
        Polynomial remainder sequence (PRS) is a fundamental tool in
        computer algebra as it gives as a sub-product the polynomial
        greatest common divisor (GCD), provided that the coefficient
-       domain is an unique factorization domain.
+       domain is a unique factorization domain.
 
-       There are several methods for computing PRS, eg.: Euclidean
+       There are several methods for computing PRS, e.g.: Euclidean
        PRS, where the most famous algorithm is used, primitive PRS
        and, finally, subresultants which are implemented here.
 
@@ -654,10 +653,17 @@ def poly_subresultants(f, g, *symbols):
        remainder sequence where R_0 = f, R_1 = g, R_k != 0 and R_k
        is similar to gcd(f, g).
 
+       The result is returned as tuple (res, R) where R is the PRS
+       sequence and res is the resultant of the input polynomials.
+
+       If only polynomial remainder sequence is important,  then by
+       setting res=False in keyword arguments expensive computation
+       of the resultant can be avoided (only PRS is returned).
+
        For more information on the implemented algorithm refer to:
 
        [1] M. Bronstein, Symbolic Integration I: Transcendental
-           Functions, Second Edition, Springer-Verlang, 2005
+           Functions, Second Edition, Springer-Verlag, 2005
 
        [2] M. Keber, Division-Free computation of subresultants
            using Bezout matrices, Tech. Report MPI-I-2006-1-006,
@@ -673,8 +679,8 @@ def poly_subresultants(f, g, *symbols):
 
     if f.is_multivariate:
         raise MultivariatePolyError(f)
-
-    symbols, flags = f.symbols, f.flags
+    else:
+        symbols = f.symbols
 
     n, m = f.degree, g.degree
 
@@ -682,46 +688,75 @@ def poly_subresultants(f, g, *symbols):
         f, g = g, f
         n, m = m, n
 
-    prs = [f, g]
+    R = [f, g]
 
     d = n - m
 
-    b = (-1)**(d + 1)
+    b = S(-1)**(d + 1)
+    c = S(-1)
 
-    h = poly_pdiv(f, g)[1]
+    B, D = [b], [d]
+
+    h = poly_prem(f, g)
     h = h.mul_term(b)
 
-    k = h.degree
-
-    c = S.NegativeOne
-
     while not h.is_zero:
-        prs.append(h)
+        k = h.degree
+        R.append(h)
 
-        coeff = g.LC
+        lc = g.LC
 
-        c = (-coeff)**d / c**(d-1)
+        C = (-lc)**d / c**(d-1)
+        c = Poly.cancel(C)
 
-        b = -coeff * c**(m-k)
+        b = -lc * c**(m-k)
 
         f, g, m, d = g, h, k, m-k
 
-        h = poly_pdiv(f, g)[1]
+        B.append(b)
+        D.append(d)
+
+        h = poly_prem(f, g)
         h = h.div_term(b)
 
-        k = h.degree
+    if not flags.get('res', True):
+        return R
 
-    return prs
+    if R[-1].degree > 0:
+        return (Poly((), *symbols), R)
+    if R[-2].is_one:
+        return (R[-1], R)
+
+    s, c, i = 1, S(1), 1
+
+    for b, d in zip(B, D)[:-1]:
+        u = R[i-1].degree
+        v = R[i  ].degree
+        w = R[i+1].degree
+
+        if u % 2 and v % 2:
+            s = -s
+
+        lc = R[i].LC
+
+        C = c*(b/lc**(1 + d))**v * lc**(u - w)
+        c = Poly.cancel(C)
+
+        i += 1
+
+    j = R[-2].degree
+
+    return (R[-1]**j*s*c, R)
 
 def poly_sqf(f, *symbols):
-    """Compute square-free decomposition of an univariate polynomial.
+    """Compute square-free decomposition of a univariate polynomial.
 
-       Given an univariate polynomial f over an unique factorization domain
+       Given a univariate polynomial f over a unique factorization domain
        returns tuple (f_1, f_2, ..., f_n),  where all  A_i are co-prime and
        square-free polynomials and f = f_1 * f_2**2 * ... * f_n**n.
 
-       >>> from sympy import *
-       >>> x,y = symbols('xy')
+       >>> from sympy.polys.algorithms import poly_sqf
+       >>> from sympy.abc import x, y
 
        >>> p, q = poly_sqf(x*(x+1)**2, x)
 
@@ -733,7 +768,7 @@ def poly_sqf(f, *symbols):
        For more information on the implemented algorithm refer to:
 
        [1] M. Bronstein, Symbolic Integration I: Transcendental
-           Functions, Second Edition, Springer-Verlang, 2005
+           Functions, Second Edition, Springer-Verlag, 2005
 
        [2] J. von zur Gathen, J. Gerhard, Modern Computer Algebra,
            Second Edition, Cambridge University Press, 2003
@@ -783,13 +818,13 @@ def poly_sqf(f, *symbols):
     return [head] + tail
 
 def poly_decompose(f, *symbols):
-    """Computes functional decomposition of an univariate polynomial.
+    """Computes functional decomposition of a univariate polynomial.
 
        Besides factorization and square-free decomposition, functional
        decomposition is another important, but very different,  way of
        breaking down polynomials into simpler parts.
 
-       Formally given an univariate polynomial f with coefficients in a
+       Formally given a univariate polynomial f with coefficients in a
        field of characteristic zero, returns tuple (f_1, f_2, ..., f_n)
        where f = f_1 o f_2 o ... f_n = f_1(f_2(... f_n)) and f_2, ...,
        f_n are monic and homogeneous polynomials of degree at least 2.
@@ -803,8 +838,8 @@ def poly_decompose(f, *symbols):
 
        where T_n and T_m are Chebyshev polynomials.
 
-       >>> from sympy import *
-       >>> x,y = symbols('xy')
+       >>> from sympy.polys.algorithms import poly_decompose
+       >>> from sympy.abc import x, y
 
        >>> p, q = poly_decompose(x**4+2*x**2 + y, x)
 
@@ -842,10 +877,10 @@ def poly_decompose(f, *symbols):
             coeff = S.Zero
 
             for j in xrange(0, k):
-                if not f.has_key(n+j-k):
+                if not n+j-k in f:
                     continue
 
-                if not q.has_key(s-j):
+                if not s-j in q:
                     continue
 
                 fc, qc = f[n+j-k], q[s-j]
@@ -906,8 +941,9 @@ def poly_decompose(f, *symbols):
 def poly_reduce(f, g, *symbols):
     """Removes common content from a pair of polynomials.
 
-       >>> from sympy import *
-       >>> x = Symbol('x')
+       >>> from sympy import Poly
+       >>> from sympy.abc import x
+       >>> from sympy.polys.algorithms import poly_reduce
 
        >>> f = Poly(2930944*x**6 + 2198208*x**4 + 549552*x**2 + 45796, x)
        >>> g = Poly(17585664*x**5 + 8792832*x**3 + 1099104*x, x)
@@ -938,3 +974,40 @@ def poly_reduce(f, g, *symbols):
 
     return f, g
 
+def poly_discriminant(p, symbol):
+    """
+    Returns the discriminant of a polynomial with respect to symbol.
+
+    The discriminant of a univariate polynomial p of degree n is defined as
+    n**(n*(n-1)/2)/a_n*resultant(p, p'), where p' is the derivative of p and a_n
+    is the leading coefficient of p.  Because the resultant of two polynomials
+    vanishes identically whenever the two polynomials share a root, and a
+    polynomial shares a root with its derivative if and only if the root is a
+    repeated root, it follows that the discriminant of a polynomial vanishes
+    identically if and only if the polynomial has a repeated root.
+
+    See also:
+    <http://en.wikipedia.org/wiki/Discriminant>
+
+    Example:
+    >>> from sympy import discriminant, Poly
+    >>> from sympy.abc import a, b, c, x
+    >>> discriminant(Poly(a*x**2 + b*x + c, x), x)
+    -4*a*c + b**2
+    >>> discriminant(Poly(2*x**5 + x**2 + 10, x), x)
+    500004320
+    >>> discriminant(Poly((x-1)*(x+1), x), x)
+    4
+    >>> discriminant(Poly((x-1)**2*(x+1), x), x)
+    0
+
+    """
+    if not isinstance(p, Poly):
+        p = Poly(p, symbol)
+    if not p.is_univariate:
+        # We need to make p univariate for poly_resultant to work
+        p = Poly(p.as_basic(), symbol) # Is there a better way to do this?
+    if p.degree == 0:
+        return S.Zero
+    return (S(int((-1)**((p.degree)*(S(p.degree) - 1)/2)))/p.lead_coeff*\
+    poly_resultant(p, p.diff(symbol))).expand()
