@@ -1,4 +1,3 @@
-
 from sympy.core.add import Add
 from sympy.core.mul import Mul
 from sympy.core.power import Pow
@@ -27,19 +26,34 @@ class CoefficientError(PolynomialError):
 
 class UnivariatePolyError(PolynomialError):
 
-    def __init__(self, poly):
-        self.message = "%s is an univariate polynomial" % poly
+    def __init__(self, expr, base_exc=None):
+        self.expr = expr
+        self.base_exc = base_exc
 
     def __str__(self):
-        return self.message
+        if self.base_exc is None:
+            return "UnivariatePolyError: %s" % (self.expr,)
+
+        return "Univariate polynomial '%s' unexpected and raised \
+        exception:\n%s: %s" % (self.expr,
+                               self.base_exc.__class__.__name__,
+                               str(self.base_exc))
 
 class MultivariatePolyError(PolynomialError):
 
-    def __init__(self, poly):
-        self.message = "%s is a multivariate polynomial" % poly
+    def __init__(self, expr, base_exc=None):
+        self.expr = expr
+        self.base_exc = base_exc
 
     def __str__(self):
-        return self.message
+        if self.base_exc is None:
+            return "MultivariatePolyError: %s" % (self.expr,)
+
+        return "Multivariate polynomial '%s' unexpected and raised \
+        exception:\n%s: %s" % (self.expr,
+                               self.base_exc.__class__.__name__,
+                               str(self.base_exc))
+
 
 ##
 ## TODO:
@@ -56,16 +70,16 @@ class MultivariatePolyError(PolynomialError):
 class Poly(Basic):
     """Represents polynomials with symbolic coefficients.
 
-       Polynomials are internally represented as two lists containing
-       coefficients and monomials (tuples of exponents) respectively.
-       Stored are only terms with non-zero coefficients, so generally
-       all polynomials are considered sparse. However algorithms will
-       detect dense polynomials and use the appropriate method to
-       solve the given problem in the most efficient way.
+       Polynomials are internally represented as two lists
+       containing coefficients and monomials (tuples of exponents),
+       respectively. Stored are only terms with non-zero
+       coefficients, so generally all polynomials are considered
+       sparse. However, algorithms will detect dense polynomials and
+       use the appropriate method to solve them in the most efficient way.
 
-       The most common way to initialize a polynomial instance is to
-       provide a valid expression together witch a set of ordered
-       symbols and, additionally, monomial ordering:
+       The most common way to initialize a polynomial
+       instance is to provide a valid expression together with a
+       set of ordered symbols and, additionally, monomial ordering:
 
             Poly(expression, x_1, x_2, ..., x_n, order='grlex')
 
@@ -74,41 +88,73 @@ class Poly(Basic):
        you can use Basic.as_poly to avoid exception handling.
 
        By default ordering of monomials can be omitted. In this case
-       graded lexicographic order will be used. Anyway remember that
+       graded lexicographic order will be used. Anyway, remember that
        'order' is a keyword argument.
 
-       Currently there are supported four standard orderings:
+       Currently, four standard orderings are supported:
 
            [1] lex       -> lexicographic order
            [2] grlex     -> graded lex order
            [3] grevlex   -> reversed grlex order
            [4] 1-el      -> first elimination order
 
-       Polynomial can be also constructed explicitly by specifying
-       a collection of coefficients and monomials. This can be done
-       in at least three different ways:
+       Polynomials can also be constructed explicitly by passing a
+       collection of coefficients and monomials in as an expression.
+       Interpretation of the collection depends on the collection's
+       type according to the following semantics:
 
-           [1] [(c_1, M_1), (c_2, M_2), ..., (c_1, M_1)]
+           [1] expression is a LIST
+               a) [(c_1, M_1), (c_2, M_2), ..., (c_n, M_n)]
+               b) [(c_1, m_1), (c_2, m_2), ..., (c_n, m_n)]
+               c) [ c_1, c_2, ..., c_n ]
 
-           [2] (c_1, c2, ..., c_n), (M_1, M_2, ..., M_n)
+                >>> from sympy import Poly, S
+                >>> from sympy.abc import a, x, y
+                >>> # a) M is a tuple (m_1, m_2, ..., m_3)
+                >>> Poly([(4, (2,2)), (3, (1,2))], x, y)
+                Poly(4*x**2*y**2 + 3*x*y**2, x, y)
+                >>> # b) m_i are integers
+                >>> Poly([(1, 2), (a, 0)], x)
+                Poly(x**2 + a, x)
+                >>> # c) only coefficients
+                >>> Poly([1, 0, a], x)
+                Poly(x**2 + a, x)
 
-           [3] { M_1 : c_1, M_2 : c_2, ..., M_n : c_n }
+           [2] expression is a TUPLE:
+               ((c_1, c2, ..., c_n), (M_1, M_2, ..., M_n))
 
-       Although all three representation look similar, they are
-       designed for different tasks and have specific properties:
+                >>> Poly( ((S(1), S(2)), ((3,), (4,))), x)
+                Poly(x**3 + 2*x**4, x)
 
-           [1] All coefficients and monomials  are validated before
+           [3] expression is a DICTIONARY:
+               { M_1 : c_1, M_2 : c_2, ..., M_n : c_n }
+
+                >>> Poly( {(1, 2): S(3), (4, 5): S(6)} ,x ,y)
+                Poly(6*x**4*y**5 + 3*x*y**2, x, y)
+
+       These initialization methods are not just conveniences for creating
+       polynomials. They are designed for different tasks and the resulting
+       polynomials have specific properties:
+
+           [1] All coefficients and monomials are validated before
                polynomial instance is created. Monomials are sorted
                with respect to the given order.
 
            [2] No validity checking, no sorting, just a raw copy.
 
-           [3] Also no validity checking however monomials are
-               sorted with respect to the given order.
+           [3] No validity checking, but monomials are sorted with
+               respect to the given order.
 
        For interactive usage choose [1] as it's safe and relatively
-       fast. Use [2] or [3] internally for time critical algorithms,
-       when you know that coefficients and monomials will be valid.
+       fast.
+
+       CAUTION:
+       Use [2] or [3] internally for time critical algorithms, when you know
+       that coefficients and monomials will be valid sympy expressions. Use
+       them with caution! If the coefficients are integers instead of sympy
+       integers (e.g. 1 instead of S(1)) the polynomial will be created but
+       you may run into problems if you try to print the polynomial. If the
+       monomials are not given as tuples of integers you will have problems.
 
        Implemented methods:
 
@@ -237,7 +283,7 @@ class Poly(Basic):
 
        [11] Substitution and evaluation:
 
-          [11.1] [U-] __call__    --> evaluates poly a the given point
+          [11.1] [U-] __call__    --> evaluates poly at the given point
           [11.2] [U-] evaluate    --> evaluates poly for specific vars
           [11.3] [--] _eval_subs  --> efficiently substitute variables
 
@@ -249,7 +295,7 @@ class Poly(Basic):
 
        [13] String representation functions:
 
-          [13.1] [U-] repr      --> returns represantation of 'self'
+          [13.1] [U-] repr      --> returns representation of 'self'
           [13.3] [U-] str       --> returns pretty representation
 
        [14] Other (derived from Basic) functionality:
@@ -276,7 +322,7 @@ class Poly(Basic):
            New York, USA, 1976, pp. 136-148
 
        [6] M. Bronstein, Symbolic Integration I: Transcendental
-           Functions, Second Edition, Springer-Verlang, 2005
+           Functions, Second Edition, Springer-Verlag, 2005
 
        See also documentation of particular methods.
 
@@ -296,6 +342,9 @@ class Poly(Basic):
                     poly = poly.as_dict()
                 else:
                     return poly
+            elif isinstance(poly, list):
+                return Poly([(c, i) for i, c in enumerate(reversed(poly))],
+                            Symbol('x'))
             else:
                 raise SymbolsError("No symbols were given")
 
@@ -340,18 +389,21 @@ class Poly(Basic):
                     raise PolynomialError("Invalid arguments," \
                         " should get '(coeffs, monoms)' tuple")
 
-        # [ (c1,M1), (c2,M2), ... ]
+        # [ (c1,M1), (c2,M2), ... ] or [ c1, c2, ... ]
         elif type(poly) is list:
             if not poly:
                 coeffs = (S.Zero,)
                 monoms = ((0,) * N,)
+            elif not isinstance(poly[0], (tuple,list)) and len(symbols) == 1:
+                return Poly([(c, i) for i, c in enumerate(reversed(poly))], \
+                            symbols[0])
             else:
                 for coeff, monom in poly:
                     coeff = sympify(coeff)
 
                     if coeff.has_any_symbols(*symbols):
                         raise CoefficientError("%s coefficient is dependent" \
-                            " of polynomial's symbols %s" % (coeff, symbols))
+                            " on polynomial's symbols %s" % (coeff, symbols))
 
                     if type(monom) is int:
                         monom = (monom,)
@@ -363,7 +415,7 @@ class Poly(Basic):
                     if not coeff:
                         continue
 
-                    if terms.has_key(monom):
+                    if monom in terms:
                         coeff += terms[monom]
 
                         if not coeff:
@@ -415,6 +467,7 @@ class Poly(Basic):
 
                 coeffs = [ terms[monom] for monom in monoms ]
 
+        assert isinstance(monoms[0][0],int) #and isinstance(coeffs[0],Basic) #you can get by with the 2nd if you don't print
         args = (tuple(coeffs), tuple(monoms),
                 symbols, order, stamp, None)
 
@@ -436,7 +489,7 @@ class Poly(Basic):
 
                 monom = monom[:N]
 
-                if terms.has_key(monom):
+                if monom in terms:
                     coeff += terms[monom]
 
                     if not coeff:
@@ -468,7 +521,7 @@ class Poly(Basic):
 
                 monom = tuple(exponents)
 
-                if terms.has_key(monom):
+                if monom in terms:
                     coeff += terms[monom]
 
                     if not coeff:
@@ -493,8 +546,8 @@ class Poly(Basic):
            expression to a polynomial use Polynomial constructor
            explicitly or Basic.as_poly method.
 
-           >>> from sympy import *
-           >>> x, y = symbols('xy')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y
 
            >>> Poly._decompose(y*x**2 + 3, x)
            {(0,): 3, (2,): y}
@@ -548,7 +601,7 @@ class Poly(Basic):
 
                 monom = tuple(monom)
 
-            if result.has_key(monom):
+            if monom in result:
                 coeff += result[monom]
 
                 if not coeff:
@@ -578,8 +631,8 @@ class Poly(Basic):
            The input fraction can be given as a single expression or
            as a tuple consisting of the numerator and denominator.
 
-           >>> from sympy import *
-           >>> x,y = symbols('xy')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y
 
            >>> Poly.cancel((x**2-y**2)/(x-y), x, y)
            x + y
@@ -665,8 +718,8 @@ class Poly(Basic):
 
            Note that the result is always returned in expanded form.
 
-           >>> from sympy import *
-           >>> x,y = symbols('xy')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y
 
            >>> p = Poly(x**2+3, x)
            >>> p.as_basic()
@@ -686,10 +739,10 @@ class Poly(Basic):
 
            This method creates dictionary efficiently, so it can be
            used in several algorithms which perform best when dicts
-           are being used, eg. addition, multiplication etc.
+           are being used, e.g. addition, multiplication etc.
 
-           >>> from sympy import *
-           >>> x,y = symbols('xy')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y
 
            >>> p = Poly(x**2+3, x)
            >>> p.as_dict()
@@ -707,8 +760,8 @@ class Poly(Basic):
            dictionary representation with all tuples converted to
            explicit integers.
 
-           >>> from sympy import *
-           >>> x,y = symbols('xy')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y
 
            >>> p = Poly(x**2+3, x)
            >>> p.as_uv_dict()
@@ -720,6 +773,9 @@ class Poly(Basic):
         else:
             raise MultivariatePolyError(self)
 
+    def doit(self):
+        return self
+
     @property
     def coeffs(self):
         return self._args[0]
@@ -796,7 +852,7 @@ class Poly(Basic):
             terms = self.as_uv_dict()
 
             for i in xrange(self.degree, -1, -1):
-                if terms.has_key(i):
+                if i in terms:
                     yield terms[i]
                 else:
                     yield S.Zero
@@ -815,7 +871,7 @@ class Poly(Basic):
             terms = self.as_uv_dict()
 
             for i in xrange(self.degree, -1, -1):
-                if terms.has_key(i):
+                if i in terms:
                     yield terms[i], (i,)
                 else:
                     yield S.Zero, (i,)
@@ -883,7 +939,7 @@ class Poly(Basic):
         terms = self.as_dict()
 
         for coeff, monom in poly.iter_terms():
-            if terms.has_key(monom):
+            if monom in terms:
                 coeff += terms[monom]
 
                 coeff = Poly.cancel(coeff)
@@ -917,7 +973,7 @@ class Poly(Basic):
         terms = self.as_dict()
 
         for coeff, monom in poly.iter_terms():
-            if terms.has_key(monom):
+            if monom in terms:
                 coeff -= terms[monom]
 
                 coeff = Poly.cancel(coeff)
@@ -1003,7 +1059,7 @@ class Poly(Basic):
                 coeff = 0
 
                 for i in xrange(k+1):
-                    if p.has_key(i) and q.has_key(k-i):
+                    if i in p and k-i in q:
                         coeff += Poly.cancel(p[i] * q[k-i])
 
                 if coeff:
@@ -1033,7 +1089,7 @@ class Poly(Basic):
                     coeff = coeff_p * coeff_q
                     coeff = Poly.cancel(coeff)
 
-                    if terms.has_key(monom):
+                    if monom in terms:
                         coeff += terms[monom]
 
                         if not coeff:
@@ -1044,6 +1100,7 @@ class Poly(Basic):
 
         return self.__class__(terms, *self.symbols, **self.flags)
 
+
     def __pow__(self, other):
         """Polynomial exponentiation using binary method.
 
@@ -1201,8 +1258,8 @@ class Poly(Basic):
     def is_squarefree(self):
         """Returns True if 'self' has no factors of multiplicity >= 2.
 
-           >>> from sympy import *
-           >>> x,y = symbols('xy')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y
 
            >>> Poly(x-1, x).is_squarefree
            True
@@ -1240,8 +1297,8 @@ class Poly(Basic):
     def as_monic(self):
         """Returns a monic polynomial.
 
-           >>> from sympy import *
-           >>> x,y = symbols('xy')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y
 
            >>> Poly(x**2 + 4, x).as_monic()
            Poly(x**2 + 4, x)
@@ -1273,8 +1330,8 @@ class Poly(Basic):
            consisting of a common denominator of those coefficients and an
            integer polynomial. Otherwise an exception is being raised.
 
-           >>> from sympy import *
-           >>> x,y = symbols("xy")
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y
 
            >>> Poly(3*x**2 + x, x).as_integer()
            (1, Poly(3*x**2 + x, x))
@@ -1308,8 +1365,8 @@ class Poly(Basic):
     def content(self):
         """Returns integer GCD of all the coefficients.
 
-           >>> from sympy import *
-           >>> x,y,z = symbols('xyz')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y, z
 
            >>> p = Poly(2*x + 5*x*y, x, y)
            >>> p.content
@@ -1340,8 +1397,8 @@ class Poly(Basic):
     def as_primitive(self):
         """Returns content and primitive part of a polynomial.
 
-           >>> from sympy import *
-           >>> x,y = symbols('xy')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y
 
            >>> p = Poly(4*x**2 + 2*x, x)
            >>> p.as_primitive()
@@ -1361,8 +1418,8 @@ class Poly(Basic):
     def as_squarefree(self):
         """Returns square-free part of a polynomial.
 
-           >>> from sympy import *
-           >>> x,y = symbols('xy')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y
 
            >>> Poly((x-1)**2, x).as_squarefree()
            Poly(x - 1, x)
@@ -1385,8 +1442,8 @@ class Poly(Basic):
     def as_reduced(self):
         """Remove GCD of monomials from 'self'.
 
-           >>> from sympy import *
-           >>> x,y = symbols('xy')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y
 
            >>> Poly(x**3 + x, x).as_reduced()
            ((1,), Poly(x**2 + 1, x))
@@ -1419,8 +1476,8 @@ class Poly(Basic):
     def map_coeffs(self, f, *args, **kwargs):
         """Apply a function to all the coefficients.
 
-           >>> from sympy import *
-           >>> x,y,u,v = symbols('xyuv')
+           >>> from sympy import Poly, expand
+           >>> from sympy.abc import x, y, u, v
 
            >>> p = Poly(x**2 + 2*x*y, x, y)
            >>> q = p.map_coeffs(lambda c: 2*c)
@@ -1451,8 +1508,8 @@ class Poly(Basic):
     def coeff(self, *monom):
         """Returns coefficient of a particular monomial.
 
-           >>> from sympy import *
-           >>> x,y = symbols('xy')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y
 
            >>> p = Poly(3*x**2*y + 4*x*y**2 + 1, x, y)
 
@@ -1538,8 +1595,8 @@ class Poly(Basic):
            positive integer which will indicate the number of times to
            perform differentiation using this symbol.
 
-           >>> from sympy import *
-           >>> x,y,z = symbols('xyz')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y, z
 
            >>> p = Poly(x**2*y + z**2, x, y)
 
@@ -1600,7 +1657,7 @@ class Poly(Basic):
                         for j in xrange(n, M[i]):
                             coeff *= j+1
 
-                        if new_poly.has_key(monom):
+                        if monom in new_poly:
                             coeff += new_poly[monom]
 
                             if not coeff:
@@ -1635,8 +1692,8 @@ class Poly(Basic):
            single positive integer which will indicate the number
            of times to perform integration using this symbol.
 
-           >>> from sympy import *
-           >>> x,y,z = symbols('xyz')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y, z
 
            >>> p = Poly(x**2*y + z**2, x, y)
 
@@ -1690,7 +1747,7 @@ class Poly(Basic):
                     for j in xrange(M[i], n):
                         coeff /= j+1
 
-                    if new_poly.has_key(monom):
+                    if monom in new_poly:
                         coeff += new_poly[monom]
 
                         if not coeff:
@@ -1929,8 +1986,8 @@ class Poly(Basic):
            symbols use 'evaluate' method instead. Alternatively one can
            use Basic.subs() for this purpose.
 
-           >>> from sympy import *
-           >>> x,y = symbols('xy')
+           >>> from sympy import Poly
+           >>> from sympy.abc import x, y
 
            >>> p = Poly(x**2 + 2*x + 1, x)
 
@@ -1960,7 +2017,7 @@ class Poly(Basic):
             for k in xrange(self.degree, -1, -1):
                 result *= point
 
-                if terms.has_key(k):
+                if k in terms:
                     result += terms[k]
 
             return result
@@ -2041,7 +2098,7 @@ class Poly(Basic):
 
                 coeff = Poly.cancel(coeff)
 
-                if terms.has_key(monom):
+                if monom in terms:
                     coeff += terms[monom]
 
                     if not coeff:
@@ -2081,7 +2138,7 @@ class Poly(Basic):
                         else:
                             monom = M[:j]+(m+n,)+M[j+1:i]+M[i+1:]
 
-                        if terms.has_key(monom):
+                        if monom in terms:
                             coeff += terms[monom]
 
                             if not coeff:
@@ -2185,4 +2242,3 @@ def multinomial_as_basic(multinomial, *symbols):
         l.append(Mul(*term))
     result = Add(*l)
     return result
-
