@@ -1,4 +1,3 @@
-
 from polynomial import Poly
 
 def LexPoly(*args):
@@ -7,7 +6,7 @@ def LexPoly(*args):
 
 from algorithms import poly_div, poly_pdiv, poly_groebner, poly_lcm, poly_gcd, \
     poly_half_gcdex, poly_gcdex, poly_sqf, poly_resultant, poly_subresultants, \
-    poly_decompose, poly_quo, poly_rem, poly_pquo, poly_prem
+    poly_decompose, poly_quo, poly_rem, poly_pquo, poly_prem, poly_discriminant
 
 from rootfinding import poly_root_factors, poly_sturm
 
@@ -38,12 +37,12 @@ _funcs = {
     'gcd'           : 2,
     'gcdex'         : 2,
     'half_gcdex'    : 2,
-    'subresultants' : 2,
     'resultant'     : 2,
     'sqf'           : 1,
     'decompose'     : 1,
     'root_factors'  : 1,
     'sturm'         : 1,
+    'discriminant'  : 1,
 }
 
 _func_def = \
@@ -69,3 +68,20 @@ def div(*args, **kwargs):
 
 div.__doc__ = poly_div.__doc__
 
+def subresultants(*args, **kwargs):
+    result = poly_subresultants(*_conv_args(2, args), **kwargs)
+
+    if type(result) is tuple:
+        res, R = result
+    else:
+        res, R = None, result
+
+    R = [ r.as_basic() for r in R ]
+
+    if res is None:
+        return R
+    else:
+        return res.as_basic(), R
+
+
+subresultants.__doc__ = poly_subresultants.__doc__
