@@ -12,7 +12,7 @@ TODO:
       top/center/bottom alignment options for left/right
 """
 
-from pretty_symbology import hobj, vobj, pretty_use_unicode
+from pretty_symbology import hobj, vobj, xsym, pretty_use_unicode
 
 class stringPict(object):
     """A ASCII picture.
@@ -84,10 +84,12 @@ class stringPict(object):
         """Put pictures next to this one.
         Returns string, baseline arguments for stringPict.
         (Multiline) strings are allowed, and are given a baseline of 0.
+        >>> from sympy.printing.pretty.stringpict import stringPict
         >>> print stringPict("10").right(" + ",stringPict("1\r-\r2",1))[0]
              1
         10 + -
              2
+
         """
         return stringPict.next(self, *args)
 
@@ -137,16 +139,18 @@ class stringPict(object):
         return '\n'.join(newPicture), newBaseline
 
     def below(self, *args):
-         """Put pictures under this picture.
-         Returns string, baseline arguments for stringPict.
-         Baseline is baseline of top picture
-         >>> print stringPict("x+3").below(stringPict.LINE, '3')[0] #doctest: +NORMALIZE_WHITESPACE
-         x+3
-         ---
-          3
-         """
-         s, baseline = stringPict.stack(self, *args)
-         return s, self.baseline
+        """Put pictures under this picture.
+        Returns string, baseline arguments for stringPict.
+        Baseline is baseline of top picture
+        >>> from sympy.printing.pretty.stringpict import stringPict
+        >>> print stringPict("x+3").below(stringPict.LINE, '3')[0] #doctest: +NORMALIZE_WHITESPACE
+        x+3
+        ---
+         3
+
+        """
+        s, baseline = stringPict.stack(self, *args)
+        return s, self.baseline
 
     def above(self, *args):
         """Put pictures above this picture.
@@ -230,10 +234,15 @@ class stringPict(object):
         root.baseline = result.baseline-result.height()+root.height()
         return result.left(root)
 
-    def terminal_string(self):
-        """Return the string form of self such that it can be printed
+    def render(self, * args, **kwargs):
+        """Return the string form of self.
+
+           Unless the argument line_break is set to False, it will
+           break the expression in a form that can be printed
            on the terminal without being broken up.
-        """
+         """
+        if not kwargs["wrap_line"]:
+            return "\n".join(self.picture)
 
         # Attempt to get a terminal width
         ncols = self.terminal_width()
@@ -245,16 +254,30 @@ class stringPict(object):
         if self.width() <= ncols:
             return type(self.picture[0])(self)
 
+        # for one-line pictures we don't need v-spacers. on the other hand, for
+        # multiline-pictures, we need v-spacers between blocks, compare:
+        #
+        #    2  2        3    | a*c*e + a*c*f + a*d  | a*c*e + a*c*f + a*d  | 3.14159265358979323
+        # 6*x *y  + 4*x*y  +  |                      | *e + a*d*f + b*c*e   | 84626433832795
+        #                     | *e + a*d*f + b*c*e   | + b*c*f + b*d*e + b  |
+        #      3    4    4    |                      | *d*f                 |
+        # 4*y*x  + x  + y     | + b*c*f + b*d*e + b  |                      |
+        #                     |                      |                      |
+        #                     | *d*f
+        do_vspacers = (self.height() > 1)
+
         i = 0
         svals = []
         while i < self.width():
             svals.extend([ sval[i:i+ncols] for sval in self.picture ])
-            svals.append("") # a vertical spacer
+            if (self.height() > 1):
+                svals.append("") # a vertical spacer
             i += ncols
 
-        del svals[-1] #  Get rid of the last spacer
-        _str = type(self.picture[0])
-        return _str.join(_str("\n"), svals)
+        if svals[-1] == '':
+            del svals[-1] #  Get rid of the last spacer
+
+        return "\n".join(svals)
 
     def terminal_width(self):
         """Return the terminal width if possible, otherwise return 0.
@@ -313,13 +336,13 @@ class prettyForm(stringPict):
     """Extension of the stringPict class that knows about
     basic math applications, optimizing double minus signs.
     "Binding" is interpreted as follows:
-    ATOM this is an atom: never needs to be parenthesised
-    FUNC this is a function application: parenthesise if added (?)
+    ATOM this is an atom: never needs to be parenthesized
+    FUNC this is a function application: parenthesize if added (?)
     DIV  this is a division: make wider division if divided
-    POW  this is a power: only parenthesise if exponent
-    MUL  this is a multiplication: parenthesise if powered
-    ADD  this is an addition: parenthesise if multiplied or powered
-    NEG  this is a negative number: optimise if added, parenthesise if multiplied or powered
+    POW  this is a power: only parenthesize if exponent
+    MUL  this is a multiplication: parenthesize if powered
+    ADD  this is an addition: parenthesize if multiplied or powered
+    NEG  this is a negative number: optimize if added, parenthesize if multiplied or powered
     """
     ATOM, FUNC, DIV, POW, MUL, ADD, NEG = range(7)
 
@@ -359,6 +382,9 @@ class prettyForm(stringPict):
             stringPict.LINE,
             den))
 
+    def __truediv__(self, o):
+        return self.__div__(o)
+
     def __mul__(self, *others):
         """Make a pretty multiplication.
         Parentheses are needed around +, - and neg.
@@ -367,13 +393,13 @@ class prettyForm(stringPict):
         if args.binding > prettyForm.MUL: arg = stringPict(*args.parens())
         result = [args]
         for arg in others:
-            result.append('*')
+            result.append(xsym('*'))
             #add parentheses for weak binders
             if arg.binding > prettyForm.MUL: arg = stringPict(*arg.parens())
             result.append(arg)
         len_res = len(result)
         for i in xrange(len_res):
-            if i < len_res-1 and result[i] == '-1' and result[i+1] == "*":
+            if i < len_res-1 and result[i] == '-1' and result[i+1] == xsym('*'):
                 # substitute -1 by -, like in -1*x -> -x
                 result.pop(i)
                 result.pop(i)
