@@ -1,20 +1,21 @@
 # -*- coding: utf-8 -*-
 
 from sympy import symbols, Symbol, sin, cos, Matrix, Integral, pi, sqrt,\
-        Function, Rational, tan, oo, Limit, ceiling, floor, conjugate
-from sympy.printing.pretty import pretty
+        Function, Rational, tan, oo, Limit, ceiling, floor, conjugate, exp, I
+from sympy.printing.pretty import pretty, pprint
 
-x,y = symbols('xy')
+x,y,k = symbols('xyk')
 th  = Symbol('theta')
 ph  = Symbol('phi')
 
 def upretty(expr):
-    return pretty(expr, True)
+    return pretty(expr, use_unicode=True)
 
 def test_upretty_greek():
     assert upretty( oo ) == u'∞'
     assert upretty( Symbol('alpha^+_1') )   ==  u'α⁺₁'
     assert upretty( Symbol('beta') )    == u'β'
+    assert upretty(Symbol('lambda')) == u'λ'
 
 def test_upretty_multiindex():
     assert upretty( Symbol('beta12') )  == u'β₁₂'
@@ -22,6 +23,17 @@ def test_upretty_multiindex():
     assert upretty( Symbol('Y_00') )    == u'Y₀₀'
     assert upretty( Symbol('F^+-') )    == u'F⁺⁻'
 
+def test_upretty_sub_super():
+    assert upretty( Symbol('beta_1_2') )  == u'β₁ ₂'
+    assert upretty( Symbol('beta^1^2') )  == u'β¹ ²'
+    assert upretty( Symbol('beta_1^2') )  == u'β²₁'
+    assert upretty( Symbol('beta_10_20') )  == u'β₁₀ ₂₀'
+    assert upretty( Symbol('beta_ax_gamma^i') )  == u'βⁱₐₓ ᵧ'
+    assert upretty( Symbol("F^1^2_3_4") )  == u'F¹ ²₃ ₄'
+    assert upretty( Symbol("F_1_2^3^4") )  == u'F³ ⁴₁ ₂'
+    assert upretty( Symbol("F_1_2_3_4") )  == u'F₁ ₂ ₃ ₄'
+    assert upretty( Symbol("F^1^2^3^4") )  == u'F¹ ² ³ ⁴'
+
 def test_upretty_subs_missingin_24():
     assert upretty( Symbol('F_beta') )  == u'Fᵦ'
     assert upretty( Symbol('F_gamma') ) == u'Fᵧ'
@@ -39,6 +51,10 @@ def test_upretty_subs_missingin_24():
     assert upretty( Symbol('F_x') )     == u'Fₓ'
 
 
+def test_upretty_nicemul():
+    assert upretty(x*y) ==  u'x⋅y'
+
+
 def test_upretty_nicerat():
     u = upretty(y*x**-2)
     s = \
@@ -175,7 +191,7 @@ def test_upretty_integral():
     u = upretty( Integral(sin(th)/cos(ph), (th,0,pi), (ph, 0, 2*pi)) )
     s = \
 u"""\
-2*π π             \n\
+2⋅π π             \n\
  ⌠  ⌠             \n\
  ⎮  ⎮ sin(θ)      \n\
  ⎮  ⎮ ────── dθ dφ\n\
@@ -191,12 +207,21 @@ u"""\
 π 1                \n\
 ⌠ ⌠                \n\
 ⎮ ⎮  2             \n\
-⎮ ⎮ x *sin(y) dx dy\n\
+⎮ ⎮ x ⋅sin(y) dx dy\n\
 ⌡ ⌡                \n\
 0 0                \
 """
     assert u == s
 
+    u = upretty( Integral(sin(x), (x, None, 1)) )
+    s = \
+u"""1          \n\
+⌠          \n\
+⎮ sin(x) dx\n\
+⌡          \n\
+           \
+"""
+    assert u == s
 
 def test_upretty_limit():
     u = upretty( Limit(x, x, oo) )
@@ -240,24 +265,42 @@ def test_upretty_matrix():
     s1 = \
 u"""\
 ⎡     2       ⎤
-⎢1 + x       1⎥
+⎢1 + x     1  ⎥
 ⎢             ⎥
-⎣     y  x + y⎦\
+⎣  y     x + y⎦\
 """
     s2 = \
 u"""\
 ⎡ 2           ⎤
-⎢x  + 1      1⎥
+⎢x  + 1    1  ⎥
 ⎢             ⎥
-⎣     y  y + x⎦\
+⎣  y     y + x⎦\
 """
     assert u in [s1, s2]
 
 
+def test_upretty_matrix2():
+    m = Matrix([[x/y, y, th], [0, exp(I*k*ph), 1]])
+    u = upretty(m)
+    s = \
+u"""\
+⎡x           ⎤
+⎢─    y     θ⎥
+⎢y           ⎥
+⎢            ⎥
+⎢    ⅈ⋅k⋅φ   ⎥
+⎣0  ℯ       1⎦\
+"""
+    assert u == s
+
+
+
 def test_upretty_seq():
     assert upretty([]) == '[]'
     assert upretty(()) == '()'
     assert upretty({}) == '{}'
+    assert upretty(set()) == 'set()'
+    assert upretty(frozenset()) == 'frozenset()'
 
     e = [x**2, 1/x, x, y, sin(th)**2/cos(ph)**2]
     u = upretty(e)
@@ -295,18 +338,29 @@ u"""\
 """
     assert u == s
 
-    e = dict(enumerate(e))
-    u = upretty(e)
+    e_= dict(enumerate(e))
+    u = upretty(e_)
     s = \
 u"""\
 ⎧                               2   ⎫
-⎪    2     1                 sin (θ)⎪
-⎨0: x , 1: ─, 2: x, 3: y, 4: ───────⎬
-⎪          x                    2   ⎪
+⎪   1      2                 sin (θ)⎪
+⎨1: ─, 0: x , 2: x, 3: y, 4: ───────⎬
+⎪   x                           2   ⎪
 ⎩                            cos (φ)⎭\
 """
     assert u == s
 
+    e_= set(e)
+    u = upretty(e_)
+    s = \
+u"""\
+   ⎛                2   ⎞
+   ⎜      1   2  sin (θ)⎟
+set⎜x, y, ─, x , ───────⎟
+   ⎜      x         2   ⎟
+   ⎝             cos (φ)⎠\
+"""
+
     e = {x: sin(x)}
     u = upretty(e)
     s = \
@@ -317,21 +371,13 @@ u"""\
 
     e = {1/x: 1/y, x: sin(x)**2}
     u = upretty(e)
-    s1 = \
-u"""\
-⎧1  1        2   ⎫
-⎨─: ─, x: sin (x)⎬
-⎩x  y            ⎭\
-"""
-
-    s2 = \
+    s = \
 u"""\
 ⎧      2     1  1⎫
 ⎨x: sin (x), ─: ─⎬
 ⎩            x  y⎭\
 """
-    # NOTE dict does not guaranty ordering
-    assert (u == s1) or (u == s2)
+    assert u == s
 
 
 def test_upretty_seq_even():
@@ -361,3 +407,13 @@ u"""\
 """
     assert u == s
 
+def test_upprint():
+    import StringIO, sys
+    fd = StringIO.StringIO()
+    sso = sys.stdout
+    sys.stdout = fd
+    try:
+        pprint(pi, use_unicode=True)
+    finally:
+        sys.stdout = sso
+    assert fd.getvalue() == u'\u03c0\n'
