@@ -1,9 +1,8 @@
 from sympy.core.basic import Basic, S, C, sympify
 from sympy.core import oo, Rational, Pow
-from sympy.core.methods import ArithMeths, RelMeths
 from sympy.core.cache import cacheit
 
-class Order(Basic, ArithMeths, RelMeths):
+class Order(Basic):
     """
     Represents O(f(x)) at the point x = 0.
 
@@ -40,8 +39,8 @@ class Order(Basic, ArithMeths, RelMeths):
 
     Examples:
     =========
-    >>> from sympy import *
-    >>> x = Symbol("x")
+    >>> from sympy import O
+    >>> from sympy.abc import x
     >>> O(x)
     O(x)
     >>> O(x)*x
@@ -77,15 +76,13 @@ class Order(Basic, ArithMeths, RelMeths):
       all symbols in the expression.
     """
 
-    precedence = Basic.Apply_precedence
-
     is_Order = True
 
     __slots__ = []
 
     @cacheit
     def __new__(cls, expr, *symbols, **assumptions):
-        expr = sympify(expr).expand(trig=True)
+        expr = sympify(expr).expand()
         if expr is S.NaN:
             return S.NaN
 
@@ -151,10 +148,15 @@ class Order(Basic, ArithMeths, RelMeths):
 
         return obj
 
+    def _hashable_content(self):
+        if self.args[0].is_number:
+            return (self.args[0],)
+        return self.args
+
     def oseries(self, order):
         return self
 
-    def nseries(self, x, x0, n):
+    def _eval_nseries(self, x, x0, n):
         return self
 
     @classmethod
@@ -211,15 +213,6 @@ class Order(Basic, ArithMeths, RelMeths):
     def symbols(self):
         return self._args[1:]
 
-    def tostr(self, level = 0):
-        if len(self.symbols) <= 1:
-            r = 'O(%s)' % self.expr.tostr()
-        else:
-            r = 'O(%s)' % (', '.join([s.tostr() for s in self.args]))
-        if self.precedence <= level:
-            r = '(%s)' % (r)
-        return r
-
     def _eval_power(b, e):
         if e.is_Number:
             return Order(b.expr ** e, *b.symbols)
@@ -242,8 +235,11 @@ class Order(Basic, ArithMeths, RelMeths):
         Return None if the inclusion relation cannot be determined (e.g. when self and
         expr have different symbols).
         """
+        from sympy import powsimp
         if expr is S.Zero:
             return True
+        if expr is S.NaN:
+            return False
         if expr.is_Order:
             if self.symbols and expr.symbols:
                 common_symbols = tuple([s for s in self.symbols if s in expr.symbols])
@@ -257,7 +253,8 @@ class Order(Basic, ArithMeths, RelMeths):
                 return None
             r = None
             for s in common_symbols:
-                l = Order.find_limit(self.expr/expr.expr, s) != 0
+                l = Order.find_limit(powsimp(self.expr/expr.expr, deep=True,\
+                combine='exp'), s) != 0
                 if r is None:
                     r = l
                 else:
@@ -273,9 +270,9 @@ class Order(Basic, ArithMeths, RelMeths):
         if isinstance(old, C.Symbol) and old in self.symbols:
             i = list(self.symbols).index(old)
             if isinstance(new, C.Symbol):
-                return Order(self.expr.subs(old, new), *(self.symbols[:i]+(new,)+self.symbols[i+1:]))
-            return Order(self.expr.subs(old, new), *(self.symbols[:i]+self.symbols[i+1:]))
-        return Order(self.expr.subs(old, new), *self.symbols)
+                return Order(self.expr._eval_subs(old, new), *(self.symbols[:i]+(new,)+self.symbols[i+1:]))
+            return Order(self.expr._eval_subs(old, new), *(self.symbols[:i]+self.symbols[i+1:]))
+        return Order(self.expr._eval_subs(old, new), *self.symbols)
 
     def _sage_(self):
         #XXX: SAGE doesn't have Order yet. Let's return 0 instead.
