@@ -1,11 +1,12 @@
-"""The module helps converting sympy expressions into shroter forms of them.
+"""The module helps converting sympy expressions into shorter forms of them.
 
 for example:
 the expression E**(pi*I) will be converted into -1
 the expression (x+x)**2 will be converted into 4*x**2
 """
 from simplify import collect, separate, together, radsimp, ratsimp, fraction, \
-    simplify, trigsimp, powsimp, combsimp, hypersimp, hypersimilar, nsimplify
+    simplify, trigsimp, powsimp, combsimp, hypersimp, hypersimilar, nsimplify, \
+    logcombine, separatevars
 
 from rewrite import cancel, trim, apart
 
