@@ -1,22 +1,26 @@
 """Module 'rewrite.py' contains advanced term rewriting methods concerning
-   partial fraction decomposition, combinig together and collecting terms.
+   partial fraction decomposition, combining together and collecting terms.
 """
 
 from sympy.core.add import Add
 from sympy.core.mul import Mul
 from sympy.core.power import Pow
-from sympy.core.function import Function
 from sympy.core.relational import Relational
 from sympy.core.symbol import Symbol, Temporary
 from sympy.core.numbers import Integer, Rational
+from sympy.core.function import Function, Lambda
 from sympy.core.basic import Basic, S, C, Atom, sympify
 
-from sympy.polynomials import factor_, div, quo, rem, gcd, egcd
+from sympy.utilities import threaded
 from sympy.simplify import together
-from sympy.matrices import Matrix
 from sympy.functions import exp
 
-def cancel(f, *syms, **flags):
+from sympy.polys import Poly, RootSum, div, quo, gcd
+from sympy.polys.algorithms import poly_quo, poly_rem, \
+    poly_sqf, poly_gcd, poly_half_gcdex
+
+@threaded()
+def cancel(f, *symbols):
     """Cancel common factors in a given rational function.
 
        Given a quotient of polynomials, performing only gcd and quo
@@ -29,10 +33,10 @@ def cancel(f, *syms, **flags):
        thread over sums and relational operators.
 
        Additionally you can specify a list of variables to perform
-       cancelation more efficiently using only those symbols.
+       cancellation more efficiently using only those symbols.
 
-       >>> from sympy import *
-       >>> x,y = symbols('xy')
+       >>> from sympy import cancel, sqrt
+       >>> from sympy.abc import x, y
 
        >>> cancel((x**2-1)/(x-1))
        1 + x
@@ -44,42 +48,14 @@ def cancel(f, *syms, **flags):
        x - 2**(1/2)
 
     """
-    f = sympify(f)
+    return Poly.cancel(f, *symbols)
 
-    if syms and not f.has(*syms):
-        return f
-    elif f.is_Add:
-        return Add(*[ cancel(g, *syms, **flags) for g in f.args ])
-    elif isinstance(f, Relational):
-        return Relational(cancel(f.lhs, *syms, **flags),
-            cancel(f.rhs, *syms, **flags), f.rel_op)
-    else:
-        g = together(f)
-
-        if not g.is_fraction(*syms):
-            return f
-        else:
-            p, q = g.as_numer_denom()
-
-            p = p.expand()
-            q = q.expand()
-
-            syms = syms or None
-
-            g = gcd(p, q, syms)
-
-            if g is not S.One:
-                p = quo(p, g, syms)
-                q = quo(q, g, syms)
-
-            return p / q
-
-def trim(f, *syms, **flags):
+def trim(f, *symbols, **flags):
     """Cancel common factors in a given formal rational expression.
 
        Given an arbitrary expression, map all functional components
        to temporary symbols, rewriting this expression to rational
-       function form and perform cancelation of common factors.
+       function form and perform cancellation of common factors.
 
        When given a rational function or a list of symbols discards
        all functional components, then this procedure is equivalent
@@ -87,12 +63,12 @@ def trim(f, *syms, **flags):
 
        Note that this procedure can thread over composite objects
        like big operators, matrices, relational operators etc. It
-       can be also called recursively (to change this behaviour
+       can be also called recursively (to change this behavior
        unset 'recursive' flag).
 
-       >>> from sympy import *
+       >>> from sympy import Function, trim, sin
 
-       >>> x,y = symbols('xy')
+       >>> from sympy.abc import x, y
        >>> f = Function('f')
 
        >>> trim((f(x)**2+f(x))/f(x))
@@ -107,17 +83,19 @@ def trim(f, *syms, **flags):
        sin(1 + f(x))
 
     """
+    f = sympify(f)
+
     if isinstance(f, Relational):
-        return Relational(trim(f.lhs, *syms, **flags),
-            trim(f.rhs, *syms, **flags), f.rel_op)
-    elif isinstance(f, Matrix):
-        return f.applyfunc(lambda g: trim(g, *syms, **flags))
+        return Relational(trim(f.lhs, *symbols, **flags),
+                          trim(f.rhs, *symbols, **flags), f.rel_op)
+    #elif isinstance(f, Matrix):
+    #    return f.applyfunc(lambda g: trim(g, *symbols, **flags))
     else:
         recursive = flags.get('recursive', True)
 
         def is_functional(g):
             return not (g.is_Atom or g.is_number) \
-                and (not syms or g.has(*syms))
+                and (not symbols or g.has(*symbols))
 
         def components(g):
             result = set()
@@ -132,10 +110,10 @@ def trim(f, *syms, **flags):
                         result |= terms
                         args.append(h)
 
-                    g = g.__class__(*args)
+                    g = g.func(*args)
                 elif g.is_Pow:
                     if recursive:
-                        base = trim(g.base, *syms, **flags)
+                        base = trim(g.base, *symbols, **flags)
                     else:
                         base = g.base
 
@@ -152,7 +130,7 @@ def trim(f, *syms, **flags):
                         g = base**g.exp
                     else:
                         if recursive:
-                            h = g = base**trim(g.exp, *syms, **flags)
+                            h = g = base**trim(g.exp, *symbols, **flags)
                         else:
                             h = g = base**g.exp
 
@@ -162,24 +140,24 @@ def trim(f, *syms, **flags):
                     if not recursive:
                         result.add(g)
                     else:
-                        g = g.__class__(*[trim(h, *syms,
-                            **flags) for h in g.args])
+                        g = g.func(*[trim(h, *symbols, **flags)
+                                          for h in g.args])
 
                         if is_functional(g):
                             result.add(g)
 
             return g, result
 
-        f = sympify(f)
-
-        if f.is_number or (syms and not f.has(*syms)):
+        if not isinstance(f, Basic) \
+           or f.is_number \
+           or not f.has_any_symbols(*symbols):
             return f
 
         f = together(f.expand())
         f, terms = components(f)
 
         if not terms:
-            return cancel(f, *syms)
+            return Poly.cancel(f, *symbols)
         else:
             mapping, reverse = {}, {}
 
@@ -190,23 +168,26 @@ def trim(f, *syms, **flags):
             p, q = f.as_numer_denom()
             f = p.expand()/q.expand()
 
-            H = cancel(f.subs(mapping), *syms)
+            if not symbols:
+                symbols = tuple(f.atoms(Symbol))
+
+            symbols = tuple(mapping.values()) + symbols
+
+            H = Poly.cancel(f.subs(mapping), *symbols)
 
             if not flags.get('extract', True):
                 return H.subs(reverse)
             else:
-                syms = syms or None
-
                 def extract(f):
                     p = f.args[0]
 
                     for q in f.args[1:]:
-                        p = gcd(p, q, syms)
+                        p = gcd(p, q, *symbols)
 
                         if p.is_number:
                             return S.One, f
 
-                    return p, Add(*[quo(g, p, syms) for g in f.args])
+                    return p, Add(*[quo(g, p, *symbols) for g in f.args])
 
                 P, Q = H.as_numer_denom()
 
@@ -222,108 +203,93 @@ def trim(f, *syms, **flags):
 
                 return ((GP*P)/(GQ*Q)).subs(reverse)
 
+@threaded()
 def apart(f, z, **flags):
     """Compute partial fraction decomposition of a rational function.
 
-       Given an rational function, performing only gcd operations over
-       the algebraic closue of its field of definition, compute full
-       partial fraction decomposition with fractions having linear
-       denominators.
+       Given a rational function 'f', performing only gcd operations
+       over the algebraic closure of the initial field of definition,
+       compute full partial fraction decomposition with fractions
+       having linear denominators.
 
        For all other kinds of expressions the input is returned in an
        unchanged form. Note however, that 'apart' function can thread
        over sums and relational operators.
 
-       Note that no factorization of the initial denominator is needed,
-       howevert at some point root finding algorithms are executed but
-       for polynomials of much lower degree than the denominator.
+       Note that no factorization of the initial denominator of 'f' is
+       needed.  The final decomposition is formed in terms of a sum of
+       RootSum instances.  By default RootSum tries to compute all its
+       roots to simplify itself. This behavior can be however avoided
+       by setting the keyword flag evaluate=False, which will make this
+       function return a formal decomposition.
 
-       If given an additional flag 'formal', then even root finding
-       algorithms are avoided and the result is formed as a combination
-       of formal summations over implicit roots of some polynomials.
+       >>> from sympy import apart
+       >>> from sympy.abc import x, y
 
-       >>> from sympy import *
-       >>> x,y = symbols('xy')
+       >>> apart(y/(x+2)/(x+1), x)
+       y/(1 + x) - y/(2 + x)
 
-       >>> apart(1/(x+2)/(x+1), x)
-       1/(1 + x) - 1/(2 + x)
-
-       >>> apart(Eq((x+1)/(x-1), E/x), x)
-       1 - 2/(1 - x) == E/x
+       >>> apart(1/(1+x**5), x, evaluate=False)
+       RootSum(Lambda(_a, -_a/(5*(x - _a))), x**5 + 1, x)
 
        For more information on the implemented algorithm refer to:
 
        [1] M. Bronstein, B. Salvy, Full partial fraction decomposition
-           of rational functions, in: M. Bronstein, ed., Proceedings
+           of rational functions,  in: M. Bronstein,  ed., Proceedings
            ISSAC '93, ACM Press, Kiev, Ukraine, 1993, pp. 157-160.
 
     """
-    f = sympify(f)
+    if not f.has(z):
+        return f
 
-    if f.is_Add:
-        return Add(*[ apart(g, z, **flags) for g in f ])
-    elif isinstance(f, Relational):
-        return Relational(apart(f.lhs, z, **flags),
-            apart(f.rhs, z, **flags), f.rel_op)
-    else:
-        if not f.has(z):
-            return f
+    f = Poly.cancel(f, z)
 
-        if f.is_fraction(z):
-            f = cancel(f, z)
-        else:
-            return f
+    P, Q = f.as_numer_denom()
 
-        P, Q = f.as_numer_denom()
+    if not Q.has(z):
+        return f
 
-        if not Q.has(z):
-            return f
+    partial, r = div(P, Q, z)
+    f, q, U = r / Q, Q, []
 
-        partial, r = div(P, Q, z)
-        f, q, U = r / Q, Q, []
+    u = Function('u')(z)
+    a = Symbol('a', dummy=True)
 
-        u = Function('u')(z)
-        A = Symbol('a', dummy=True)
+    for k, d in enumerate(poly_sqf(q, z)):
+        n, b = k + 1, d.as_basic()
+        U += [ u.diff(z, k) ]
 
-        formal = flags.get('formal', False)
+        h = together(Poly.cancel(f*b**n, z) / u**n)
 
-        for k, d in enumerate(factor_.sqf(q, z)):
-            n, d = k + 1, d.as_basic()
-            U += [ u.diff(z, k) ]
+        H, subs = [h], []
 
-            h = together(cancel(f * d**n, z) / u**n)
+        for j in range(1, n):
+            H += [ H[-1].diff(z) / j ]
 
-            H, subs = [h], []
+        for j in range(1, n+1):
+            subs += [ (U[j-1], b.diff(z, j) / j) ]
 
-            for j in range(1, n):
-                H += [ H[-1].diff(z) / j ]
+        for j in range(0, n):
+            P, Q = together(H[j]).as_numer_denom()
 
-            for j in range(1, n+1):
-                subs += [ (U[j-1], d.diff(z, j) / j) ]
+            for i in range(0, j+1):
+                P = P.subs(*subs[j-i])
 
-            for j in range(0, n):
-                P, Q = together(H[j]).as_numer_denom()
+            Q = Q.subs(*subs[0])
 
-                for i in range(0, j+1):
-                    P = P.subs(*subs[j-i])
+            P, Q = Poly(P, z), Poly(Q, z)
 
-                Q = Q.subs(*subs[0])
+            G = poly_gcd(P, d)
+            D = poly_quo(d, G)
 
-                G = gcd(P, d, z)
-                D = quo(d, G, z)
+            B, g = poly_half_gcdex(Q, D)
+            b = poly_rem(P * poly_quo(B, g), D)
 
-                g, B, _ = egcd(Q, D, z)
-                b = rem(P * B / g, D, z)
+            numer = b.as_basic()
+            denom = (z-a)**(n-j)
 
-                denom = (z - A)**(n-j)
+            expr = numer.subs(z, a) / denom
 
-                rootof = C.RootOf(D, z)
+            partial += RootSum(Lambda(a, expr), D, **flags)
 
-                if not formal:
-                    for root in rootof.roots():
-                        partial += cancel(b.subs(z, root)) \
-                            / denom.subs(A, root)
-                else:
-                    partial += C.Sum(b.subs(z, A) / denom, (A, rootof))
-
-        return partial
+    return partial
