@@ -1,8 +1,9 @@
 from sympy import Symbol, symbols, together, hypersimp, factorial, binomial, \
         collect, Function, powsimp, separate, sin, exp, Rational, fraction, \
         simplify, trigsimp, cos, tan, cot, log, ratsimp, Matrix, pi, integrate, \
-        solve, nsimplify, GoldenRatio, sqrt, E, I, sympify, atan, Derivative, S
-
+        solve, nsimplify, GoldenRatio, sqrt, E, I, sympify, atan, Derivative, \
+        S, diff, oo, Eq, Integer, gamma, acos, Integral, logcombine, separatevars
+from sympy.utilities import all
 from sympy.utilities.pytest import XFAIL
 
 def test_ratsimp():
@@ -50,6 +51,12 @@ def test_trigsimp1():
     assert trigsimp(5*cos(x/2)**2 + 2*sin(x/2)**2) in \
                 [2 + 3*cos(x/2)**2, 5 - 3*sin(x/2)**2]
 
+    assert trigsimp(sin(x)/cos(x)) == tan(x)
+    assert trigsimp(2*tan(x)*cos(x)) == 2*sin(x)
+    assert trigsimp(cot(x)**3*sin(x)**3) == cos(x)**3
+    assert trigsimp(y*tan(x)**2/sin(x)**2) == y/cos(x)**2
+    assert trigsimp(cot(x)/cos(x)) == 1/sin(x)
+
     assert trigsimp(cos(0.12345)**2 + sin(0.12345)**2) == 1
     e = 2*sin(x)**2 + 2*cos(x)**2
     assert trigsimp(log(e), deep=True) == log(2)
@@ -90,6 +97,8 @@ def test_factorial_simplify():
 def test_simplify():
     x,y,z,k,n,m,w,f,s,A = symbols('xyzknmwfsA')
 
+    assert all(simplify(tmp)==tmp for tmp in [I,E,oo,x,-x,-oo,-E,-I])
+
     e = 1/x + 1/y
     assert e != (x+y)/(x*y)
     assert simplify(e) == (x+y)/(x*y)
@@ -140,8 +149,7 @@ def test_simplify():
 def test_simplify_issue_1308():
     assert simplify(exp(-Rational(1,2)) + exp(-Rational(3,2))) == \
         (1 + E)*exp(-Rational(3,2))
-    assert simplify(exp(1)+exp(-exp(1))) == \
-        (1 + exp(1 + E))*exp(-E)
+    assert simplify(exp(1)+exp(-exp(1))) == (1 + exp(1 + E))*exp(-E)
 
 
 def test_simplify_fail1():
@@ -214,21 +222,49 @@ def test_separate():
     assert separate((exp((x*y)**z)*exp(y))**2) == exp(2*(x*y)**z)*exp(2*y)
     assert separate((exp((x*y)**z)*exp(y))**2, deep=True) == exp(2*x**z*y**z)*exp(2*y)
 
-@XFAIL
 def test_separate_X1():
+    x, y, z = map(Symbol, 'xyz')
     assert separate((exp(x)*exp(y))**z) == exp(x*z)*exp(y*z)
 
 def test_powsimp():
-    x,y,n = symbols('xyn')
+    x,y,z,n = symbols('xyzn')
     f = Function('f')
     assert powsimp( 4**x * 2**(-x) * 2**(-x) ) == 1
     assert powsimp( (-4)**x * (-2)**(-x) * 2**(-x) ) == 1
 
     assert powsimp( f(4**x * 2**(-x) * 2**(-x)) )   == f(4**x * 2**(-x) * 2**(-x))
     assert powsimp( f(4**x * 2**(-x) * 2**(-x)), deep = True )  == f(1)
+    assert exp(x)*exp(y) == exp(x)*exp(y)
+    assert powsimp(exp(x)*exp(y)) == exp(x+y)
+    assert powsimp(exp(x)*exp(y)*2**x*2**y) == (2*E)**(x + y)
+    assert powsimp(exp(x)*exp(y)*2**x*2**y, combine='exp') == exp(x+y)*2**(x+y)
+    assert powsimp(exp(x)*exp(y)*exp(2)*sin(x)+sin(y)+2**x*2**y) == exp(2+x+y)*sin(x)+sin(y)+2**(x+y)
+    assert powsimp(sin(exp(x)*exp(y))) == sin(exp(x)*exp(y))
+    assert powsimp(sin(exp(x)*exp(y)), deep=True) == sin(exp(x+y))
+    assert powsimp(x**2*x**y) == x**(2+y)
+    # This should remain factored, because 'exp' with deep=True is supposed
+    # to act like old automatic exponent combining.
+    assert powsimp((1 + E*exp(E))*exp(-E), combine='exp', deep=True) == (1 + exp(1 + E))*exp(-E)
+    assert powsimp((1 + E*exp(E))*exp(-E), deep=True) == exp(1) + exp(-E)
+    # This should not change without deep.  Otherwise, simplify() will fail.
+    assert powsimp((1 + E*exp(E))*exp(-E)) == (1 + E*exp(E))*exp(-E)
+    assert powsimp((1 + E*exp(E))*exp(-E), combine='exp') == (1 + E*exp(E))*exp(-E)
+    assert powsimp((1 + E*exp(E))*exp(-E), combine='base') == (1 + E*exp(E))*exp(-E)
     x,y = symbols('xy', nonnegative=True)
     n = Symbol('n', real=True)
     assert powsimp( y**n * (y/x)**(-n) ) == x**n
+    assert powsimp(x**(x**(x*y)*y**(x*y))*y**(x**(x*y)*y**(x*y)),deep=True) == (x*y)**(x*y)**(x*y)
+    assert powsimp(2**(2**(2*x)*x), deep=False) == 2**(2**(2*x)*x)
+    assert powsimp(2**(2**(2*x)*x), deep=True) == 2**(x*4**x)
+    assert powsimp(exp(-x + exp(-x)*exp(-x*log(x))), deep=False, combine='exp') == exp(-x + exp(-x)*exp(-x*log(x)))
+    assert powsimp(exp(-x + exp(-x)*exp(-x*log(x))), deep=False, combine='exp') == exp(-x + exp(-x)*exp(-x*log(x)))
+    assert powsimp((x+y)/(3*z), deep=False, combine='exp') == (x+y)/(3*z)
+    assert powsimp((x/3+y/3)/z, deep=True, combine='exp') == (x/3+y/3)/z
+    assert powsimp(exp(x)/(1 + exp(x)*exp(y)), deep=True) == exp(x)/(1 + exp(x + y))
+    assert powsimp(x*y**(z**x*z**y), deep=True) == x*y**(z**(x + y))
+    assert powsimp((z**x*z**y)**x, deep=True) == (z**(x + y))**x
+    assert powsimp(x*(z**x*z**y)**x, deep=True) == x*(z**(x + y))**x
+
 
 def test_collect_1():
     """Collect with respect to a Symbol"""
@@ -265,7 +301,7 @@ def test_collect_3():
     assert collect(a*x*log(x) + b*(x*log(x)), x*log(x)) == x*(a + b)*log(x)
     assert collect(a*x**2*log(x)**2 + b*(x*log(x))**2, x*log(x)) == x**2*log(x)**2*(a + b)
 
-    # with respect to a product of tree symbols
+    # with respect to a product of three symbols
     assert collect(y*x*z+a*x*y*z, x*y*z) == (1 + a)*x*y*z
 
 def test_collect_4():
@@ -293,6 +329,19 @@ def test_collect_D():
     assert collect(a*fx + b*fx, fx) == (a + b)*fx
     assert collect(a*D(fx,x) + b*D(fx,x), fx)   == (a + b)*D(fx, x)
     assert collect(a*fxx     + b*fxx    , fx)   == (a + b)*D(fx, x)
+    # 1685
+    assert collect(5*f(x)+3*fx, fx) == 5*f(x) + 3*fx
+    assert collect(f(x) + f(x)*diff(f(x), x) + x*diff(f(x), x)*f(x), f(x).diff(x)) ==\
+    (x*f(x) + f(x))*D(f(x), x) + f(x)
+    assert collect(f(x) + f(x)*diff(f(x), x) + x*diff(f(x), x)*f(x), f(x).diff(x), exact=True) ==\
+    (x*f(x) + f(x))*D(f(x), x) + f(x)
+    assert collect(1/f(x) + 1/f(x)*diff(f(x), x) + x*diff(f(x), x)/f(x), f(x).diff(x), exact=True) ==\
+    (1/f(x) + x/f(x))*D(f(x), x) + 1/f(x)
+
+@XFAIL
+def collect_issues():
+    assert collect(1/f(x) + 1/f(x)*diff(f(x), x) + x*diff(f(x), x)/f(x), f(x).diff(x)) !=\
+    (1 + x*D(f(x), x) + D(f(x), x))/f(x)
 
 def test_collect_D_0():
     D = Derivative
@@ -304,6 +353,32 @@ def test_collect_D_0():
     #                                           -- (a + b)*D(D(f,x), x)
     assert collect(a*fxx     + b*fxx    , fxx)  == (a + b)*fxx
 
+def test_separatevars():
+    x,y,z,n = symbols('xyzn')
+    assert separatevars(2*n*x*z+2*x*y*z) == 2*x*z*(n+y)
+    assert separatevars(x*z+x*y*z) == x*z*(1+y)
+    assert separatevars(pi*x*z+pi*x*y*z) == pi*x*z*(1+y)
+    assert separatevars(x*y**2*sin(x) + x*sin(x)*sin(y)) == x*(sin(y) + y**2)*sin(x)
+    assert separatevars(x*exp(x+y)+x*exp(x)) == x*(1 + exp(y))*exp(x)
+    assert separatevars((x*(y+1))**z) == x**z*(1 + y)**z
+    assert separatevars(1+x+y+x*y) == (x+1)*(y+1)
+    assert separatevars(y / pi * exp(-(z - x) / cos(n))) == y * exp((x - z) / cos(n)) / pi
+    # 1759
+    p=Symbol('p',positive=True)
+    assert separatevars(sqrt(p**2 + x*p**2)) == p*sqrt(1 + x)
+    assert separatevars(sqrt(y*(p**2 + x*p**2))) == p*sqrt(y)*sqrt(1 + x)
+
+@XFAIL
+def test_separatevars_advanced_factor():
+    # If factor() is ever improved to factor non-symbolic expressions, this
+    # should XPASS
+    assert separatevars(1 + log(x)*log(y) + log(x) + log(y)) == (log(x) + 1)*(log(y) + 1)
+    assert separatevars(1 + x - log(z) - x*log(z) - exp(y)*log(z) - \
+        x*exp(y)*log(z) + x*exp(y) + exp(y)) == \
+        (1 + x)*(1 - log(z))*(1 + exp(y))
+    x, y = symbols('xy', positive=True)
+    assert separatevars(1 + log(x**log(y)) + log(x*y)) == (log(x) + 1)*(log(y) + 1)
+
 def test_hypersimp():
     n, k = symbols('nk', integer=True)
 
@@ -353,3 +428,76 @@ def test_nsimplify():
     assert nsimplify(0.33333, tolerance=1e-4) == Rational(1,3)
     assert nsimplify(2.0**(1/3.), tolerance=0.001) == Rational(635,504)
     assert nsimplify(2.0**(1/3.), tolerance=0.001, full=True) == 2**Rational(1,3)
+
+def test_extract_minus_sign():
+    x = Symbol("x")
+    y = Symbol("y")
+    a = Symbol("a")
+    b = Symbol("b")
+    assert simplify(-x/-y) == x/y
+    assert simplify(-x/y) == -x/y
+    assert simplify(x/y) == x/y
+    assert simplify(x/-y) == -x/y
+    assert simplify(-x/0) == -oo*x
+    assert simplify(S(-5)/0) == -oo
+    assert simplify(-a*x/(-y-b)) == a*x/(b + y)
+
+def test_diff():
+    x = Symbol("x")
+    y = Symbol("y")
+    f = Function("f")
+    g = Function("g")
+    assert simplify(g(x).diff(x)*f(x).diff(x)-f(x).diff(x)*g(x).diff(x)) == 0
+    assert simplify(2*f(x)*f(x).diff(x)-diff(f(x)**2,x)) == 0
+    assert simplify(diff(1/f(x),x)+f(x).diff(x)/f(x)**2) == 0
+    assert simplify(f(x).diff(x,y)-f(x).diff(y,x)) == 0
+
+def test_logcombine_1():
+    x, y = symbols("xy")
+    a = Symbol("a")
+    z, w = symbols("zw", positive=True)
+    b = Symbol("b", real=True)
+    assert logcombine(log(x)+2*log(y)) == log(x) + 2*log(y)
+    assert logcombine(log(x)+2*log(y), assume_pos_real=True) == log(x*y**2)
+    assert logcombine(a*log(w)+log(z)) == a*log(w) + log(z)
+    assert logcombine(b*log(z)+b*log(x)) == log(z**b) + b*log(x)
+    assert logcombine(b*log(z)-log(w)) == log(z**b/w)
+    assert logcombine(log(x)*log(z)) == log(x)*log(z)
+    assert logcombine(log(w)*log(x)) == log(w)*log(x)
+    assert logcombine(cos(-2*log(z)+b*log(w))) == cos(log(w**b/z**2))
+    assert logcombine(log(log(x)-log(y))-log(z), assume_pos_real=True) == \
+        log(log((x/y)**(1/z)))
+    assert logcombine((2+I)*log(x), assume_pos_real=True) == I*log(x)+log(x**2)
+    assert logcombine((x**2+log(x)-log(y))/(x*y), assume_pos_real=True) == \
+        log(x**(1/(x*y))*y**(-1/(x*y)))+x/y
+    assert logcombine(log(x)*2*log(y)+log(z), assume_pos_real=True) == \
+        log(z*y**log(x**2))
+    assert logcombine((x*y+sqrt(x**4+y**4)+log(x)-log(y))/(pi*x**Rational(2,3)*\
+        y**Rational(3,2)), assume_pos_real=True) == \
+        log(x**(1/(pi*x**Rational(2,3)*y**Rational(3,2)))*y**(-1/(pi*\
+        x**Rational(2,3)*y**Rational(3,2)))) + (x**4 + y**4)**Rational(1,2)/(pi*\
+        x**Rational(2,3)*y**Rational(3,2)) + x**Rational(1,3)/(pi*y**Rational(1,2))
+    assert logcombine(Eq(log(x), -2*log(y)), assume_pos_real=True) == \
+        Eq(log(x*y**2), Integer(0))
+    assert logcombine(Eq(y, x*acos(-log(x/y))), assume_pos_real=True) == \
+        Eq(y, x*acos(log(y/x)))
+    assert logcombine(gamma(-log(x/y))*acos(-log(x/y)), assume_pos_real=True) == \
+        acos(log(y/x))*gamma(log(y/x))
+    assert logcombine((2+3*I)*log(x), assume_pos_real=True) == \
+        log(x**2)+3*I*log(x)
+    assert logcombine(Eq(y, -log(x)), assume_pos_real=True) == Eq(y, log(1/x))
+    assert logcombine(Integral((sin(x**2)+cos(x**3))/x,x), assume_pos_real=True) == \
+        Integral((sin(x**2)+cos(x**3))/x,x)
+    assert logcombine(Integral((sin(x**2)+cos(x**3))/x,x)+ (2+3*I)*log(x), \
+        assume_pos_real=True) == log(x**2)+3*I*log(x) + \
+        Integral((sin(x**2)+cos(x**3))/x,x)
+
+@XFAIL
+def test_logcombine_2():
+    # The same as one of the tests above, but with Rational(a,b) replaced with a/b.
+    # This fails because of a bug in matches.  See issue 1274.
+    x, y = symbols("xy")
+    assert logcombine((x*y+sqrt(x**4+y**4)+log(x)-log(y))/(pi*x**(2/3)*y**(3/2)), \
+        assume_pos_real=True) == log(x**(1/(pi*x**(2/3)*y**(3/2)))*y**(-1/\
+        (pi*x**(2/3)*y**(3/2)))) + (x**4 + y**4)**(1/2)/(pi*x**(2/3)*y**(3/2)) + \
+        x**(1/3)/(pi*y**(1/2))
