@@ -1,5 +1,4 @@
-
-""" This module contain solvers for all kinds of equations:
+﻿""" This module contain solvers for all kinds of equations:
 
     - algebraic, use solve()
 
@@ -9,37 +8,43 @@
 
     - transcendental, use tsolve()
 
-    - nonlinear (numerically), use msolve() (you will need a good starting point)
+    - nonlinear (numerically), use nsolve()
+      (you will need a good starting point)
 
 """
 
 from sympy.core.sympify import sympify
-from sympy.core.basic import Basic, S, C, Mul, Add
+from sympy.core.basic import S, Mul
+from sympy.core.add import Add
 from sympy.core.power import Pow
 from sympy.core.symbol import Symbol, Wild
 from sympy.core.relational import Equality
-from sympy.core.function import Derivative, diff
 from sympy.core.numbers import ilcm
 
 from sympy.functions import sqrt, log, exp, LambertW
 from sympy.simplify import simplify, collect
 from sympy.matrices import Matrix, zeros
 from sympy.polys import roots
+from sympy.functions.elementary.piecewise import Piecewise, piecewise_fold
 
 from sympy.utilities import any, all
+from sympy.utilities.iterables import iff
 from sympy.utilities.lambdify import lambdify
-from sympy.solvers.numeric import newton
+from sympy.mpmath import findroot
 
 from sympy.solvers.polysys import solve_poly_system
 
+from warnings import warn
+
 # Codes for guess solve strategy
 GS_POLY = 0
 GS_RATIONAL = 1
-GS_POLY_CV_1 = 2 # can be converted to a polynomial equation via the change of variable y -> x**n
+GS_POLY_CV_1 = 2 # can be converted to a polynomial equation via the change of variable y -> x**a, a real
 GS_POLY_CV_2 = 3 # can be converted to a polynomial equation multiplying on both sides by x**m
                  # for example, x + 1/x == 0. Multiplying by x yields x**2 + x == 0
 GS_RATIONAL_CV_1 = 4 # can be converted to a rational equation via the change of variable y -> x**n
-GS_TRASCENDENTAL = 5
+GS_PIECEWISE = 5
+GS_TRANSCENDENTAL = 6
 
 def guess_solve_strategy(expr, symbol):
     """
@@ -54,31 +59,17 @@ def guess_solve_strategy(expr, symbol):
     Examples
     ========
     >>> from sympy import Symbol, Rational
-    >>> x = Symbol('x')
+    >>> from sympy.solvers.solvers import guess_solve_strategy
+    >>> from sympy.abc import x
     >>> guess_solve_strategy(x**2 + 1, x)
     0
     >>> guess_solve_strategy(x**Rational(1,2) + 1, x)
     2
+
     """
     eq_type = -1
     if expr.is_Add:
-        items = expr.args
-        for item in items:
-            if item.is_Number or item.is_Symbol:
-                eq_type = max(eq_type, GS_POLY)
-            elif item.is_Mul:
-                for arg in item.args:
-                    eq_type = max(guess_solve_strategy(arg, symbol), eq_type)
-            elif item.is_Pow and item.base.has(symbol):
-                if item.exp.is_Integer:
-                    if item.exp > 0:
-                        eq_type = max(eq_type, GS_POLY)
-                    else:
-                        eq_type = max(eq_type, GS_POLY_CV_2)
-                elif item.exp.is_Rational:
-                    eq_type = max(eq_type, GS_POLY_CV_1)
-            elif item.is_Function:
-                return GS_TRASCENDENTAL
+        return max([guess_solve_strategy(i, symbol) for i in expr.args])
 
     elif expr.is_Mul:
         # check for rational functions
@@ -93,22 +84,29 @@ def guess_solve_strategy(expr, symbol):
             else:
                 raise NotImplementedError
         else:
-            return max(map(guess_solve_strategy, expr.args, [symbol]*len(expr.args)))
+            return max([guess_solve_strategy(i, symbol) for i in expr.args])
 
     elif expr.is_Symbol:
         return GS_POLY
 
     elif expr.is_Pow:
         if expr.exp.has(symbol):
-            return GS_TRASCENDENTAL
-        elif expr.exp.is_Number and expr.base.has(symbol):
-            if expr.exp.is_Integer:
+            return GS_TRANSCENDENTAL
+        elif not expr.exp.has(symbol) and expr.base.has(symbol):
+            if expr.exp.is_Integer and expr.exp > 0:
                 eq_type = max(eq_type, GS_POLY)
-            else:
+            elif expr.exp.is_Integer and expr.exp < 0:
+                eq_type = max(eq_type, GS_POLY_CV_2)
+            elif expr.exp.is_Rational:
                 eq_type = max(eq_type, GS_POLY_CV_1)
+            else:
+                return GS_TRANSCENDENTAL
+
+    elif expr.is_Piecewise:
+        return GS_PIECEWISE
 
     elif expr.is_Function and expr.has(symbol):
-        return GS_TRASCENDENTAL
+        return GS_TRANSCENDENTAL
 
     elif not expr.has(symbol):
         return GS_POLY
@@ -118,23 +116,22 @@ def guess_solve_strategy(expr, symbol):
 def solve(f, *symbols, **flags):
     """Solves equations and systems of equations.
 
-       Currently supported are univariate polynomial and transcendental
-       equations and systems of linear and polynomial equations.  Input
-       is formed as a single expression or an equation,  or an iterable
-       container in case of an equation system.  The type of output may
-       vary and depends heavily on the input. For more details refer to
-       more problem specific functions.
+       Currently supported are univariate polynomial, transcendental
+       equations, piecewise combinations thereof and systems of linear
+       and polynomial equations.  Input is formed as a single expression
+       or an equation,  or an iterable container in case of an equation
+       system.  The type of output may vary and depends heavily on the
+       input. For more details refer to more problem specific functions.
 
        By default all solutions are simplified to make the output more
-       readable. If this is not the expected behavior,  eg. because of
-       speed issues, set simplified=False in function arguments.
+       readable. If this is not the expected behavior (e.g., because of
+       speed issues) set simplified=False in function arguments.
 
-       To solve equations and systems of equations of other kind, eg.
-       recurrence relations of differential equations use rsolve() or
-       dsolve() functions respectively.
+       To solve equations and systems of equations like recurrence relations
+       or differential equations, use rsolve() or dsolve(), respectively.
 
-       >>> from sympy import *
-       >>> x,y = symbols('xy')
+       >>> from sympy import I, solve
+       >>> from sympy.abc import x, y
 
        Solve a polynomial equation:
 
@@ -147,40 +144,104 @@ def solve(f, *symbols, **flags):
        {x: -3, y: 1}
 
     """
-    if not symbols:
-        raise ValueError('no symbols were given')
+    def sympit(w):
+        return map(sympify, iff(isinstance(w,(list, tuple, set)), w, [w]))
+    # make f and symbols into lists of sympified quantities
+    # keeping track of how f was passed since if it is a list
+    # a dictionary of results will be returned.
+    bare_f = not isinstance(f, (list, tuple, set))
+    f, symbols = (sympit(w) for w in [f, symbols])
 
+    for i, fi in enumerate(f):
+        if isinstance(fi, Equality):
+            f[i] = fi.lhs - fi.rhs
+
+    if not symbols:
+        #get symbols from equations or supply dummy symbols since
+        #solve(3,x) returns []...though it seems that it should raise some sort of error TODO
+        symbols = set([])
+        for fi in f:
+            symbols |= fi.atoms(Symbol) or set([Symbol('x',dummy=True)])
+        symbols = list(symbols)
+
+    if bare_f:
+        f=f[0]
     if len(symbols) == 1:
         if isinstance(symbols[0], (list, tuple, set)):
             symbols = symbols[0]
 
-    symbols = map(sympify, symbols)
+    result = list()
 
-    if any(not s.is_Symbol for s in symbols):
-        raise TypeError('not a Symbol')
+    # Begin code handling for Function and Derivative instances
+    # Basic idea:  store all the passed symbols in symbols_passed, check to see
+    # if any of them are Function or Derivative types, if so, use a dummy
+    # symbol in their place, and set symbol_swapped = True so that other parts
+    # of the code can be aware of the swap.  Once all swapping is done, the
+    # continue on with regular solving as usual, and swap back at the end of
+    # the routine, so that whatever was passed in symbols is what is returned.
+    symbols_new = []
+    symbol_swapped = False
+
+    symbols_passed = list(symbols)
+
+    for i, s in enumerate(symbols):
+        if s.is_Symbol:
+            s_new = s
+        elif s.is_Function:
+            symbol_swapped = True
+            s_new = Symbol('F%d' % i, dummy=True)
+        elif s.is_Derivative:
+            symbol_swapped = True
+            s_new = Symbol('D%d' % i, dummy=True)
+        else:
+            raise TypeError('not a Symbol or a Function')
+        symbols_new.append(s_new)
+
+        if symbol_swapped:
+            swap_back_dict = dict(zip(symbols_new, symbols))
+    # End code for handling of Function and Derivative instances
 
     if not isinstance(f, (tuple, list, set)):
-        f = sympify(f)
 
-        if isinstance(f, Equality):
-            f = f.lhs - f.rhs
+        # Create a swap dictionary for storing the passed symbols to be solved
+        # for, so that they may be swapped back.
+        if symbol_swapped:
+            swap_dict = zip(symbols, symbols_new)
+            f = f.subs(swap_dict)
+            symbols = symbols_new
+
+        # Any embedded piecewise functions need to be brought out to the
+        # top level so that the appropriate strategy gets selected.
+        f = piecewise_fold(f)
 
         if len(symbols) != 1:
-            raise NotImplementedError('multivariate equation')
+            result = {}
+            for s in symbols:
+                result[s] = solve(f, s, **flags)
+            if flags.get('simplified', True):
+                for s, r in result.items():
+                    result[s] = map(simplify, r)
+            return result
 
         symbol = symbols[0]
-
         strategy = guess_solve_strategy(f, symbol)
 
         if strategy == GS_POLY:
             poly = f.as_poly( symbol )
-            assert poly is not None
+            if poly is None:
+                raise NotImplementedError("Cannot solve equation " + str(f) + " for "
+                    + str(symbol))
+            # for cubics and quartics, if the flag wasn't set, DON'T do it
+            # by default since the results are quite long. Perhaps one could
+            # base this decision on a certain crtical length of the roots.
+            if poly.degree > 2:
+                flags['simplified'] = flags.get('simplified', False)
             result = roots(poly, cubics=True, quartics=True).keys()
 
         elif strategy == GS_RATIONAL:
             P, Q = f.as_numer_denom()
             #TODO: check for Q != 0
-            return solve(P, symbol, **flags)
+            result = solve(P, symbol, **flags)
 
         elif strategy == GS_POLY_CV_1:
             args = list(f.args)
@@ -200,20 +261,18 @@ def solve(f, *symbols, **flags):
                     m = exponents_denom[0]
                 else:
                     # get the GCD of the denominators
-                    m = ilcm(*exponents_denom)
+                    m = reduce(ilcm, exponents_denom)
                 # x -> y**m.
                 # we assume positive for simplification purposes
                 t = Symbol('t', positive=True, dummy=True)
                 f_ = f.subs(symbol, t**m)
                 if guess_solve_strategy(f_, t) != GS_POLY:
-                    raise TypeError("Could not convert to a polynomial equation: %s" % f_)
+                    raise NotImplementedError("Could not convert to a polynomial equation: %s" % f_)
                 cv_sols = solve(f_, t)
-                result = list()
                 for sol in cv_sols:
                     result.append(sol**m)
 
             elif isinstance(f, Mul):
-                result = []
                 for mul_arg in args:
                     result.extend(solve(mul_arg, symbol))
 
@@ -237,18 +296,53 @@ def solve(f, *symbols, **flags):
             # TODO: we might have introduced unwanted solutions
             # when multiplied by x**-m
 
-        elif strategy == GS_TRASCENDENTAL:
+        elif strategy == GS_PIECEWISE:
+            result = set()
+            for expr, cond in f.args:
+                candidates = solve(expr, *symbols)
+                if isinstance(cond, bool) or cond.is_Number:
+                    if not cond:
+                        continue
+
+                    # Only include solutions that do not match the condition
+                    # of any of the other pieces.
+                    for candidate in candidates:
+                        matches_other_piece = False
+                        for other_expr, other_cond in f.args:
+                            if isinstance(other_cond, bool) \
+                               or other_cond.is_Number:
+                                continue
+                            if bool(other_cond.subs(symbol, candidate)):
+                                matches_other_piece = True
+                                break
+                        if not matches_other_piece:
+                            result.add(candidate)
+                else:
+                    for candidate in candidates:
+                        if bool(cond.subs(symbol, candidate)):
+                            result.add(candidate)
+
+            result = list(result)
+
+        elif strategy == GS_TRANSCENDENTAL:
             #a, b = f.as_numer_denom()
             # Let's throw away the denominator for now. When we have robust
             # assumptions, it should be checked, that for the solution,
             # b!=0.
             result = tsolve(f, *symbols)
         elif strategy == -1:
-            raise Exception('Could not parse expression %s' % f)
+            raise ValueError('Could not parse expression %s' % f)
         else:
-            raise NotImplementedError("No algorithms where implemented to solve equation %s" % f)
+            raise NotImplementedError("No algorithms are implemented to solve equation %s" % f)
 
-        if flags.get('simplified', True):
+        # This symbol swap should not be necessary for the single symbol case: if you've
+        # solved for the symbol the it will not appear in the solution. Right now, however
+        # ode's are getting solutions for solve (even though they shouldn't be -- see the
+        # swap_back test in test_solvers).
+        if symbol_swapped:
+            result = [ri.subs(swap_back_dict) for ri in result]
+
+        if flags.get('simplified', True) and strategy != GS_RATIONAL:
             return map(simplify, result)
         else:
             return result
@@ -256,13 +350,16 @@ def solve(f, *symbols, **flags):
         if not f:
             return {}
         else:
+            # Create a swap dictionary for storing the passed symbols to be
+            # solved for, so that they may be swapped back.
+            if symbol_swapped:
+                swap_dict = zip(symbols, symbols_new)
+                f = [fi.subs(swap_dict) for fi in f]
+                symbols = symbols_new
+
             polys = []
 
             for g in f:
-                g = sympify(g)
-
-                if isinstance(g, Equality):
-                    g = g.lhs - g.rhs
 
                 poly = g.as_poly(*symbols)
 
@@ -283,23 +380,36 @@ def solve(f, *symbols, **flags):
                         except ValueError:
                             matrix[i, m] = -coeff
 
-                return solve_linear_system(matrix, *symbols, **flags)
+                soln = solve_linear_system(matrix, *symbols, **flags)
             else:
-                return solve_poly_system(polys)
+                soln = solve_poly_system(polys)
+
+            # Use swap_dict to ensure we return the same type as what was
+            # passed
+            if symbol_swapped:
+                if isinstance(soln, dict):
+                    res = {}
+                    for k in soln.keys():
+                        res.update({swap_back_dict[k]: soln[k]})
+                    return res
+                else:
+                    return soln
+            else:
+                return soln
 
 def solve_linear_system(system, *symbols, **flags):
     """Solve system of N linear equations with M variables, which means
        both Cramer and over defined systems are supported. The possible
        number of solutions is zero, one or infinite. Respectively this
        procedure will return None or dictionary with solutions. In the
-       case of over definend system all arbitrary parameters are skiped.
+       case of over defined system all arbitrary parameters are skipped.
        This may cause situation in with empty dictionary is returned.
-       In this case it means all symbols can be assigne arbitray values.
+       In this case it means all symbols can be assigned arbitrary values.
 
        Input to this functions is a Nx(M+1) matrix, which means it has
        to be in augmented form. If you are unhappy with such setting
        use 'solve' method instead, where you can input equations
-       explicitely. And don't worry aboute the matrix, this function
+       explicitly. And don't worry about the matrix, this function
        is persistent and will make a local copy of it.
 
        The algorithm used here is fraction free Gaussian elimination,
@@ -307,8 +417,8 @@ def solve_linear_system(system, *symbols, **flags):
        Then solutions are found using back-substitution. This approach
        is more efficient and compact than the Gauss-Jordan method.
 
-       >>> from sympy import *
-       >>> x, y = symbols('xy')
+       >>> from sympy import Matrix, solve_linear_system
+       >>> from sympy.abc import x, y
 
        Solve the following system:
 
@@ -325,7 +435,7 @@ def solve_linear_system(system, *symbols, **flags):
 
     i, m = 0, matrix.cols-1  # don't count augmentation
 
-    while i < matrix.lines:
+    while i < matrix.rows:
         if i == m:
             # an overdetermined system
             if any(matrix[i:,m]):
@@ -337,7 +447,7 @@ def solve_linear_system(system, *symbols, **flags):
 
         if not matrix[i, i]:
             # there is no pivot in current column
-            # so try to find one in other colums
+            # so try to find one in other columns
             for k in xrange(i+1, m):
                 if matrix[i, k]:
                     break
@@ -360,7 +470,7 @@ def solve_linear_system(system, *symbols, **flags):
         # divide all elements in the current row by the pivot
         matrix.row(i, lambda x, _: x * pivot_inv)
 
-        for k in xrange(i+1, matrix.lines):
+        for k in xrange(i+1, matrix.rows):
             if matrix[k, i]:
                 coeff = matrix[k, i]
 
@@ -370,13 +480,13 @@ def solve_linear_system(system, *symbols, **flags):
 
         i += 1
 
-    # if there weren't any problmes, augmented matrix is now
+    # if there weren't any problems, augmented matrix is now
     # in row-echelon form so we can check how many solutions
     # there are and extract them using back substitution
 
     simplified = flags.get('simplified', True)
 
-    if len(syms) == matrix.lines:
+    if len(syms) == matrix.rows:
         # this system is Cramer equivalent so there is
         # exactly one solution to this system of equations
         k, solutions = i-1, {}
@@ -396,7 +506,7 @@ def solve_linear_system(system, *symbols, **flags):
             k -= 1
 
         return solutions
-    elif len(syms) > matrix.lines:
+    elif len(syms) > matrix.rows:
         # this system will have infinite number of solutions
         # dependent on exactly len(syms) - i parameters
         k, solutions = i-1, {}
@@ -427,14 +537,15 @@ def solve_undetermined_coeffs(equ, coeffs, sym, **flags):
     """Solve equation of a type p(x; a_1, ..., a_k) == q(x) where both
        p, q are univariate polynomials and f depends on k parameters.
        The result of this functions is a dictionary with symbolic
-       values of those parameters with respect to coefficiens in q.
+       values of those parameters with respect to coefficients in q.
 
        This functions accepts both Equations class instances and ordinary
        SymPy expressions. Specification of parameters and variable is
        obligatory for efficiency and simplicity reason.
 
-       >>> from sympy import *
-       >>> a, b, c, x = symbols('a', 'b', 'c', 'x')
+       >>> from sympy import Eq
+       >>> from sympy.abc import a, b, c, x
+       >>> from sympy.solvers import solve_undetermined_coeffs
 
        >>> solve_undetermined_coeffs(Eq(2*a*x + a+b, x), [a, b], x)
        {a: 1/2, b: -1/2}
@@ -453,218 +564,29 @@ def solve_undetermined_coeffs(equ, coeffs, sym, **flags):
     if not any([ equ.has(sym) for equ in system ]):
         # consecutive powers in the input expressions have
         # been successfully collected, so solve remaining
-        # system using Gaussian ellimination algorithm
+        # system using Gaussian elimination algorithm
         return solve(system, *coeffs, **flags)
     else:
         return None # no solutions
 
 def solve_linear_system_LU(matrix, syms):
     """ LU function works for invertible only """
-    assert matrix.lines == matrix.cols-1
-    A = matrix[:matrix.lines,:matrix.lines]
+    assert matrix.rows == matrix.cols-1
+    A = matrix[:matrix.rows,:matrix.rows]
     b = matrix[:,matrix.cols-1:]
     soln = A.LUsolve(b)
     solutions = {}
-    for i in range(soln.lines):
+    for i in range(soln.rows):
         solutions[syms[i]] = soln[i,0]
     return solutions
 
-def dsolve(eq, funcs):
-    """
-    Solves any (supported) kind of differential equation.
-
-    Usage
-
-        dsolve(f, y(x)) -> Solve a differential equation f for the function y
-
-
-    Details
-
-        @param f: ordinary differential equation (either just the left hand
-            side, or the Equality class)
-
-        @param y: indeterminate function of one variable
-
-        - you can declare the derivative of an unknown function this way:
-        >>> from sympy import *
-        >>> x = Symbol('x') # x is the independent variable
-
-        >>> f = Function("f")(x) # f is a function of x
-        >>> f_ = Derivative(f, x) # f_ will be the derivative of f with respect to x
-
-        - This function just parses the equation "eq" and determines the type of
-        differential equation by its order, then it determines all the coefficients and then
-        calls the particular solver, which just accepts the coefficients.
-        - "eq" can be either an Equality, or just the left hand side (in which
-          case the right hand side is assumed to be 0)
-        - see test_ode.py for many tests, that serve also as a set of examples
-          how to use dsolve
-
-    Examples
-
-        >>> from sympy import *
-        >>> x = Symbol('x')
-
-        >>> f = Function('f')
-        >>> dsolve(Derivative(f(x),x,x)+9*f(x), f(x))
-        C1*sin(3*x) + C2*cos(3*x)
-        >>> dsolve(Eq(Derivative(f(x),x,x)+9*f(x)+1, 1), f(x))
-        C1*sin(3*x) + C2*cos(3*x)
-
-    """
-
-    if isinstance(eq, Equality):
-        if eq.rhs != 0:
-            return dsolve(eq.lhs-eq.rhs, funcs)
-        eq = eq.lhs
-
-    #currently only solve for one function
-    if isinstance(funcs, Basic) or len(funcs) == 1:
-        if isinstance(funcs, (list, tuple)): # normalize args
-            f = funcs[0]
-        else:
-            f = funcs
-
-        x = f.args[0]
-        f = f.func
-
-        #We first get the order of the equation, so that we can choose the
-        #corresponding methods. Currently, only first and second
-        #order odes can be handled.
-        order = deriv_degree(eq, f(x))
-
-        if  order > 2 :
-           raise NotImplementedError("dsolve: Cannot solve " + str(eq))
-        elif order == 2:
-            return solve_ODE_second_order(eq, f(x))
-        elif order == 1:
-            return solve_ODE_first_order(eq, f(x))
-        else:
-            raise NotImplementedError("Not a differential equation!")
-
-def deriv_degree(expr, func):
-    """ get the order of a given ode, the function is implemented
-    recursively """
-    a = Wild('a', exclude=[func])
-
-    order = 0
-    if isinstance(expr, Derivative):
-        order = len(expr.symbols)
-    else:
-        for arg in expr.args:
-            if isinstance(arg, Derivative):
-                order = max(order, len(arg.symbols))
-            elif expr.match(a):
-                order = 0
-            else :
-                for arg1 in arg.args:
-                    order = max(order, deriv_degree(arg1, func))
-
-    return order
-
-def solve_ODE_first_order(eq, f):
-    """
-    solves many kinds of first order odes, different methods are used
-    depending on the form of the given equation. Now the linear
-    and Bernoulli cases are implemented.
-    """
-    from sympy.integrals.integrals import integrate
-    x = f.args[0]
-    f = f.func
-
-    #linear case: a(x)*f'(x)+b(x)*f(x)+c(x) = 0
-    a = Wild('a', exclude=[f(x)])
-    b = Wild('b', exclude=[f(x)])
-    c = Wild('c', exclude=[f(x)])
-
-    r = eq.match(a*diff(f(x),x) + b*f(x) + c)
-    if r:
-        t = C.exp(integrate(r[b]/r[a], x))
-        tt = integrate(t*(-r[c]/r[a]), x)
-        return (tt + Symbol("C1"))/t
-
-    #Bernoulli case: a(x)*f'(x)+b(x)*f(x)+c(x)*f(x)^n = 0
-    n = Wild('n', exclude=[f(x)])
-
-    r = eq.match(a*diff(f(x),x) + b*f(x) + c*f(x)**n)
-    if r:
-        t = C.exp((1-r[n])*integrate(r[b]/r[a],x))
-        tt = (r[n]-1)*integrate(t*r[c]/r[a],x)
-        return ((tt + Symbol("C1"))/t)**(1/(1-r[n]))
-
-    #other cases of first order odes will be implemented here
-
-    raise NotImplementedError("solve_ODE_first_order: Cannot solve " + str(eq))
-
-def solve_ODE_second_order(eq, f):
-    """
-    solves many kinds of second order odes, different methods are used
-    depending on the form of the given equation. So far the constants
-    coefficients case and a special case are implemented.
-    """
-    x = f.args[0]
-    f = f.func
-
-    #constant coefficients case: af''(x)+bf'(x)+cf(x)=0
-    a = Wild('a', exclude=[x])
-    b = Wild('b', exclude=[x])
-    c = Wild('c', exclude=[x])
-
-    r = eq.match(a*f(x).diff(x,x) + c*f(x))
-    if r:
-        return Symbol("C1")*C.sin(sqrt(r[c]/r[a])*x)+Symbol("C2")*C.cos(sqrt(r[c]/r[a])*x)
-
-    r = eq.match(a*f(x).diff(x,x) + b*diff(f(x),x) + c*f(x))
-    if r:
-        r1 = solve(r[a]*x**2 + r[b]*x + r[c], x)
-        if r1[0].is_real:
-            if len(r1) == 1:
-                return (Symbol("C1") + Symbol("C2")*x)*exp(r1[0]*x)
-            else:
-                return Symbol("C1")*exp(r1[0]*x) + Symbol("C2")*exp(r1[1]*x)
-        else:
-            r2 = abs((r1[0] - r1[1])/(2*S.ImaginaryUnit))
-            return (Symbol("C2")*C.cos(r2*x) + Symbol("C1")*C.sin(r2*x))*exp((r1[0] + r1[1])*x/2)
-
-    #other cases of the second order odes will be implemented here
-
-    #special equations, that we know how to solve
-    a = Wild('a')
-    t = x*C.exp(f(x))
-    tt = a*t.diff(x, x)/t
-    r = eq.match(tt.expand())
-    if r:
-        return -solve_ODE_1(f(x), x)
-
-    t = x*C.exp(-f(x))
-    tt = a*t.diff(x, x)/t
-    r = eq.match(tt.expand())
-    if r:
-        #check, that we've rewritten the equation correctly:
-        #assert ( r[a]*t.diff(x,2)/t ) == eq.subs(f, t)
-        return solve_ODE_1(f(x), x)
-
-    neq = eq*C.exp(f(x))/C.exp(-f(x))
-    r = neq.match(tt.expand())
-    if r:
-        #check, that we've rewritten the equation correctly:
-        #assert ( t.diff(x,2)*r[a]/t ).expand() == eq
-        return solve_ODE_1(f(x), x)
-
-    raise NotImplementedError("solve_ODE_second_order: cannot solve " + str(eq))
-
-def solve_ODE_1(f, x):
-    """ (x*exp(-f(x)))'' = 0 """
-    C1 = Symbol("C1")
-    C2 = Symbol("C2")
-    return -C.log(C1+C2/x)
-
 x = Symbol('x', dummy=True)
 a,b,c,d,e,f,g,h = [Wild(t, exclude=[x]) for t in 'abcdefgh']
 patterns = None
 
 def _generate_patterns():
-    """Generates patterns for transcendental equations.
+    """
+    Generates patterns for transcendental equations.
 
     This is lazily calculated (called) in the tsolve() function and stored in
     the patterns global variable.
@@ -694,8 +616,8 @@ def tsolve(eq, sym):
     not unique. In some cases, a complex solution may be returned
     even though a real solution exists.
 
-        >>> from sympy import *
-        >>> x = Symbol('x')
+        >>> from sympy import tsolve, log
+        >>> from sympy.abc import x
 
         >>> tsolve(3**(2*x+5)-4, x)
         [(-5*log(3) + log(4))/(2*log(3))]
@@ -713,11 +635,12 @@ def tsolve(eq, sym):
     eq2 = eq.subs(sym, x)
     # First see if the equation has a linear factor
     # In that case, the other factor can contain x in any way (as long as it
-    # is finite), and we have a direct solution
+    # is finite), and we have a direct solution to which we add others that
+    # may be found for the remaining portion.
     r = Wild('r')
     m = eq2.match((a*x+b)*r)
     if m and m[a]:
-        return [(-b/a).subs(m).subs(x, sym)]
+        return [(-b/a).subs(m).subs(x, sym)] + solve(m[r], x)
     for p, sol in patterns:
         m = eq2.match(p)
         if m:
@@ -770,7 +693,8 @@ def tsolve(eq, sym):
             if f1.is_Function:
                 break
         else:
-            assert False, 'tsolve: at least one Function expected at this point'
+            raise NotImplementedError("Unable to solve the equation" + \
+                "(tsolve: at least one Function expected at this point")
 
         # perform the substitution
         lhs_ = lhs.subs(f1, t)
@@ -791,57 +715,106 @@ def tsolve(eq, sym):
 
     raise NotImplementedError("Unable to solve the equation.")
 
-
-def msolve(args, f, x0, tol=None, maxsteps=None, verbose=False, norm=None,
-           modules=['mpmath', 'sympy']):
+def msolve(*args, **kwargs):
     """
-    Solves a nonlinear equation system numerically.
+    Compatibility wrapper pointing to nsolve().
+
+    msolve() has been renamed to nsolve(), please use nsolve() directly."""
+    warn('msolve() is has been renamed, please use nsolve() instead',
+         DeprecationWarning)
+    args[0], args[1] = args[1], args[0]
+    return nsolve(*args, **kwargs)
+
+# TODO: option for calculating J numerically
+def nsolve(*args, **kwargs):
+    """
+    Solve a nonlinear equation system numerically.
+
+    nsolve(f, [args,] x0, modules=['mpmath'], **kwargs)
 
     f is a vector function of symbolic expressions representing the system.
-    args are the variables.
+    args are the variables. If there is only one variable, this argument can be
+    omitted.
     x0 is a starting vector close to a solution.
 
-    Be careful with x0, not using floats might give unexpected results.
+    Use the modules keyword to specify which modules should be used to evaluate
+    the function and the Jacobian matrix. Make sure to use a module that
+    supports matrices. For more information on the syntax, please see the
+    docstring of lambdify.
 
-    Use modules to specify which modules should be used to evaluate the
-    function and the Jacobian matrix. Make sure to use a module that supports
-    matrices. For more information on the syntax, please see the docstring
-    of lambdify.
+    Overdetermined systems are supported.
 
-    Currently only fully determined systems are supported.
-
-    >>> from sympy import Symbol, Matrix
+    >>> from sympy import Symbol, nsolve
+    >>> import sympy
+    >>> sympy.mpmath.mp.dps = 15
     >>> x1 = Symbol('x1')
     >>> x2 = Symbol('x2')
     >>> f1 = 3 * x1**2 - 2 * x2**2 - 1
     >>> f2 = x1**2 - 2 * x1 + x2**2 + 2 * x2 - 8
-    >>> msolve((x1, x2), (f1, f2), (-1., 1.))
+    >>> print nsolve((f1, f2), (x1, x2), (-1, 1))
     [-1.19287309935246]
     [ 1.27844411169911]
+
+    For one-dimensional functions the syntax is simplified:
+
+    >>> from sympy import sin, nsolve
+    >>> from sympy.abc import x
+    >>> nsolve(sin(x), x, 2)
+    3.14159265358979
+    >>> nsolve(sin(x), 2)
+    3.14159265358979
+
+    mpmath.findroot is used, you can find there more extensive documentation,
+    especially concerning keyword parameters and available solvers.
     """
+    # interpret arguments
+    if len(args) == 3:
+        f = args[0]
+        fargs = args[1]
+        x0 = args[2]
+    elif len(args) == 2:
+        f = args[0]
+        fargs = None
+        x0 = args[1]
+    elif len(args) < 2:
+        raise TypeError('nsolve expected at least 2 arguments, got %i'
+                        % len(args))
+    else:
+        raise TypeError('nsolve expected at most 3 arguments, got %i'
+                        % len(args))
+    modules = kwargs.get('modules', ['mpmath'])
     if isinstance(f,  (list,  tuple)):
         f = Matrix(f).T
-    if len(args) != f.cols:
-        raise NotImplementedError('need exactly as many variables as equations')
+    if not isinstance(f, Matrix):
+        # assume it's a sympy expression
+        if isinstance(f, Equality):
+            f = f.lhs - f.rhs
+        f = f.evalf()
+        atoms = f.atoms(Symbol)
+        if fargs is None:
+            fargs = atoms.copy().pop()
+        if not (len(atoms) == 1 and (fargs in atoms or fargs[0] in atoms)):
+            raise ValueError('expected a one-dimensional and numerical function')
+
+        # the function is much better behaved if there is no denominator
+        f = f.as_numer_denom()[0]
+
+        f = lambdify(fargs, f, modules)
+        return findroot(f, x0, **kwargs)
+    if len(fargs) > f.cols:
+        raise NotImplementedError('need at least as many equations as variables')
+    verbose = kwargs.get('verbose', False)
     if verbose:
         print 'f(x):'
         print f
     # derive Jacobian
-    J = f.jacobian(args)
+    J = f.jacobian(fargs)
     if verbose:
         print 'J(x):'
         print J
     # create functions
-    f = lambdify(args, f.T, modules)
-    J = lambdify(args, J, modules)
-    # solve system using Newton's method
-    kwargs = {}
-    if tol:
-        kwargs['tol'] = tol
-    if maxsteps:
-        kwargs['maxsteps'] = maxsteps
-    kwargs['verbose'] = verbose
-    if norm:
-        kwargs['norm'] = norm
-    x = newton(f, x0, J, **kwargs)
+    f = lambdify(fargs, f.T, modules)
+    J = lambdify(fargs, J, modules)
+    # solve the system numerically
+    x = findroot(f, x0, J=J, **kwargs)
     return x
