@@ -1,8 +1,5 @@
-
 from sympy import S, symbols, Integer, Rational, Poly, \
-    PolynomialError, sqrt, I, solve_poly_system
-
-import py
+    PolynomialError, sqrt, I, solve_poly_system, raises
 
 x, y, z = symbols('xyz')
 
@@ -27,4 +24,4 @@ def test_solve_poly_system():
     assert solve_poly_system([x**2+y+z-1, x+y**2+z-1, x+y+z**2-1], x, y, z) == \
         [(a, a, a), (0, 0, 1), (0, 1, 0), (b, b, b), (1, 0, 0)]
 
-    py.test.raises(PolynomialError, "solve_poly_system([x**3-y**3], x, y)")
+    raises(PolynomialError, "solve_poly_system([x**3-y**3], x, y)")
