@@ -13,7 +13,8 @@ except ImportError:
     disabled = True
 
 
-from sympy import Rational, Symbol, list2numpy, sin, Real, Matrix, lambdify
+from sympy import (Rational, Symbol, list2numpy, sin, Real, Matrix, lambdify,
+        symarray, symbols)
 import sympy
 
 from sympy import mpmath
@@ -241,3 +242,37 @@ def test_lambdify_transl():
     for sym, mat in NUMPY_TRANSLATIONS.iteritems():
         assert sym in sympy.functions.__dict__ or sym in ("Matrix", )
         assert mat in numpy.__dict__
+
+def test_symarray():
+    """Test creation of numpy arrays of sympy symbols."""
+
+    import numpy as np
+    import numpy.testing as npt
+
+    syms = symbols('_0 _1 _2')
+    s1 = symarray(3)
+    s2 = symarray(3)
+    npt.assert_array_equal (s1, np.array(syms, dtype=object))
+    assert s1[0] is s2[0]
+
+    a = symarray(3, 'a')
+    b = symarray(3, 'b')
+    assert not(a[0] is b[0])
+
+    asyms = symbols('a_0 a_1 a_2')
+    npt.assert_array_equal (a, np.array(asyms, dtype=object))
+
+    # Multidimensional checks
+    a2d = symarray((2,3), 'a')
+    assert a2d.shape == (2,3)
+    a00, a12 = symbols('a_0_0, a_1_2')
+    assert a2d[0,0] is a00
+    assert a2d[1,2] is a12
+
+    a3d = symarray((2,3,2), 'a')
+    assert a3d.shape == (2,3,2)
+    a000, a120, a121 = symbols('a_0_0_0, a_1_2_0 a_1_2_1')
+    assert a3d[0,0,0] is a000
+    assert a3d[1,2,0] is a120
+    assert a3d[1,2,1] is a121
+
