@@ -2,9 +2,9 @@
 """
 import sys
 
-from iterables import make_list, flatten, subsets
+from iterables import iff, make_list, flatten, subsets, numbered_symbols
 
-if sys.version_info[1] < 5:
+if sys.version_info[0] <= 2 and sys.version_info[1] < 5:
     from iterables import any, all
 else:
     any = any
@@ -13,9 +13,8 @@ else:
 from lambdify import lambdify
 from source import source
 
-from decorator import threaded
+from decorator import threaded, deprecated
 
 from runtests import test, doctest
 
 from pytest import raises
-
