@@ -35,16 +35,16 @@ Performance
 ===========
 
 Python functions using the math module are *quite* fast. For simple functions
-they are faster than functions compiled to machine code. So you should try
-whether lambdify is fast enough for you.
+they are faster than functions compiled to machine code. So you should test
+to see whether lambdify is fast enough for you.
 
 Iterating is slow in Python (it's probably the biggest bottle neck).
 frange allows you to iterate using machine code. This can result in huge
 speedups. You might want to use NumPy: http://numpy.org/
 For simple functions it's faster, but for big ones frange can be several times
-more efficent.
+more efficient.
 
-You should try which solution is the best for your application.
+You should experiment to see which solution is best for your application.
 
 You can run the included benchmarks to see the real performance on your machine.
 
@@ -57,7 +57,7 @@ You will probably need to compile libtcc on your own. Get the sources of tcc:
 http://bellard.org/tcc/
 
 Currently it only works for a recent development version. So you might want to
-run the following commands (you have to use your own pathes of course):
+run the following commands (you have to use your own paths of course):
 
 $ cvs -z3 -d:pserver:anonymous@cvs.savannah.nongnu.org:/sources/tinycc co tinycc
 $ cd tinycc
@@ -226,7 +226,7 @@ def __run(cmd):
     Checks the exit code of a ran command.
     """
     if not cmd == 0:
-        raise Exception('could not run libtcc command')
+        raise RuntimeError('could not run libtcc command')
 
 def _compile(code, argcount=None, fname='f', fprototype=None):
     """
@@ -264,10 +264,11 @@ def clambdify(args, expr, **kwargs):
     Supports all standard C math functions, pi and e.
 
     >>> from sympy import symbols, sqrt
-    >>> x, y = symbols('xy')
+    >>> from sympy.abc import x, y
     >>> cf = clambdify((x,y), sqrt(x*y))
     >>> cf(0.5, 4)
     1.4142135623730951
+
     """
     # convert function to lambda string
     s = getlambdastr(args, expr.evalf(21))
@@ -303,6 +304,7 @@ def frange(*args, **kwargs):
     1.0
     1.41421356237
     1.73205080757
+
     """
     if len(args) > 4:
         raise TypeError('expected at most 4 arguments, got %i' % len(args))
@@ -531,7 +533,7 @@ def benchmark():
             psyco.bind(psyf)
         except ImportError:
             pass
-        code = '''for x in (i/1000. for i in xrange(1000)):
+        code = '''for x in (i/1000. for i in range(1000)):
         f(%s)''' % ('x,'*len(var)).rstrip(',')
         t1 = Timer(code, 'from __main__ import cf as f')
         t2 = Timer(code, 'from __main__ import pf as f')
