@@ -4,7 +4,7 @@ lambda functions which can be used to calculate numerical values very fast.
 """
 
 from __future__ import division
-from sympy.core import sympify
+from sympy.core.sympify import sympify
 
 # These are the namespaces the lambda functions will use.
 MATH = {}
@@ -31,6 +31,8 @@ MPMATH_TRANSLATIONS = {
     "oo":"inf",
     #"uppergamma":"upper_gamma",
     "LambertW":"lambertw",
+    "Matrix":"matrix",
+    "conjugate":"conj",
 }
 
 NUMPY_TRANSLATIONS = {
@@ -72,7 +74,7 @@ def _import(module, reload="False"):
     These dictionaries map names of python functions to their equivalent in
     other modules.
     """
-    if not MODULES.has_key(module):
+    if not module in MODULES:
         raise NameError("This module can't be used for lambdification.")
     namespace, translations, import_commands = MODULES[module]
     # Clear namespace or exit
@@ -99,8 +101,9 @@ def lambdify(args, expr, modules=None):
     Returns a lambda function for fast calculation of numerical values.
 
     Usage:
-    >>> from sympy import symbols, sqrt, sin
-    >>> x,y,z = symbols('xyz')
+    >>> from sympy import sqrt, sin
+    >>> from sympy.utilities import lambdify
+    >>> from sympy.abc import x, y, z
     >>> f = lambdify(x, x**2)
     >>> f(2)
     4
@@ -117,7 +120,7 @@ def lambdify(args, expr, modules=None):
     If not specified differently by the user, Sympy functions are replaced as
     far as possible by either python-math, numpy (if available) or mpmath
     functions - exactly in this order.
-    To change this behaviour, the "modules" argument can be used.
+    To change this behavior, the "modules" argument can be used.
     It accepts:
      - the strings "math", "mpmath", "numpy", "sympy"
      - any modules (e.g. math)
@@ -138,7 +141,7 @@ def lambdify(args, expr, modules=None):
         >> import numpy
         >> f = lambdify((x,y), tan(x*y), numpy)
 
-        Attention: There are naming diferences between numpy and sympy. So if
+        Attention: There are naming differences between numpy and sympy. So if
                    you simply take the numpy module, e.g. sympy.atan will not be
                    translated to numpy.arctan. Use the modified module instead
                    by passing the string "numpy".
@@ -180,7 +183,7 @@ def lambdify(args, expr, modules=None):
 
 def _get_namespace(m):
     """
-    This is used by _lambdify to parse it's arguments.
+    This is used by _lambdify to parse its arguments.
     """
     if isinstance(m, str):
         _import(m)
@@ -196,12 +199,13 @@ def lambdastr(args, expr):
     """
     Returns a string that can be evaluated to a lambda function.
 
-    >>> from sympy import symbols
-    >>> x,y,z = symbols('xyz')
+    >>> from sympy.abc import x, y, z
+    >>> from sympy.utilities.lambdify import lambdastr
     >>> lambdastr(x, x**2)
     'lambda x: (x**2)'
     >>> lambdastr((x,y,z), [z,y,x])
     'lambda x,y,z: ([z, y, x])'
+
     """
 
     #XXX: This has to be done here because of circular imports
