@@ -18,7 +18,7 @@ from sympy.core.mul import Mul
 from sympy.core.power import Pow
 from sympy.core.function import Derivative, Function, FunctionClass, Lambda,\
         WildFunction
-from sympy.core.interval import Interval
+from sympy.core.sets import Interval
 from sympy.core.multidimensional import vectorize
 from sympy.core.cache import Memoizer
 #from sympy.core.ast_parser import SymPyParser, SymPyTransformer
@@ -26,7 +26,7 @@ from sympy.core.cache import Memoizer
 from sympy import symbols
 
 
-def check(a):
+def check(a, check_attr = True):
     """ Check that pickling and copying round-trips.
     """
     for protocol in [0, 1, 2, copy.copy, copy.deepcopy]:
@@ -42,6 +42,8 @@ def check(a):
         d2 = dir(b)
         assert d1==d2
 
+        if not check_attr:
+            continue
         def c(a,b,d):
             for i in d:
                 if not hasattr(a,i):
@@ -192,20 +194,16 @@ def test_geometry():
               Ellipse, Ellipse(p1,3,4), Line, Line(p1,p2), LinearEntity,
               LinearEntity(p1,p2), Ray, Ray(p1,p2), Segment, Segment(p1,p2),
               Polygon, Polygon(p1,p2,p3,p4), RegularPolygon, RegularPolygon(p1,4,5),
-              Triangle):
-        # XXX: Instance of Triangle hangs because of hasattr in check().
-        # Triangle(p1,p2,p3)
-        check(c)
+              Triangle, Triangle(p1,p2,p3)):
+        check(c, check_attr = False)
         pass
 
 #================== integrals ====================
 from sympy.integrals.integrals import Integral
-from sympy.integrals.trigonometry import Cos_2k_integrate, Sin_2k_integrate
 
 def test_integrals():
     x = Symbol("x")
-    for c in (Integral, Integral(x), Cos_2k_integrate, Cos_2k_integrate(x, 3),
-              Sin_2k_integrate, Sin_2k_integrate(x, 2)):
+    for c in (Integral, Integral(x)):
         check(c)
 
 #================== matrices ====================
